/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop;

import javassist.ClassPool;
import javassist.CtClass;
import javassist.LoaderClassPath;
import javassist.NotFoundException;

import java.util.Iterator;
import java.util.Map;

/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.23.2.5 $
 */
public class AOPClassPool extends ClassPool
{
   protected AspectManager manager;
   protected ClassLoader dcl;
   protected LoaderClassPath classPath;

   protected AOPClassPool(ClassLoader cl, ClassPool src, AspectManager manager)
   {
      super(src);
      this.manager = manager;
      this.dcl = cl;
      if (cl != null)
      {
         classPath = new LoaderClassPath(cl);
         this.insertClassPath(classPath);
      }
      childFirstLookup = true;
   }

   protected AOPClassPool(ClassPool src, AspectManager manager)
   {
      this(null, src, manager);
   }

   public void close()
   {
      this.removeClassPath(classPath);
      classPath.close();
      classes.clear();
   }

   public void flushClass(String classname)
   {
      classes.remove(classname);
   }

   protected CtClass getCached(String classname)
   {
      CtClass clazz = getCachedLocally(classname);
      if (clazz == null)
      {
         Map registeredCLs = manager.getRegisteredCLs();
         synchronized (registeredCLs)
         {
            Iterator it = registeredCLs.values().iterator();
            while (it.hasNext())
            {
               AOPClassPool pool = (AOPClassPool) it.next();
               clazz = pool.getCachedLocally(classname);
               if (clazz != null)
               {
                  return clazz;
               }
            }
         }
      }
      // *NOTE* NEED TO TEST WHEN SUPERCLASS IS IN ANOTHER UCL!!!!!!
      return clazz;
   }

   protected CtClass getCachedLocally(String classname)
   {
      return (CtClass) classes.get(classname);
   }

   public synchronized CtClass getLocally(String classname)
   throws NotFoundException
   {
      CtClass clazz = (CtClass) classes.get(classname);
      if (clazz == null)
      {
         clazz = createCtClass(classname, true);
         if (clazz == null) throw new NotFoundException(classname);
         classes.put(classname, clazz);
      }
      return clazz;
   }


   public static AOPClassPool createAOPClassPool(ClassLoader cl, ClassPool src, AspectManager manager)
   {
      return AspectManager.classPoolFactory.create(cl, src, manager);
   }

   public static AOPClassPool createAOPClassPool(ClassPool src, AspectManager manager)
   {
      return AspectManager.classPoolFactory.create(src, manager);
   }
}
