/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop;

import org.jboss.util.file.ArchiveBrowser;
import org.jboss.util.file.ClassFileFilter;

import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class Deployment
{
   public static boolean searchClasspath = true;

   public static void deploy()
   {
      try
      {

         deployThroughClassAnnotations();
         preconfigThroughClassPath();
         preconfigThroughSystemProperty();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }

   /**
    * This is for running standalone.  AspectManager will automatically search classpath for resources
    * of jboss-aop.xml and load them.  The AspectManagerService will turn this off if you are running
    * from within JBoss.
    */
   public static void preconfigThroughClassPath()
   {
      String search = System.getProperty("jboss.aop.search.classpath", null);
      if (search != null) searchClasspath = (new Boolean(search)).booleanValue();
      if (AspectManager.verbose) System.out.println("[debug] jboss.aop.search.classpath: '" + search + "' " + searchClasspath);
      if (searchClasspath)
      {
         try
         {
            Enumeration en = Thread.currentThread().getContextClassLoader().getResources("META-INF/jboss-aop.xml");
            while (en.hasMoreElements())
            {
               URL url = (URL) en.nextElement();
               if (AspectManager.verbose) System.out.println("[deploying] " + url);
               AspectXmlLoader.deployXML(url);
            }
         }
         catch (Exception ex)
         {
            throw new RuntimeException(ex);
         }
      }
   }

   /**
    * Load XML files through a system property value jboss.aop.path
    */
   public static void deployThroughClassAnnotations()
   {
      String path = System.getProperty("jboss.aop.class.path", null);
      if (path == null)
      {
         if (AspectManager.verbose) System.out.println("[debug] jboss.aop.class.path is NULL");
         return;
      }
      if (AspectManager.verbose) System.out.println("[debug] jboss.aop.class.path: " + path);
      StringTokenizer t = new StringTokenizer(path, File.pathSeparator);
      while (t.hasMoreTokens())
      {
         String token = t.nextToken();
         File f = new File(token);
         if (!f.exists())
         {
            System.err.println("[error] Unable to find jboss.aop.class.path: " + f.getName());
         }
         try
         {
            URL url = f.toURL();
            Iterator it = ArchiveBrowser.getBrowser(url, new ClassFileFilter());
            AspectAnnotationLoader loader = new AspectAnnotationLoader();
            loader.deployInputStreamIterator(it);
         }
         catch (Exception ex)
         {
            ex.printStackTrace();
            if (ex instanceof RuntimeException)
               throw (RuntimeException) ex;
            else
               throw new RuntimeException("[error] failed to load aop class path: " + f.toString(), ex);
         }
      }
   }

   /**
    * Load XML files through a system property value jboss.aop.path
    */
   public static void preconfigThroughSystemProperty()
   {
      String path = System.getProperty("jboss.aop.path", null);
      if (AspectManager.verbose) System.out.println("[debug] jboss.aop.path: " + path);
      if (path == null) return;
      StringTokenizer t = new StringTokenizer(path, File.pathSeparator);
      int j = 0;
      while (t.hasMoreTokens())
      {
         String token = t.nextToken();
         if (AspectManager.verbose) System.out.println("jboss.aop.path[" + j + "]: " + token);
         File f = new File(token);
         try
         {
            if (f.isDirectory())
            {
               FilenameFilter filter = new FilenameFilter()
               {
                  public boolean accept(File dir, String name)
                  {
                     return name.endsWith("aop.xml");
                  }
               };
               File[] files = f.listFiles(filter);
               for (int i = 0; i < files.length; i++)
               {
                  deployXmlFile(files[i]);
               }
            }
            else
            {
               deployXmlFile(f);
            }
         }
         catch (Exception ex)
         {
            ex.printStackTrace();
            if (ex instanceof RuntimeException)
               throw (RuntimeException) ex;
            else
               throw new RuntimeException("[error] failed to load aop path: " + f.toString(), ex);
         }
      }
   }

   private static void deployXmlFile(File f) throws Exception
   {
      URL url = f.toURL();
      if (AspectManager.verbose) System.out.println("[deploying] " + url);
      AspectXmlLoader.deployXML(url);
   }
}
