/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.InvocationResponse;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.proxy.ClassProxy;
import org.jboss.aop.proxy.ClassProxyFactory;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
/**
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.9 $
 */

public class Dispatcher
{
   public static final String DISPATCHER = "DISPATCHER";
   public static final String OID = "OID";
   public static final Dispatcher singleton = new Dispatcher();

   Object maplock = new Object();
   HashMap targetMap = new HashMap();
   

   public boolean isRegistered(Object oid)
   {
      synchronized (maplock)
      {
         return targetMap.containsKey(oid);
      }
   }
   /**
    *  Register an Object ID with an actual target object
    */
   public void registerTarget(Object oid, Object target)
   {
      synchronized (maplock)
      {
         targetMap.put(oid, target);
      }
   }

   public void unregisterTarget(Object oid)
   {
      synchronized (maplock)
      {
         targetMap.remove(oid);
      }
   }

   public Object getRegistered(Object oid)
   {
      synchronized (maplock)
      {
         return targetMap.get(oid);
      }
   }
   

   public InvocationResponse invoke(Invocation invocation) throws Throwable
   {
      Object oid = invocation.getMetaData(DISPATCHER, OID);

      Object target = null;
      synchronized (maplock)
      {
         target = targetMap.get(oid);
      }

      if (target instanceof ClassProxy)
      {
         ClassProxy proxy = (ClassProxy)target;
         return proxy._dynamicInvoke(invocation);
      }
      else if (target instanceof Advised)
      {
         ClassAdvisor advisor = (ClassAdvisor)((Advised)target)._getAdvisor();
         return advisor.dynamicInvoke(target, invocation);
      }
      else if (target instanceof ClassAdvisor) // Container
      {
         ClassAdvisor advisor = (ClassAdvisor)target;
         return advisor.dynamicInvoke(null, invocation);
      }
      else
      {
         if (invocation instanceof MethodInvocation)
         {
            MethodInvocation methodInvocation = (MethodInvocation)invocation;
            // For non-advised methods, we can only do public method invocations
            long methodHash = methodInvocation.getMethodHash();
            HashMap methodMap = ClassProxyFactory.getMethodMap(target.getClass());
            Method method = (Method)methodMap.get(new Long(methodHash));
            Object[] args = methodInvocation.getArguments();
            try
            {
               return new InvocationResponse(method.invoke(target, args));
            }
            catch (InvocationTargetException ex)
            {
               throw ex.getTargetException();
            }
         }
         else
         {
            throw new RuntimeException("field invocations not implemented");
         }
      }
   }
}
