/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.metadata.SimpleMetaData;

/**
 * Holds an object instance's metadata and attached interceptors
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.9 $
 */
public interface InstanceAdvisor
{
   public SimpleMetaData getMetaData();
   public Interceptor[] getInterceptors();
   public Interceptor[] getInterceptors(Interceptor[] baseChain);
   public boolean hasAspects();

   public void insertInterceptor(Interceptor interceptor);
   public void removeInterceptor(String name);
   public void appendInterceptor(Interceptor interceptor);

   public void insertInterceptorStack(String stackName);
   public void removeInterceptorStack(String name);
   public void appendInterceptorStack(String stackName);

   /**
    * Get the instance of an aspect.  An aspect encapsulates
    * a set of advices.
    * @param aspectName
    * @return
    */
   public Object getPerInstanceAspect(String aspectName);
}
