/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.joinpoint.Joinpoint;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.7 $
 *
 **/
public class AdviceFactory implements InterceptorFactory
{
   private String advice;
   private AspectDefinition aspect;

   public AdviceFactory(AspectDefinition aspect, String adviceName)
   {
      this.aspect = aspect;
      this.advice = adviceName;
   }

   public AspectDefinition getAspect()
   {
      return aspect;
   }

   public String getAdvice()
   {
      return advice;
   }

   public boolean isDeployed()
   {
      return aspect.isDeployed();
   }

   public Interceptor create(Advisor advisor, Joinpoint joinpoint)
   {
      if (aspect.getScope() == Scope.PER_VM)
      {
         try
         {
            return PerVmAdvice.generateOptimized(advice, aspect);
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else if (aspect.getScope() == Scope.PER_CLASS)
      {
         try
         {
            return PerClassAdvice.generate(advisor, advice, aspect);
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else if (aspect.getScope() == Scope.PER_INSTANCE)
      {
         return new PerInstanceAdvice(advice, aspect, advisor);
      }
      else if (aspect.getScope() == Scope.PER_JOINPOINT)
      {
         try
         {
            return PerJoinpointAdvice.createInterceptor(advisor, joinpoint, aspect, advice);
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else
      {
         //if (aspect.getScope() == null) System.err.println("scope is null: " + aspect.getName() + "." + advice);
      }
      return null;
   }

   public String getName()
   {
      return aspect.getName() + "." + advice;
   }
}
