/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.joinpoint.Joinpoint;

import java.util.ArrayList;

/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 */
public class AdviceStack
{
   protected String name;
   protected ArrayList interceptorFactories;

   public AdviceStack(String name, ArrayList factories)
   {
      this.name = name;
      interceptorFactories = factories;
   }

   public String getName() { return name; }

   public ArrayList getInterceptorFactories() { return interceptorFactories; }


   public Interceptor[] createInterceptors(Advisor advisor, Joinpoint jp)
   {
      ArrayList interceptors = new ArrayList();
      for (int i = 0; i < interceptorFactories.size(); i++)
      {
         InterceptorFactory factory = (InterceptorFactory) interceptorFactories.get(i);
         if (factory.isDeployed()) interceptors.add(factory.create(advisor, jp));
      }
      return (Interceptor[])interceptors.toArray(new Interceptor[interceptors.size()]);
   }

   public Interceptor[] createInterceptors()
   {
      return createInterceptors(null, null);
   }
}
