/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.AspectManager;

import java.util.HashSet;
import java.util.Iterator;

/**
 * Contains definition of aspect or interceptor
 * Scope defaults to PER_VM if it is null.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.7 $
 */
public class AspectDefinition
{
   protected String name;
   protected Scope scope;
   protected HashSet advisors = new HashSet();
   protected AspectFactory factory;
   protected boolean deployed = true;

   /**
    *
    * @param name
    * @param scope defaults to PER_VM if null
    * @param factory
    */
   public AspectDefinition(String name, Scope scope, AspectFactory factory)
   {
      this.name = name;
      this.scope = scope;
      this.factory = factory;
      if (this.scope == null) this.scope = Scope.PER_VM;
      if (this.name == null) this.name = this.factory.getName();
   }

   public void undeploy() { this.deployed = false; }
   public boolean isDeployed() { return deployed; }

   public AspectFactory getFactory()
   {
      return factory;
   }

   public String getName()
   {
      return name;
   }

   public Scope getScope()
   {
      return scope;
   }

   public int hashCode()
   {
      return name.hashCode();
   }

   public boolean equals(Object obj)
   {
      if (obj == this) return true;
      if (!(obj instanceof AspectDefinition)) return false;
      return name.equals(((AspectDefinition) obj).name);
   }

   public void addAdvisor(Advisor advisor)
   {
      // Don't hold a direct reference to an advisor because of undeploy and redeploy
      advisors.add(advisor.getName());
      if (scope == Scope.PER_CLASS)
         advisor.addPerClassAspect(this);
      else if (scope == Scope.PER_INSTANCE) advisor.addPerInstanceAspect(this);
   }

   public void clearAdvisors()
   {
      for (Iterator it = advisors.iterator(); it.hasNext();)
      {
         String advisorName = (String) it.next();
         Advisor advisor = AspectManager.instance().getAdvisor(advisorName);
         if (advisor != null)
         {
            if (scope == Scope.PER_CLASS)
               advisor.removePerClassAspect(this);
            else if (scope == Scope.PER_INSTANCE) advisor.removePerInstanceAspect(this);
         }
      }
      advisors.clear();
   }

}
