/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.InstanceAdvisor;
import org.jboss.aop.joinpoint.Joinpoint;
import org.jboss.util.xml.XmlLoadable;
import org.w3c.dom.Element;

/**
 * Lazy loading of AspectFactory
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class AspectFactoryDelegator implements AspectFactory
{
   private AspectFactory factory;
   private String factoryClass;
   private Element element;

   public AspectFactoryDelegator(String factory, Element element)
   {
      this.factoryClass = factory;
      this.element = element;
   }

   public String getName()
   {
      return factoryClass;
   }

   public Element getElement()
   {
      return element;
   }

   public void setElement(Element element)
   {
      this.element = element;
   }

   public synchronized AspectFactory getFactory()
   {
      if (factory == null)
      {
         try
         {
            Class clazz = Thread.currentThread().getContextClassLoader().loadClass(factoryClass);
            factory = (AspectFactory) clazz.newInstance();
            if (factory instanceof XmlLoadable)
            {
               ((XmlLoadable) factory).importXml(element);
            }
         }
         catch (ClassNotFoundException e)
         {
            throw new RuntimeException(e);
         }
         catch (InstantiationException e)
         {
            throw new RuntimeException(e);
         }
         catch (IllegalAccessException e)
         {
            throw new RuntimeException(e);
         }
      }
      return factory;
   }

   public Object createPerVM()
   {
      Object aspect = getFactory().createPerVM();
      return aspect;
   }

   public Object createPerClass(Advisor advisor)
   {
      Object aspect = getFactory().createPerClass(advisor);
      return aspect;
   }

   public Object createPerInstance(Advisor advisor, InstanceAdvisor instanceAdvisor)
   {
      Object aspect = getFactory().createPerInstance(advisor, instanceAdvisor);
      return aspect;
   }

   public Object createPerJoinpoint(Advisor advisor, Joinpoint jp)
   {
      Object aspect = getFactory().createPerJoinpoint(advisor, jp);
      return aspect;
   }

   public Object createPerJoinpoint(Advisor advisor, InstanceAdvisor instanceAdvisor, Joinpoint jp)
   {
      Object aspect = getFactory().createPerJoinpoint(advisor, instanceAdvisor, jp);
      return aspect;
   }

}
