/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.pointcut.CFlowMatcher;
import org.jboss.aop.pointcut.ast.ASTCFlowExpression;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.6 $
 *
 **/
public class CFlowInterceptor implements Interceptor
{
   Interceptor[] chain;
   ASTCFlowExpression expr;
   String cflowString;

   public CFlowInterceptor(String cflowString, ASTCFlowExpression expr, Interceptor[] chain)
   {
      this.chain = chain;
      this.expr = expr;
      this.cflowString = cflowString;
   }
   public String getName()
   {
      return "CFlowInterceptor";
   }

   public String getCFlowString()
   {
      return cflowString;
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      if (new CFlowMatcher().matches(expr, invocation))
      {
         Invocation wrapper = invocation.getWrapper(chain);
         return wrapper.invokeNext();
      }
      // no match for cflow so just go down chain
      return invocation.invokeNext();
   }
}
