/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.pointcut.DynamicCFlow;
import org.jboss.util.xml.XmlLoadable;
import org.w3c.dom.Element;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class DynamicCFlowDefinition
{
   private String name;
   private String className;
   private Element element;
   private Class pClass;

   public DynamicCFlowDefinition(Element element, String className, String name)
   {
      this.className = className;
      this.element = element;
      this.name = name;
   }

   public String getName()
   {
      return name;
   }

   public String getClassName()
   {
      return className;
   }

   public DynamicCFlow create()
   {
      if (pClass == null)
      {
         try
         {
            pClass = Thread.currentThread().getContextClassLoader().loadClass(className);
         }
         catch (ClassNotFoundException e)
         {
            throw new RuntimeException("dynamic cflow class not found: " + className);
         }
      }
      try
      {
         DynamicCFlow cflow = (DynamicCFlow) pClass.newInstance();
         if (cflow instanceof XmlLoadable)
         {
            ((XmlLoadable) cflow).importXml(element);
         }
         return cflow;
      }
      catch (InstantiationException e)
      {
         throw new RuntimeException(e);
      }
      catch (IllegalAccessException e)
      {
         throw new RuntimeException(e);
      }

   }

}
