
/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.joinpoint.Joinpoint;
import org.jboss.util.xml.XmlLoadable;
import org.w3c.dom.Element;

public class GenericInterceptorFactory implements InterceptorFactory
{
   private Class clazz = null;
   private String classname;
   private Element element;
   private boolean deployed = true;

   public GenericInterceptorFactory(String classname, Element element)
   {
      this.classname = classname;
      this.element = element;
   }

   public GenericInterceptorFactory(Class clazz)
   {
      this.clazz = clazz;
      this.classname = clazz.getName();
   }

   public void undeploy() { deployed = false; }

   public boolean isDeployed()
   {
      return deployed;
   }

   public String getClassName() { return classname; }

   public Interceptor create(Advisor advisor, Joinpoint joinpoint)
   {
      try
      {
         synchronized (this)
         {
            if (clazz == null)
            {
               clazz = Thread.currentThread().getContextClassLoader().loadClass(classname);
            }
         }
         Interceptor interceptor = (Interceptor)clazz.newInstance();
         if (interceptor instanceof XmlLoadable && element != null)
         {
            ((XmlLoadable)interceptor).importXml(element);
         }
         return interceptor;
      }
      catch (Exception re)
      {
         if (re instanceof RuntimeException) throw (RuntimeException)re;
         throw new RuntimeException(re);
      }
   }
}

