/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.Advised;
import org.jboss.aop.Advisor;
import org.jboss.aop.ClassInstanceAdvisor;
import org.jboss.aop.joinpoint.Invocation;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 **/
public class PerInstanceAdvice extends AbstractAdvice
{
   private boolean initialized = false;
   AspectDefinition aspectDefinition;

   public PerInstanceAdvice(String adviceName, AspectDefinition a, Advisor advisor)
   {
      aspectDefinition = a;
      this.adviceName = adviceName;
      advisor.addPerInstanceAspect(a);
   }

   public String getName()
   {
      return aspectDefinition.getName() + "." + adviceName;
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      Object targetObject = invocation.getTargetObject();
      if (targetObject == null) return invocation.invokeNext(); // static method call or static field call

      Advised advised = (Advised)targetObject;
      ClassInstanceAdvisor advisor = (ClassInstanceAdvisor)advised._getInstanceAdvisor();
      Object aspect = advisor.getPerInstanceAspect(aspectDefinition);
      if (!initialized)
      {
         init(adviceName, aspect.getClass());
      }
      Method advice = resolveAdvice(invocation);
      Object[] args = {invocation};

      try
      {
         return advice.invoke(aspect, args);
      }
      catch (InvocationTargetException e)
      {
         throw e.getCause();  //To change body of catch statement use Options | File Templates.
      }
   }

}
