/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.CtMethod;
import javassist.CtNewMethod;
import javassist.NotFoundException;
import org.jboss.aop.AspectManager;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.7 $
 *
 **/
public class PerVmAdvice
{
   private static WeakHashMap generated = new WeakHashMap();
   private static long counter = 0;

   public static synchronized Interceptor generateOptimized(String adviceName, AspectDefinition a) throws Exception
   {
      Map map = null;
      if (generated.containsKey(a))
      {
         map = (Map)generated.get(a);
         if (map != null)
         {
            if (map.containsKey(adviceName))
            {
               return (Interceptor) map.get(adviceName);
            }
         }
         else
         {
            map = new HashMap();
            generated.put(a, map);
         }
      }
      else
      {
         map = new HashMap();
         generated.put(a, map);
      }

      Object aspect = AspectManager.instance().getPerVMAspect(a);
      Interceptor rtn = generateInterceptor(aspect, adviceName);

      map.put(adviceName, rtn);
      return rtn;

   }

   public static Interceptor generateInterceptor(Object aspect, String adviceName) throws NotFoundException, CannotCompileException, IOException, InstantiationException, IllegalAccessException, NoSuchFieldException
   {
      Method[] methods = aspect.getClass().getMethods();
      ArrayList matches = new ArrayList();
      for (int i = 0; i < methods.length; i++)
      {
         if (methods[i].getName().equals(adviceName)) matches.add(methods[i]);
      }

      ClassPool pool = AspectManager.instance().findClassPool(aspect.getClass().getClassLoader());
      CtClass clazz = pool.makeClass("org.jboss.aop.advice." + aspect.getClass().getName() + counter++);
      
      // We need to know whether this Interceptor is actually advice.
      CtClass interceptorInterface = pool.get("org.jboss.aop.advice.Interceptor");
      CtClass abstractAdviceClass = pool.get("org.jboss.aop.advice.AbstractAdvice");
      clazz.setSuperclass(abstractAdviceClass);

      // aspect field
      CtClass aspectClass = pool.get(aspect.getClass().getName());
      CtField field = new CtField(aspectClass, "aspectField", clazz);
      field.setModifiers(javassist.Modifier.PUBLIC);
      clazz.addField(field);
      // getName()
      CtMethod getNameTemplate = interceptorInterface.getDeclaredMethod("getName");
      CtMethod getName = CtNewMethod.make(getNameTemplate.getReturnType(), "getName", getNameTemplate.getParameterTypes(), getNameTemplate.getExceptionTypes(), null, clazz);
      String getNameBody =
              "{ " +
              "   return \"" + aspect.getClass().getName() + "." + adviceName + "\"; " +
              "}";
      getName.setBody(getNameBody);
      getName.setModifiers(javassist.Modifier.PUBLIC);
      clazz.addMethod(getName);

      // invoke
      String invokeBody =
              "public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws java.lang.Throwable " +
              "{  ";
      if (matches.size() > 1)
      {
         for (int i = 0; i < matches.size(); i++)
         {
            Method advice = (Method) matches.get(i);
            String param = advice.getParameterTypes()[0].getName();
            invokeBody += "   if (invocation instanceof " + param + ") return aspectField." + adviceName + "((" + param + ")invocation); ";
         }
         invokeBody += "   return (org.jboss.aop.joinpoint.Invocation)null; ";
      }
      else
      {
         Method advice = (Method)matches.get(0);
         String param = advice.getParameterTypes()[0].getName();
         invokeBody += "return aspectField." + adviceName + "((" + param + ")invocation); ";
      }
      invokeBody += "}";
      CtMethod invoke = CtNewMethod.make(invokeBody, clazz);
      invoke.setModifiers(javassist.Modifier.PUBLIC);
      clazz.addMethod(invoke);
      Class iclass = clazz.toClass();
      
      Interceptor rtn = (Interceptor)iclass.newInstance();
      Field f = iclass.getField("aspectField");
      f.set(rtn, aspect);
      return rtn;
   }

}
