/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.advice;

import org.jboss.aop.Advisor;
import org.jboss.aop.AspectManager;
import org.jboss.aop.joinpoint.Joinpoint;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 *
 **/
public class ScopedInterceptorFactory implements InterceptorFactory
{
   private AspectDefinition aspect;

   public ScopedInterceptorFactory(AspectDefinition aspect)
   {
      this.aspect = aspect;
   }

   public AspectDefinition getInterceptorDefinition()
   {
      return aspect;
   }

   public boolean isDeployed()
   {
      return aspect.isDeployed();
   }

   public Interceptor create(Advisor advisor, Joinpoint joinpoint)
   {
      if (aspect.getScope() == Scope.PER_VM)
      {
         return (Interceptor)AspectManager.instance().getPerVMAspect(aspect);
      }
      else if (aspect.getScope() == Scope.PER_CLASS)
      {
         if (advisor.getAdviceInterceptor(aspect, "invoke") != null) return advisor.getAdviceInterceptor(aspect, "invoke");
         advisor.addPerClassAspect(aspect);
         Interceptor interceptor = (Interceptor)aspect.getFactory().createPerClass(advisor);
         advisor.addAdviceInterceptor(aspect, "invoke", interceptor);
         return interceptor;
      }
      else if (aspect.getScope() == Scope.PER_INSTANCE)
      {
         return new PerInstanceInterceptor(aspect, advisor);
      }
      else if (aspect.getScope() == Scope.PER_JOINPOINT)
      {
         try
         {
            return PerJoinpointInterceptor.createInterceptor(advisor, joinpoint, aspect);
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else
      {
         //if (aspect.getScope() == null) System.err.println("scope is null: " + aspect.getName() + "." + advice);
      }
      return null;
   }

   public String getName()
   {
      return aspect.getName();
   }
}
