/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.annotation;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Map;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 */
public class AnnotationProxy implements InvocationHandler
{
   Map map;

   public AnnotationProxy(Map valueMap)
   {
      map = valueMap;
   }

   public Object invoke(Object proxy, Method method, Object[] args)
           throws Throwable
   {
      if (method.getName().equals("equals"))
      {
         return doEquals(proxy, args[0]);
      }
      else if (method.getName().equals("hashCode"))
      {
         return doHashCode();
      }
      else if (method.getName().equals("toString"))
      {
         return map.toString();
      }
      /*
      Object obj = map.get(method.getName());
      if (!method.getReturnType().equals(obj.getClass()))
      {
         System.err.println("***** " + method.toString() + " has bad return type: " + obj.getClass().getName());
      }
      return obj;
      */
      return map.get(method.getName());
   }

   private Object doEquals(Object proxy, Object obj)
   {
      if (obj == proxy) return Boolean.TRUE;
      if (obj == null) return Boolean.FALSE;

      Class[] intfs = proxy.getClass().getInterfaces();
      if (!intfs[0].isAssignableFrom(obj.getClass()))
      {
         return Boolean.FALSE;
      }
      try
      {
         AnnotationProxy ih = (AnnotationProxy) Proxy.getInvocationHandler(obj);
      }
      catch (Exception ex)
      {
         return Boolean.FALSE;
      }
      return Boolean.TRUE;
   }

   private Object doHashCode()
   {
      return new Integer(map.hashCode());
   }

   public static Object createProxy(javassist.bytecode.annotation.Annotation info) throws Exception
   {
      Class annotation = Thread.currentThread().getContextClassLoader().loadClass(info.getTypeName());
      return createProxy(info, annotation);
   }

   public static Object createProxy(javassist.bytecode.annotation.Annotation info, Class annotation) throws Exception
   {
      Map map = ProxyMapCreator.createProxyMap(annotation, info);
      AnnotationProxy proxyHandler = new AnnotationProxy(map);
      return java.lang.reflect.Proxy.newProxyInstance(annotation.getClassLoader(), new Class[]{annotation}, proxyHandler);
   }

   public static Object createProxy(Map map, Class annotation) throws Exception
   {
      AnnotationProxy proxyHandler = new AnnotationProxy(map);
      return java.lang.reflect.Proxy.newProxyInstance(annotation.getClassLoader(), new Class[]{annotation}, proxyHandler);
   }
}
