/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.annotation;

import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
import javassist.CtMember;

import java.lang.reflect.Member;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Repository for annotations that is used by the ClassAdvisor to override annotations.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 */
public class AnnotationRepository
{
   Map annotations = new ConcurrentReaderHashMap();

   public Object resolveAnnotation(Member m, Class annotation)
   {
      return resolveAnnotation(m, annotation.getName());
   }

   public Object resolveAnnotation(Member m, String annotation)
   {
      Map map = (Map)annotations.get(m);
      if (map != null)
      {
         return map.get(annotation);
      }
      return null;
   }

   public void addAnnotation(Member m, Class annotation, Object value)
   {
      Map map = (Map)annotations.get(m);
      if (map == null)
      {
         map = new HashMap();
         annotations.put(m, map);
      }
      map.put(annotation.getName(), value);
   }

   public boolean hasAnnotation(Member m, Class annotation)
   {
      return resolveAnnotation(m, annotation) != null;
   }

   public boolean hasAnnotation(Member m, String annotation)
   {
      return resolveAnnotation(m, annotation) != null;
   }

   public boolean hasAnnotation(CtMember m, String annotation)
   {
      Set set = (Set)annotations.get(m);
      if (set != null) return set.contains(annotation);
      return false;
   }

   public void addAnnotation(CtMember m, String annotation)
   {
      Set set = (Set)annotations.get(m);
      if (set == null)
      {
         set = new HashSet();
      }
      set.add(annotation);
   }
}
