/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.annotation.compiler;

import com.thoughtworks.qdox.model.AbstractJavaEntity;
import com.thoughtworks.qdox.model.DocletTag;
import org.jboss.aop.annotation.ast.ASTAnnotation;
import org.jboss.aop.annotation.ast.ASTStart;
import org.jboss.aop.annotation.ast.AnnotationParser;
import org.jboss.aop.annotation.ast.ParseException;

import java.io.StringReader;
import java.util.Map;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 *
 **/
public class AnnotationDocletTag implements DocletTag
{
   private String name;
   private final String value;
   private final int lineNumber;
   private ASTAnnotation ast;

   private AbstractJavaEntity owner;

   public AnnotationDocletTag(String name, String value, int lineNumber)
   {
      this.name = name;
      this.value = value;
      this.lineNumber = lineNumber;
      if (name.startsWith("@"))
      {
         if (name.indexOf('(') != -1)
         {
            throw new RuntimeException("illegal annotation syntax for doclet at line number " + lineNumber + ".  You should have a space after the tag name otherwise the compiler messes up. " + name + " ***value=" + value);
            /* Can't do this because there may be a space in a string @annotation("hello world")
            if (value == null) value = "";
            String full = name + value;
            int index = full.indexOf('(');
            name = full.substring(0, index);
            value = full.substring(index);
            */
         }
         AnnotationParser parser = new AnnotationParser(new StringReader(name + value));
         try
         {
            ASTStart start = parser.Start();
            ast = (ASTAnnotation) start.jjtGetChild(0);
         }
         catch (ParseException e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
         this.name = name.substring(1);
      }
   }

   public AnnotationDocletTag(String name, String value)
   {
      this(name, value, 0);
   }

   public ASTAnnotation getAnnotation()
   {
      return ast;
   }

   public String getName()
   {
      return name;
   }

   public String getValue()
   {
      return value;
   }

   public String[] getParameters()
   {
      return null;
   }

   public Map getNamedParameterMap()
   {
      return null;
   }

   public String getNamedParameter(String key)
   {
      return null;
   }

   public int getLineNumber()
   {
      return lineNumber;
   }

   public final AbstractJavaEntity getContext()
   {
      return owner;
   }

   public void setContext(AbstractJavaEntity owner)
   {
      this.owner = owner;
   }

}
