/*
 * Created on Jul 26, 2004
 */
package org.jboss.aop.contrib;

import javassist.CannotCompileException;
import javassist.ClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.LoaderClassPath;
import javassist.NotFoundException;
import org.jboss.aop.AOPClassPool;
import org.jboss.aop.AspectManager;
import org.jboss.aop.ClassPoolFactory;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;

/**
 * @author Marshall
 * 
 * This is a class pool factory used by JBossIDE to create an AOP Class Pool
 */
public class IDEClassPoolFactory implements ClassPoolFactory {

	private ArrayList classPaths;
	
	public IDEClassPoolFactory ()
	{
		classPaths = new ArrayList();
	}
	
	public AOPClassPool create(ClassLoader loader, ClassPool pool, AspectManager manager) {
		SimpleClassPool classPool = new SimpleClassPool(loader, pool, manager);
		return classPool;
	}

	public AOPClassPool create(ClassPool pool, AspectManager manager) {
		SimpleClassPool classPool = new SimpleClassPool(pool, manager);
		return classPool;
	}

	public void insertClasspath (String path)
	{
		try {
			classPaths.add(new URL(path));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private class SimpleClassPool extends AOPClassPool
	{
		private ClassPath projectClasspath;
		private Loader loader;
		
		public SimpleClassPool (ClassLoader loader, ClassPool pool, AspectManager manager)
		{
			super (loader, pool, manager);
			loadClasspath();
		}
		
		public SimpleClassPool (ClassPool pool, AspectManager manager)
		{
			super (pool, manager);
			loadClasspath();

		}
		
		private void loadClasspath()
		{
			childFirstLookup = true;

			URL urlPaths[] = (URL[]) classPaths.toArray(new URL[classPaths.size()]);
			loader = new Loader(urlPaths, Thread.currentThread().getContextClassLoader());
			dcl = loader;
			
			Thread.currentThread().setContextClassLoader(loader);
			
			classPath = new LoaderClassPath(loader);
			insertClassPath(classPath);
		}

		public Class toClass(CtClass cc)
			throws CannotCompileException
		{
			try {
				return loader.loadClass(cc.getName());
			}
                        catch (Exception e)
                        {
                           try
                           {
			   return loader.loadClass(cc.getName(), cc.toBytecode());
                           }
                           catch(IOException e2)
                           {
                              throw new CannotCompileException(e2);
                           }
			}
		}
	}
	
	private class Loader extends URLClassLoader
	{
		public Loader (URL urls[], ClassLoader src)
		{
			super (urls, src);
		}
		
        public Class loadClass(String name, byte[] classfile)
            throws ClassFormatError
        {
    		Class c = defineClass(name, classfile, 0, classfile.length);

            resolveClass(c);
            return c;
        }
	}
}
