/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.expressions;

import javassist.CtConstructor;
import javassist.NotFoundException;

import java.lang.reflect.Constructor;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 **/
public class ConstructorExpression
{
   protected static final Pattern PATTERN = Pattern.compile("(.*)\\((.*)\\)");
   public String originalExpression;
   public ParameterExpression params;
   public Pattern classExpr;
   public String className;

   public ConstructorExpression(String ex)
   {
      Matcher m = PATTERN.matcher(ex);
      if (!m.matches()) throw new IllegalStateException("ConstructorExpression is invalid: " + ex);
      className = m.group(1);
      params = new ParameterExpression(m.group(2));
      className = className.replaceAll("\\.", "\\\\.");
      className = className.replaceAll("\\*", ".*");
      classExpr = Pattern.compile(className);
   }


   public boolean matches(Constructor con)
   {
      Matcher cm = classExpr.matcher(con.getDeclaringClass().getName());
      if (cm.matches() == false) return false;

      return params.matches(con.getParameterTypes());
   }

   public boolean matchParamsOnly(Constructor con)
   {
      return params.matches(con.getParameterTypes());
   }

   public boolean matchParamsOnly(CtConstructor con) throws NotFoundException
   {
      return params.matches(con.getParameterTypes());
   }

   public boolean matches(CtConstructor con) throws NotFoundException
   {
      Matcher cm = classExpr.matcher(con.getDeclaringClass().getName());
      if (cm.matches() == false) return false;

      return params.matches(con.getParameterTypes());
   }
}
