/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.expressions;

import javassist.CtMethod;
import javassist.NotFoundException;

import java.lang.reflect.Method;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 **/
public class MethodExpression
{
   protected static final Pattern PATTERN = Pattern.compile("(.*)\\s+(.*)\\((.*)\\)");
   public String originalExpression;
   public ParameterExpression params;
   public Pattern methodNameExpr;
   public String methodName;
   public String returnString;
   public Pattern returnExpr;

   public MethodExpression(String ex)
   {
      originalExpression = ex;
      Matcher m = PATTERN.matcher(ex);
      if (!m.matches()) throw new IllegalStateException("MethodExpression is invalid: " + ex);
      returnString = m.group(1);
      returnString = returnString.replaceAll("\\.", "\\\\.");
      returnString = returnString.replaceAll("\\*", ".*");
      returnExpr = Pattern.compile(returnString);
      methodName = m.group(2);
      methodName = methodName.replaceAll("\\.", "\\\\.");
      methodName = methodName.replaceAll("\\*", ".*");
      methodNameExpr = Pattern.compile(methodName);
      params = new ParameterExpression(m.group(3));
   }


   public boolean matches(Method method)
   {
      String returnType = ParameterExpression.simpleType(method.getReturnType());
      Matcher ret = returnExpr.matcher(returnType);
      if (!ret.matches()) return false;

      Matcher cm = methodNameExpr.matcher(method.getName());
      if (cm.matches() == false) return false;

      return params.matches(method.getParameterTypes());
   }

   public boolean matches(CtMethod method) throws NotFoundException
   {
      String returnType = method.getReturnType().getName();
      Matcher ret = returnExpr.matcher(returnType);
      if (!ret.matches()) return false;

      Matcher cm = methodNameExpr.matcher(method.getName());
      if (cm.matches() == false) return false;

      return params.matches(method.getParameterTypes());
   }
}
