/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.expressions;

import javassist.CtClass;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 **/
public class ParameterExpression
{
   protected static final Pattern ALL;
   protected static final Pattern PARAMS;

   static
   {
      ALL = Pattern.compile("\\s*\\.\\.\\s*");
      PARAMS = Pattern.compile("\\s*(.*)\\s*");
   }


   public String originalExpression;
   public Pattern[] parameterPatterns;
   public boolean isAll = false;

   protected static String simpleType(Class type)
   {
      Class ret = type;
      if (ret.isArray())
      {
         Class arr = ret;
         String array = "";
         while (arr.isArray())
         {
            array += "[]";
            arr = arr.getComponentType();
         }
         return arr.getName();
      }
      return ret.getName();
   }
   /**
    *
    * @param expression should be (..) or comma delimited (int, .*, java.lang.String)
    */
   public ParameterExpression(String expression)
   {
      originalExpression = expression;
      Matcher all = ALL.matcher(expression);
      isAll = all.matches();
      if (!isAll)
      {
         Matcher params = PARAMS.matcher(expression);
         if (!params.matches())
         {
            throw new RuntimeException("Illegal parameter list expression: " + expression);
         }
         StringTokenizer tokenizer = new StringTokenizer(expression, ",");
         ArrayList paramList = new ArrayList();
         while (tokenizer.hasMoreTokens())
         {
            String token = tokenizer.nextToken().trim();
            token = token.replaceAll("\\.", "\\\\.");
            token = token.replaceAll("\\*", ".*");
            paramList.add(Pattern.compile(token));
         }
         parameterPatterns = (Pattern[])paramList.toArray(new Pattern[paramList.size()]);
      }
   }

   public boolean matches(Class[] params)
   {
      if (isAll)
      {
         return true;
      }

      if (params.length != parameterPatterns.length) return false;

      for (int i = 0; i < params.length; i++)
      {
         String asString = simpleType(params[i]);
         Matcher m = parameterPatterns[i].matcher(asString);
         if (!m.matches()) return false;
      }
      return true;
   }

   public boolean matches(CtClass[] params)
   {
      if (isAll) return true;

      if (params.length != parameterPatterns.length) return false;

      for (int i = 0; i < params.length; i++)
      {
         String asString = params[i].getName();
         Matcher m = parameterPatterns[i].matcher(asString);
         if (!m.matches()) return false;
      }
      return true;
   }
}
