/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.hook;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jboss.aop.AspectManager;

/**
 * Generate the instrumented version of the classloader and store it in the filesystem.
 * @author kevin
 */
public class GenerateInstrumentedClassLoader
{
    /**
     * Store the instrumented classloader in the filesystem.
     * @param args The filename as the first argument.
     */
    public static void main(final String[] args)
    {
        if (args.length != 1)
        {
            System.err.println("Usage: java " + GenerateInstrumentedClassLoader.class.getName() + " <output directory>") ;
            System.exit(1) ;
        }
        final String filename = args[0] + File.separatorChar +
            "java" + File.separatorChar +
            "lang" + File.separatorChar +
            "ClassLoader.class" ;
        final File file = new File(filename) ;
        if (file.exists())
        {
            if (!file.canWrite())
            {
                System.err.println("Cannot write to existing file: " + file.getAbsolutePath()) ;
                System.exit(2) ;
            }
        }
        else
        {
            final File dir = file.getParentFile() ;
            if (!dir.exists())
            {
                dir.mkdirs() ;
            }
            if (!dir.canWrite())
            {
                System.err.println("Cannot write to parent directory: " + dir.getAbsolutePath()) ;
            }
        }
        
        final byte[] bytes ;
        try
        {
            bytes = AspectManager.instance().getInstrumentedClassLoader() ;
        }
        catch (final Throwable th)
        {
            System.err.println("Unexpected exception caught during instrumentation: " + th.getMessage()) ;
            th.printStackTrace(System.err) ;
            System.exit(5) ;
            return ;
        }
        
        try
        {
            final FileOutputStream fos = new FileOutputStream(file) ;
            fos.write(bytes) ;
            fos.close() ;
        }
        catch (final IOException ioe)
        {
            System.err.println("Unexpected exception caught while writing class file: " + ioe.getMessage()) ;
            ioe.printStackTrace(System.err) ;
            System.exit(6) ;
        }
    }
}
