/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.joinpoint;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.metadata.SimpleMetaData;

import java.util.Map;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.12 $
 *
 **/
public interface Invocation
{
   Map getResponseContextInfo();

   void setResponseContextInfo(Map responseContextInfo);

   void addResponseAttachment(Object key, Object val);

   Object getResponseAttachment(Object key);

   /**
    * Return all the contextual untyped data attached to this invocation
    */
   SimpleMetaData getMetaData();

   /**
    * Set all the contextual untyped data attached to this invocation
    */
   void setMetaData(SimpleMetaData data);

   /**
    * Resolve class level untyped metadata based on a key  and the attribute of the key
    * @param key
    * @param attr
    * @return
    */
   Object resolveClassMetaData(Object key, Object attr);

   /**
    * Abstraction for resolving an annotation so that
    * it can be overriden from the Class
    * @param annotation
    * @return
    */
   Object resolveClassAnnotation(Class annotation);

   /**
    * Abstraction for resolving an annotation so that
    * it can be overriden from the Method, Constructor, Field, etc.
    *
    * @param annotation
    * @return
    */
   Object resolveAnnotation(Class annotation);

   /**
    * Invoke on the next interceptor in the chain.  If this is already
    * the end of the chain, reflection will call the constructor, field, or
    * method you are invoking on.
    */
   Object invokeNext() throws Throwable;

   /**
    * Invoke on the next interceptor in the chain.  If this is already
    * the end of the chain, reflection will call the constructor, field, or
    * method you are invoking on.
    *
    * The Invocation will use a new set of interceptors to do the invocation
    */
   Object invokeNext(Interceptor[] newInterceptors) throws Throwable;

   Interceptor[] getInterceptors();

   /**
    * This method resolves untyped metadata based on the context of the invocation.
    * It iterates through its list of MetaDataResolvers to find out the
    * value of the metadata desired.
    *
    * This list usually is ThreadMetaData, InstanceAdvisor.getMetaData
    * ClassAdvisor.getMethodMetaData (or field, or constructor)
    * ClassAdvisor.getDefaultMetaData
    */
   Object getMetaData(Object group, Object attr);

   Object getTargetObject();

   void setTargetObject(Object targetObject);

   /**
    * Get a wrapper invocation object that can insert a new chain of interceptors
    * at runtime to the invocation flow.  CFlow makes use of this.
    * When the wrapper object finishes its invocation chain it delegates back to
    * the wrapped invocation.
    * @param newchain
    * @return
    */
   Invocation getWrapper(Interceptor[] newchain);

   /**
    * Copies complete state of Invocation object so that it could possibly
    * be reused in a spawned thread.
    * @return
    */
   public Invocation copy();


}
