/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.metadata;

import javassist.CtClass;
import javassist.NotFoundException;
import org.jboss.aop.Advisor;
import org.jboss.aop.AspectManager;
import org.jboss.aop.pointcut.Util;
import org.jboss.aop.pointcut.ast.ClassExpression;

import java.util.ArrayList;

/**
 *
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 */
public abstract class ClassMetaDataBinding
{
   protected ClassExpression classExpr;
   protected String expr;
   protected String name;
   protected String tag;
   protected ArrayList advisors = new ArrayList();
   protected ClassMetaDataLoader loader;

   public ClassMetaDataBinding(ClassMetaDataLoader loader, String name, String tag, String exp)
   {
      this.name = name;
      this.tag = tag;
      this.loader = loader;
      expr = exp;
      this.classExpr = new ClassExpression(expr);
   }

   public ClassMetaDataLoader getLoader()
   {
      return loader;
   }

   public String getName()
   {
      return name;
   }

   public void addAdvisor(Advisor advisor)
   {
      advisors.add(advisor.getName());
      advisor.addClassMetaData(this);
   }

   public void clearAdvisors()
   {
      for (int i = 0; i < advisors.size(); i++)
      {
         String advisorName = (String) advisors.get(i);
         Advisor advisor = AspectManager.instance().getAdvisor(advisorName);
         if (advisor != null)
            advisor.removeClassMetaData(this);
      }
   }

   public boolean equals(Object obj)
   {
      if (obj == this) return true;
      if (!(obj instanceof ClassMetaDataBinding)) return false;
      return ((ClassMetaDataBinding) obj).getName().equals(name);
   }

   public int hashCode()
   {
      return name.hashCode();
   }

   public boolean matches(Advisor advisor, Class clazz)
   {
      return Util.matchesClassExpr(classExpr, clazz, advisor);
   }

   public boolean matches(Advisor advisor, CtClass clazz) throws NotFoundException
   {
      return Util.matchesClassExpr(classExpr, clazz, advisor);
   }
   public String getClassExpr()
   {
      return expr;
   }

}
