/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.metadata;
import javassist.CtMethod;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.util.PayloadKey;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
/**
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.10 $
 *
 */
public class MethodMetaData implements MetaDataResolver
{
   HashMap methodMetaData = new HashMap();

   public boolean hasTag(String group)
   {
      Iterator values = methodMetaData.values().iterator();
      while (values.hasNext())
      {
         SimpleMetaData map = (SimpleMetaData)values.next();
         if (map.hasTag(group)) return true;
      }
      return false;
   }
   public void tagMethod(Method method, Object tag)
   {
      addMethodMetaData(method, tag, EMPTY_TAG, new Object(), PayloadKey.TRANSIENT);
   }
   public void addMethodMetaData(Method method, Object tag, Object attr, Object value)
   {
      addMethodMetaData(method, tag, attr, value, PayloadKey.MARSHALLED);
   }
   public synchronized void addMethodMetaData(Method method, Object tag, Object attr, Object value, PayloadKey type)
   {
      addMethodMetaData(method.toString(), tag, attr, value, type);
   }

   private synchronized void addMethodMetaData(String key, Object tag, Object attr, Object value, PayloadKey type)
   {
      SimpleMetaData methodData = (SimpleMetaData)methodMetaData.get(key);
      if (methodData == null)
      {
         methodData = new SimpleMetaData();
         methodMetaData.put(key, methodData);
      }
      methodData.addMetaData(tag, attr, value, type);
   }

   public synchronized boolean hasTag(Method method, String tag)
   {
      SimpleMetaData meta = getMethodMetaData(method);
      if (meta == null) return false;
      return meta.hasTag(tag);
   }

   public synchronized Iterator getMethods()
   {
      return methodMetaData.keySet().iterator();
   }

   /**
    * This requires a method signature derived from java.lang.reflect.Method.toString()
    * @param method
    * @return
    */
   public synchronized SimpleMetaData getMethodMetaData(String method)
   {
      return (SimpleMetaData)methodMetaData.get(method);
   }

   public SimpleMetaData getMethodMetaData(Method method)
   {
      return getMethodMetaData(method.toString());
   }

   public synchronized Object getMethodMetaData(Method method, Object tag, Object attr)
   {
      SimpleMetaData methodData = (SimpleMetaData)methodMetaData.get(method.toString());
      if (methodData == null) return null;
      return methodData.getMetaData(tag, attr);
   }

   public synchronized void clear()
   {
      methodMetaData.clear();
   }

   public Object resolve(Invocation invocation, Object tag, Object attr)
   {
      MethodInvocation methodInvocation = (MethodInvocation)invocation;
      return getMethodMetaData(methodInvocation.getMethod(), tag, attr);
   }

   public synchronized SimpleMetaData getAllMetaData(Invocation invocation)
   {
      MethodInvocation methodInvocation = (MethodInvocation)invocation;
      return (SimpleMetaData)methodMetaData.get(methodInvocation.getMethod().toString());
   }

   //--- temporary interface until metadata is bound to actual class, this is needed for loader/compiler to
   // match annotations to pointcuts

   private String getCtMethodKey(CtMethod method)
   {
      return method.getName() + ":" + method.getSignature();
   }

   public void tagMethod(CtMethod method, Object tag)
   {
      addMethodMetaData(method, tag, EMPTY_TAG, PayloadKey.TRANSIENT);
   }
   public void addMethodMetaData(CtMethod method, Object tag, Object attr, Object value)
   {
      addMethodMetaData(getCtMethodKey(method), tag, attr, value, PayloadKey.MARSHALLED);
   }
   public void addMethodMetaData(CtMethod method, Object tag, Object attr, Object value, PayloadKey type)
   {
      addMethodMetaData(getCtMethodKey(method), tag, attr, value, type);
   }

   public synchronized boolean hasGroup(CtMethod method, String tag)
   {
      SimpleMetaData meta = getMethodMetaData(getCtMethodKey(method));
      if (meta == null) return false;
      return meta.hasTag(tag);
   }


}
