/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.metadata;

import org.jboss.aop.expressions.ConstructorExpression;
import org.jboss.aop.expressions.MethodExpression;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
/**
 *
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 *
 */
public class SimpleClassMetaDataBinding extends ClassMetaDataBinding
{
   public class QueuedMetaData
   {
      protected Pattern expr;
      public String tag;
      public String attr;
      public Object value;

      public QueuedMetaData(String exp, String tag, String attr, Object value)
      {
         exp = exp.replaceAll("\\.", "\\\\.");
         exp = exp.replaceAll("\\*", ".*");
         this.expr = Pattern.compile(exp);
         this.tag = tag;
         this.attr = attr;
         this.value = value;
      }

      public String getExpr() { return this.expr.toString(); }
      public boolean matches(String match)
      {
         Matcher m = expr.matcher(match);
         return m.matches();
      }
   }

   public class QueuedMethodMetaData
   {
      public MethodExpression expr;
      public String tag;
      public String attr;
      public Object value;

      public QueuedMethodMetaData(String expr, String tag, String attr, Object value)
      {
         this.expr = new MethodExpression(expr);
         this.tag = tag;
         this.attr = attr;
         this.value = value;
      }
   }

   public class QueuedConstructorMetaData
   {
      public ConstructorExpression expr;
      public String tag;
      public String attr;
      public Object value;

      public QueuedConstructorMetaData(String expr, String tag, String attr, Object value)
      {
         this.expr = new ConstructorExpression(expr);
         this.tag = tag;
         this.attr = attr;
         this.value = value;
      }
   }

   protected SimpleMetaData defaultMetaData = new SimpleMetaData();
   protected SimpleMetaData classMetaData = new SimpleMetaData();
   protected ArrayList queuedMethodMetaData = new ArrayList();
   protected ArrayList queuedFieldMetaData = new ArrayList();
   protected ArrayList queuedConstructorMetaData = new ArrayList();

   public SimpleClassMetaDataBinding(ClassMetaDataLoader loader, String name, String tag, String expr)
   {
      super(loader, name, tag, expr);
   }

   public void addDefaultMetaData(String tag, String attr, Object value)
   {
      defaultMetaData.addMetaData(tag, attr, value);
   }

   public void addClassMetaData(String tag, String attr, Object value)
   {
      classMetaData.addMetaData(tag, attr, value);
   }

   /**
    * Queues up binding for method meta data
    */
   public void queueMethodMetaData(String methodExpr, String tag, String attr, Object value)
   {
      queuedMethodMetaData.add(new QueuedMethodMetaData(methodExpr, tag, attr, value));
   }

   public ArrayList getQueuedMethodMetaData()
   {
      return queuedMethodMetaData;
   }

   /**
    * Queues up binding for method meta data
    */
   public void queueFieldMetaData(String fieldExpr, String tag, String attr, Object value)
   {
      queuedFieldMetaData.add(new QueuedMetaData(fieldExpr, tag, attr, value));
   }

   public ArrayList getQueuedFieldMetaData()
   {
      return queuedFieldMetaData;
   }

   /**
    * Queues up binding for method meta data
    */
   public void queueConstructorMetaData(String constructorExpr, String tag, String attr, Object value)
   {
      queuedConstructorMetaData.add(new QueuedConstructorMetaData(constructorExpr, tag, attr, value));
   }

   public ArrayList getQueuedConstructorMetaData()
   {
      return queuedConstructorMetaData;
   }

   public SimpleMetaData getDefaultMetaData()
   {
      return defaultMetaData;
   }

   public SimpleMetaData getClassMetaData()
   {
      return classMetaData;
   }

}
