/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.metadata;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.util.PayloadKey;


/**
 * Metadata can be associated with a Thread of execution.
 * This class manages this thread metadata.  It is a singleton
 * instance wrapped by a ThreadLocal.  Invocations usually
 * have this repository as part of their MetaDataResolver chain
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 */
public class ThreadMetaData implements MetaDataResolver
{
   ThreadLocal metaData = new ThreadLocal();

   public void addMetaData(Object tag, Object attr, Object value)
   {
      addMetaData(tag, attr, value, PayloadKey.MARSHALLED);
   }

   public void addMetaData(Object tag, Object attr, Object value, PayloadKey type)
   {
      SimpleMetaData data = (SimpleMetaData)metaData.get();
      if (data == null)
      {
         data = new SimpleMetaData();
         metaData.set(data);
      }
      data.addMetaData(tag, attr, value, type);
   }

   public Object getMetaData(Object tag, Object attr)
   {
      SimpleMetaData data = (SimpleMetaData)metaData.get();
      if (data == null) return null;
      return data.getMetaData(tag, attr);
   }

   public synchronized void removeMetaData(Object tag, Object attr)
   {
      SimpleMetaData data = (SimpleMetaData)metaData.get();
      if (data == null) return;
      data.removeMetaData(tag, attr);
   }

   public synchronized void removeGroupData(Object tag)
   {
      SimpleMetaData data = (SimpleMetaData)metaData.get();
      if (data == null) return;
      data.removeGroupData(tag);
   }

   public Object resolve(Invocation invocation, Object tag, Object attr)
   {
      return getMetaData(tag, attr);
   }

   public SimpleMetaData getAllMetaData(Invocation invocation)
   {
      return (SimpleMetaData)metaData.get();
   }
   public void clear()
   {
      metaData.set(null);
   }


   private static final ThreadMetaData me = new ThreadMetaData();
   public static ThreadMetaData instance() { return me; }
}
