/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTExecution;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ParseException;
import org.jboss.aop.pointcut.ast.PointcutExpressionParser;
import org.jboss.aop.pointcut.ast.SimpleNode;

import java.io.StringReader;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 *
 **/
public class CFlow
{
   public static int NOT_FOUND = -2;

   private String original;
   private SimpleNode point;
   private boolean not;

   public CFlow(String expr, boolean not)
   {
      original = expr;
      expr = "execution(" + expr + ")";
      ASTExecution exc = null;
      try
      {
         exc = new PointcutExpressionParser(new StringReader(expr)).execution();
      }
      catch (ParseException e)
      {
         throw new RuntimeException("Illegal cflow expression: " + original, e);
      }
      point = (SimpleNode) exc.jjtGetChild(0);
      this.not = not;
   }

   public int matches(StackTraceElement[] stack, int index)
   {
      int found = NOT_FOUND;
      if (point instanceof ASTMethod)
      {
         found = matches((ASTMethod) point, stack, index);
      }
      else
      {
         found = matches((ASTConstructor) point, stack, index);
      }
      if ((found == NOT_FOUND) && not) return index;
      if (found > -1 && not) return NOT_FOUND;
      return found;
   }

   private int matches(ASTMethod method, StackTraceElement[] stack, int index)
   {
      for (int i = index; i >= 0; i--)
      {
         if (!method.getClazz().matches(stack[i].getClassName()))
         {
            continue;
         }
         if (method.getMethodIdentifier().matches(stack[i].getMethodName()))
         {
            return i - 1;
         }
      }
      return -2;
   }
   private int matches(ASTConstructor con, StackTraceElement[] stack, int index)
   {
      for (int i = index; i >= 0; i--)
      {
         if (!con.getClazz().matches(stack[i].getClassName()))
         {
            continue;
         }
         if (stack[i].getMethodName().equals("<init>"))
         {
            return i - 1; // match
         }
      }
      return -2;
   }
}
