/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import org.jboss.aop.AspectManager;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.pointcut.ast.ASTAndCFlow;
import org.jboss.aop.pointcut.ast.ASTCFlow;
import org.jboss.aop.pointcut.ast.ASTCFlowBoolean;
import org.jboss.aop.pointcut.ast.ASTCFlowExpression;
import org.jboss.aop.pointcut.ast.ASTCompositeCFlow;
import org.jboss.aop.pointcut.ast.ASTNotCFlow;
import org.jboss.aop.pointcut.ast.ASTOrCFlow;
import org.jboss.aop.pointcut.ast.ASTSubCFlow;
import org.jboss.aop.pointcut.ast.Node;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 **/
public class CFlowMatcher extends MatcherHelper
{
   StackTraceElement[] stack;
   Invocation invocation;
   public CFlowMatcher()
   {
      super(null);
   }

   public boolean matches(ASTCFlowExpression expr, Invocation invocation)
   {
      this.invocation = invocation;
      Boolean rtn = (Boolean)expr.jjtAccept(this, null);
      return rtn.booleanValue();
   }

   private StackTraceElement[] getStack()
   {
      stack = new Throwable().getStackTrace();
      return stack;
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      throw new RuntimeException("SHOULD NOT BE REACHABLE");
   }


   public Object visit(ASTCFlowExpression node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, data);
   }

   public Object visit(ASTCFlowBoolean node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, data);
   }

   public Object visit(ASTNotCFlow node, Object data)
   {
      Boolean bool = (Boolean)node.jjtGetChild(0).jjtAccept(this, data);
      boolean val = bool.booleanValue();
      return val ? Boolean.FALSE : Boolean.TRUE;
   }

   public Object visit(ASTCompositeCFlow node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, data);
   }

   public Object visit(ASTSubCFlow node, Object data)
   {
      for (int i = 0; i < node.jjtGetNumChildren(); i++)
      {
         data = node.jjtGetChild(i).jjtAccept(this, data);
      }
      return data;
   }

   public Object visit(ASTAndCFlow node, Object left)
   {
      Node andChild = node.jjtGetChild(0); // should only have one child
      boolean val = ((Boolean)left).booleanValue();
      return new Boolean(val && ((Boolean)andChild.jjtAccept(this, Boolean.FALSE)).booleanValue());
   }

   public Object visit(ASTOrCFlow node, Object left)
   {
      Node orChild = node.jjtGetChild(0); // should only have one child
      boolean val = ((Boolean)left).booleanValue();
      return new Boolean(val || ((Boolean)orChild.jjtAccept(this, Boolean.FALSE)).booleanValue());
   }

   public Object visit(ASTCFlow node, Object data)
   {
      CFlowStack cflow = AspectManager.instance().getCFlowStack(node.getPointcutName());
      if (cflow != null) return new Boolean(cflow.matches(getStack()));

      DynamicCFlow dcflow = AspectManager.instance().getDynamicCFlow(node.getPointcutName());
      return new Boolean(dcflow.shouldExecute(invocation));
   }
}
