/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import org.jboss.aop.AspectManager;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTCall;
import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTField;
import org.jboss.aop.pointcut.ast.ASTHas;
import org.jboss.aop.pointcut.ast.ASTHasField;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ASTWithin;
import org.jboss.aop.pointcut.ast.ASTWithincode;
import org.jboss.aop.pointcut.ast.Node;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.6 $
 *
 **/
public class CallMatcher extends MatcherHelper
{
   ClassAdvisor advisor;
   AccessibleObject within;
   Class calledClass;
   Method calledMethod;

   public CallMatcher(ClassAdvisor advisor, AccessibleObject within, Class calledClass, Method calledMethod, ASTStart start)
   {
      super(start);
      this.advisor = advisor;
      this.within = within;
      this.calledClass = calledClass;
      this.calledMethod = calledMethod;
   }

   public Object visit(ASTCall node, Object data)
   {
      try
      {
         if (!(node.getBehavior() instanceof ASTMethod)) return Boolean.FALSE;
         ASTMethod astMethod = (ASTMethod) node.getBehavior();
         ClassAdvisor calledAdvisor = AspectManager.instance().getAdvisorIfAdvised(calledClass);
         MethodMatcher methodMatcher = new MethodMatcher(calledAdvisor, calledMethod, null);
         if (!(methodMatcher.matches(astMethod)).booleanValue()) return Boolean.FALSE;
         return Boolean.TRUE;
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
   }

   public Object visit(ASTHas node, Object data)
   {
         Node n = node.jjtGetChild(0);
         if (n instanceof ASTMethod)
         {
            return new Boolean(Util.has(calledClass, (ASTMethod)n, advisor));
         }
         else
         {
            return new Boolean(Util.has(calledClass, (ASTConstructor)n, advisor));
         }
   }

   public Object visit(ASTHasField node, Object data)
   {
      ASTField f = (ASTField)node.jjtGetChild(0);
      return new Boolean(Util.has(calledClass, f, advisor));
   }
   public Object visit(ASTWithin node, Object data)
   {
      WithinMatcher visitor = new WithinMatcher(advisor, this.within, null);
      return node.jjtAccept(visitor, null);
   }

   public Object visit(ASTWithincode node, Object data)
   {
      WithinMatcher visitor = new WithinMatcher(advisor, this.within, null);
      return node.jjtAccept(visitor, null);
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      return new Boolean(p.matchesCall(advisor, within, calledClass, calledMethod));
   }

}
