/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtClass;
import javassist.CtConstructor;
import javassist.NotFoundException;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTAll;
import org.jboss.aop.pointcut.ast.ASTAttribute;
import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTException;
import org.jboss.aop.pointcut.ast.ASTParameter;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ClassExpression;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.13.2.3 $
 *
 **/
public class ConstructorMatcher extends MatcherHelper
{
   protected ClassAdvisor advisor;
   protected CtConstructor ctCon;
   protected Constructor refCon;
   protected int conModifiers;
   protected String classname;

   public ConstructorMatcher(ClassAdvisor advisor, CtConstructor con, ASTStart start) throws NotFoundException
   {
      super(start);
      this.advisor = advisor;
      this.start = start;
      this.conModifiers = con.getModifiers();
      this.classname = con.getDeclaringClass().getName();
      this.ctCon = con;
   }

   public ConstructorMatcher(ClassAdvisor advisor, Constructor con, ASTStart start)
   {
      super(start);
      this.advisor = advisor;
      this.start = start;
      this.conModifiers = con.getModifiers();
      this.classname = con.getDeclaringClass().getName();
      this.refCon = con;
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      throw new RuntimeException("SHOULD NOT BE CALLED");
   }

   public Object visit(ASTConstructor node, Object data)
   {
      return matches(node);
   }

   public Boolean matches(ASTConstructor node)
   {
      if (node.getAttributes().size() > 0)
      {
         for (int i = 0; i < node.getAttributes().size(); i++)
         {
            ASTAttribute attr = (ASTAttribute) node.getAttributes().get(i);
            if (!Util.matchModifiers(attr, conModifiers)) return Boolean.FALSE;
         }
      }

      if (ctCon != null)
      {
         if (!Util.matchesClassExpr(node.getClazz(), ctCon.getDeclaringClass(), advisor)) return Boolean.FALSE;
      }
      else
      {
         if (!Util.matchesClassExpr(node.getClazz(), refCon.getDeclaringClass(), advisor)) return Boolean.FALSE;
      }


      if (node.getConstructorAnnotation() != null)
      {
         String sub = node.getConstructorAnnotation().getOriginal().substring(1);
         if (ctCon != null)
         {
            if (!advisor.getConstructorMetaData().hasGroup(ctCon, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
               if (!advisor.hasAnnotation(ctCon, sub)) return Boolean.FALSE;
               }
            }
         }
         else
         {
            if (!advisor.getConstructorMetaData().hasTag(refCon, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
               try
               {
                  if (!advisor.hasAnnotation(refCon, sub)) return Boolean.FALSE;
               }
               catch (Exception e)
               {
                  throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
               }
               }
            }
         }
      }
      
      //Match exceptions
      ArrayList nodeExceptions = node.getExceptions();
      if (ctCon != null)
      {
         try
         {
            if (!Util.matchExceptions(nodeExceptions, ctCon.getExceptionTypes()))
            {
               return Boolean.FALSE;
            }
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);
         }
      }
      else
      {
         if (!Util.matchExceptions(nodeExceptions, refCon.getExceptionTypes()))
         {
            return Boolean.FALSE;
         }
      }
      
      if (node.isAnyParameters()) return Boolean.TRUE;
      if (ctCon != null)
      {
         try
         {
            if (node.getParameters().size() != ctCon.getParameterTypes().length) return Boolean.FALSE;
            CtClass[] params = ctCon.getParameterTypes();
            for (int i = 0; i < params.length; i++)
            {
               ASTParameter ast = (ASTParameter) node.getParameters().get(i);
               ClassExpression exp = ast.getType();
               if (!exp.matches(params[i].getName())) return Boolean.FALSE;
            }
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else
      {
         if (node.getParameters().size() != refCon.getParameterTypes().length) return Boolean.FALSE;
         Class[] params = refCon.getParameterTypes();
         for (int i = 0; i < params.length; i++)
         {
            ASTParameter ast = (ASTParameter) node.getParameters().get(i);
            ClassExpression exp = ast.getType();
            String asString = ClassExpression.simpleType(params[i]);
            if (!exp.matches(asString)) return Boolean.FALSE;
         }

      }
      return Boolean.TRUE;
   }

   public Object visit(ASTAll node, Object data)
   {
      if (node.getClazz().isAnnotation())
      {
         String sub = node.getClazz().getOriginal().substring(1);
         if (ctCon != null)
         {
            if (!advisor.getConstructorMetaData().hasGroup(ctCon, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  if (!advisor.hasAnnotation(ctCon, sub))
                  {
                     return Boolean.FALSE;
                  }
               }
            }
         }
         else
         {
            if (!advisor.getConstructorMetaData().hasTag(refCon, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  try
                  {
                     if (!advisor.hasAnnotation(refCon, sub)) return Boolean.FALSE;
                  }
                  catch (Exception e)
                  {
                     throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
                  }
               }
            }
         }
      }
      else if (node.getClazz().isInstanceOf())
      {
         if (ctCon != null)
         {
            try
            {
               if (!Util.subtypeOf(ctCon.getDeclaringClass(), node.getClazz())) return Boolean.FALSE;
            }
            catch (NotFoundException e)
            {
               throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
            }
         }
         else if (!Util.subtypeOf(refCon.getDeclaringClass(), node.getClazz())) return Boolean.FALSE;

      }
		else if (node.getClazz().isTypedef())
		{
			if (ctCon != null)
			{
				try
				{
					if (!Util.matchesTypedef(ctCon.getDeclaringClass(), node.getClazz(), advisor)) return Boolean.FALSE;
				}
				catch (Exception e)
				{
					throw new RuntimeException(e);
				}
			}
			else if (!Util.matchesTypedef(refCon.getDeclaringClass(), node.getClazz(), advisor)) return Boolean.FALSE;
		}      
      else if (!node.getClazz().matches(classname))
      {
         return Boolean.FALSE;
      }

      return Boolean.TRUE;
   }
}
