/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtConstructor;
import javassist.NotFoundException;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTExecution;
import org.jboss.aop.pointcut.ast.ASTField;
import org.jboss.aop.pointcut.ast.ASTHas;
import org.jboss.aop.pointcut.ast.ASTHasField;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.Node;

import java.lang.reflect.Constructor;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.5 $
 *
 **/
public class ExecutionConstructorMatcher extends ConstructorMatcher
{
   public ExecutionConstructorMatcher(ClassAdvisor advisor, CtConstructor con, ASTStart start) throws NotFoundException
   {
      super(advisor, con, start);
   }

   public ExecutionConstructorMatcher(ClassAdvisor advisor, Constructor con, ASTStart start)
   {
      super(advisor, con, start);
   }

   public Object visit(ASTExecution node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, data);
   }

   public Object visit(ASTHas node, Object data)
   {
      Node n = node.jjtGetChild(0);
      if (n instanceof ASTMethod)
      {
         if (ctCon != null)
         {
            return new Boolean(Util.has(ctCon.getDeclaringClass(), (ASTMethod) n, advisor));
         }
         else
         {
            return new Boolean(Util.has(refCon.getDeclaringClass(), (ASTMethod) n, advisor));

         }
      }
      else
      {
         if (ctCon != null)
         {
            return new Boolean(Util.has(ctCon.getDeclaringClass(), (ASTConstructor) n, advisor));
         }
         else
         {
            return new Boolean(Util.has(refCon.getDeclaringClass(), (ASTConstructor) n, advisor));

         }
      }
   }

   public Object visit(ASTHasField node, Object data)
   {
      ASTField f = (ASTField) node.jjtGetChild(0);
      if (ctCon != null)
      {
      return new Boolean(Util.has(ctCon.getDeclaringClass(), f, advisor));
      }
      else
      {
         return new Boolean(Util.has(refCon.getDeclaringClass(), f, advisor));
      }

   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      try
      {
         if (refCon != null) return new Boolean(p.matchesExecution(advisor, refCon));
         return new Boolean(p.matchesExecution(advisor, ctCon));
      }
      catch (NotFoundException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
   }
}
