/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtMethod;
import javassist.NotFoundException;
import org.jboss.aop.Advisor;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTExecution;
import org.jboss.aop.pointcut.ast.ASTField;
import org.jboss.aop.pointcut.ast.ASTHas;
import org.jboss.aop.pointcut.ast.ASTHasField;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.Node;

import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.5 $
 *
 **/
public class ExecutionMethodMatcher extends MethodMatcher
{
   public ExecutionMethodMatcher(Advisor advisor, CtMethod method, ASTStart start) throws NotFoundException
   {
      super(advisor, method, start);
   }

   public ExecutionMethodMatcher(Advisor advisor, Method method, ASTStart start)
   {
      super(advisor, method, start);
   }

   public Object visit(ASTExecution node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, data);
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      try
      {
         if (refMethod != null) return new Boolean(p.matchesExecution(advisor, refMethod));
         return new Boolean(p.matchesExecution(advisor, ctMethod));
      }
      catch (NotFoundException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
   }

   public Object visit(ASTHas node, Object data)
   {
      Node n = node.jjtGetChild(0);
      if (n instanceof ASTMethod)
      {
         if (ctMethod != null)
         {
            return new Boolean(Util.has(ctMethod.getDeclaringClass(), (ASTMethod) n, (ClassAdvisor) advisor));
         }
         else
         {
            return new Boolean(Util.has(refMethod.getDeclaringClass(), (ASTMethod) n, (ClassAdvisor) advisor));

         }
      }
      else
      {
         if (ctMethod != null)
         {
            return new Boolean(Util.has(ctMethod.getDeclaringClass(), (ASTConstructor) n, (ClassAdvisor) advisor));
         }
         else
         {
            return new Boolean(Util.has(refMethod.getDeclaringClass(), (ASTConstructor) n, (ClassAdvisor) advisor));

         }
      }
   }

   public Object visit(ASTHasField node, Object data)
   {
      ASTField f = (ASTField) node.jjtGetChild(0);
      if (ctMethod != null)
      {
         return new Boolean(Util.has(ctMethod.getDeclaringClass(), f, (ClassAdvisor) advisor));
      }
      else
      {
         return new Boolean(Util.has(refMethod.getDeclaringClass(), f, (ClassAdvisor)advisor));
      }

   }


}
