/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.joinpoint.ConstructorCalledByConstructorInvocation;
import org.jboss.aop.joinpoint.ConstructorCalledByMethodInvocation;
import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodCalledByConstructorInvocation;
import org.jboss.aop.joinpoint.MethodCalledByMethodInvocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.pointcut.ast.ParseException;

/**
 * Allows you to match an Invocation to a given joinpoint.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 **/
public class JoinPointMatcher
{
   Pointcut p;

   /**
    *
    * @param expr a pointcut expression to match against
    * @throws ParseException
    */
   public JoinPointMatcher(String expr) throws ParseException
   {
      p = new PointcutExpression("test", expr);
   }

   public boolean matches(Invocation invocation)
   {
      if (invocation instanceof MethodInvocation)
      {
         MethodInvocation mi = (MethodInvocation)invocation;
         return p.matchesExecution(mi.getAdvisor(), mi.getMethod());
      }
      else if (invocation instanceof ConstructorInvocation)
      {
         ConstructorInvocation mi = (ConstructorInvocation)invocation;
         return p.matchesExecution((ClassAdvisor)mi.getAdvisor(), mi.getConstructor());
      }
      else if (invocation instanceof FieldReadInvocation)
      {
         FieldReadInvocation mi = (FieldReadInvocation)invocation;
         return p.matchesGet((ClassAdvisor)mi.getAdvisor(), mi.getField());
      }
      else if (invocation instanceof FieldWriteInvocation)
      {
         FieldWriteInvocation mi = (FieldWriteInvocation)invocation;
         return p.matchesSet((ClassAdvisor)mi.getAdvisor(), mi.getField());
      }
      else if (invocation instanceof MethodCalledByMethodInvocation)
      {
         MethodCalledByMethodInvocation mi = (MethodCalledByMethodInvocation)invocation;
         return p.matchesCall((ClassAdvisor)mi.getAdvisor(), mi.getCallingMethod(), mi.getCalledMethod().getDeclaringClass(), mi.getCalledMethod());
      }
      else if (invocation instanceof MethodCalledByConstructorInvocation)
      {
         MethodCalledByConstructorInvocation mi = (MethodCalledByConstructorInvocation)invocation;
         return p.matchesCall((ClassAdvisor)mi.getAdvisor(), mi.getCalling(), mi.getCalledMethod().getDeclaringClass(), mi.getCalledMethod());
      }
      else if (invocation instanceof ConstructorCalledByConstructorInvocation)
      {
         ConstructorCalledByConstructorInvocation mi = (ConstructorCalledByConstructorInvocation)invocation;
         return p.matchesCall((ClassAdvisor)mi.getAdvisor(), mi.getCallingConstructor(), mi.getCalledConstructor().getDeclaringClass(), mi.getCalledConstructor());
      }
      else if (invocation instanceof ConstructorCalledByMethodInvocation)
      {
         ConstructorCalledByMethodInvocation mi = (ConstructorCalledByMethodInvocation)invocation;
         return p.matchesCall((ClassAdvisor)mi.getAdvisor(), mi.getCallingMethod(), mi.getCalledConstructor().getDeclaringClass(), mi.getCalledConstructor());
      }
      throw new RuntimeException("UNKNOWN JOINPOINT TYPE: " + invocation.getClass().getName());
   }
}
