/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import org.jboss.aop.Advisor;
import org.jboss.aop.pointcut.ast.ASTAll;
import org.jboss.aop.pointcut.ast.ASTAttribute;
import org.jboss.aop.pointcut.ast.ASTException;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ASTParameter;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ClassExpression;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.13.2.3 $
 */
public class MethodMatcher extends MatcherHelper
{
   protected Advisor advisor;
   protected CtMethod ctMethod;
   protected Method refMethod;
   protected int methodModifiers;
   protected String methodReturnType;
   protected String classname;
   protected String methodName;

   public MethodMatcher(Advisor advisor, CtMethod method, ASTStart start) throws NotFoundException
   {
      super(start);
      this.advisor = advisor;
      this.start = start;
      this.methodModifiers = method.getModifiers();
      this.methodReturnType = method.getReturnType().getName();
      this.classname = method.getDeclaringClass().getName();
      this.ctMethod = method;
      this.methodName = ctMethod.getName();
   }

   public MethodMatcher(Advisor advisor, Method method, ASTStart start)
   {
      super(start);
      this.advisor = advisor;
      this.start = start;
      this.methodModifiers = method.getModifiers();
      this.methodReturnType = method.getReturnType().getName();
      this.classname = method.getDeclaringClass().getName();
      this.refMethod = method;
      this.methodName = refMethod.getName();
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      throw new RuntimeException("SHOULD NOT BE CALLED");
   }

   public Object visit(ASTMethod node, Object data)
   {
      return matches(node);
   }

   public Boolean matches(ASTMethod node)
   {
      if (node.getAttributes().size() > 0)
      {
         for (int i = 0; i < node.getAttributes().size(); i++)
         {
            ASTAttribute attr = (ASTAttribute) node.getAttributes().get(i);
            if (!Util.matchModifiers(attr, methodModifiers)) return Boolean.FALSE;
         }
      }
      if (!node.getReturnType().matches(methodReturnType)) return Boolean.FALSE;


      if (ctMethod != null)
      {
         if (!Util.matchesClassExpr(node.getClazz(), ctMethod.getDeclaringClass(), advisor)) return Boolean.FALSE;
      }
      else
      {
         if (!Util.matchesClassExpr(node.getClazz(), refMethod.getDeclaringClass(), advisor)) return Boolean.FALSE;
      }


      if (node.getMethodIdentifier().isAnnotation())
      {
         if (advisor == null) return Boolean.FALSE;
         String sub = node.getMethodIdentifier().getOriginal().substring(1);
         if (ctMethod != null)
         {
            if (!advisor.getMethodMetaData().hasGroup(ctMethod, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  if (!advisor.hasAnnotation(ctMethod, sub)) return Boolean.FALSE;
               }
            }
         }
         else
         {
            if (!advisor.getMethodMetaData().hasTag(refMethod, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  try
                  {
                     if (!advisor.hasAnnotation(refMethod, sub)) return Boolean.FALSE;
                  }
                  catch (Exception e)
                  {
                     throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
                  }
               }
            }
         }
      }
      else
      {
         if (!node.getMethodIdentifier().matches(methodName)) return Boolean.FALSE;
      }
      
      //Match exceptions
      ArrayList nodeExceptions = node.getExceptions();
      if (ctMethod != null)
      {
         try
         {
            if (!Util.matchExceptions(nodeExceptions, ctMethod.getExceptionTypes()))
            {
               return Boolean.FALSE;
            }
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);
         }
      }
      else
      {
         if (!Util.matchExceptions(nodeExceptions, refMethod.getExceptionTypes()))
         {
            return Boolean.FALSE;
         }
      }
      
      
      if (node.isAnyParameters()) return Boolean.TRUE;
      if (ctMethod != null)
      {
         try
         {
            if (node.getParameters().size() != ctMethod.getParameterTypes().length) return Boolean.FALSE;
            CtClass[] params = ctMethod.getParameterTypes();
            for (int i = 0; i < params.length; i++)
            {
               ASTParameter ast = (ASTParameter) node.getParameters().get(i);
               ClassExpression exp = ast.getType();
               if (!exp.matches(params[i].getName())) return Boolean.FALSE;
            }
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      else
      {
         if (node.getParameters().size() != refMethod.getParameterTypes().length) return Boolean.FALSE;
         Class[] params = refMethod.getParameterTypes();
         for (int i = 0; i < params.length; i++)
         {
            ASTParameter ast = (ASTParameter) node.getParameters().get(i);
            ClassExpression exp = ast.getType();
            String asString = ClassExpression.simpleType(params[i]);
            if (!exp.matches(asString)) return Boolean.FALSE;
         }

      }
      return Boolean.TRUE;
   }

   public Object visit(ASTAll node, Object data)
   {
      if (node.getClazz().isAnnotation())
      {
         String sub = node.getClazz().getOriginal().substring(1);
         if (ctMethod != null)
         {
            if (!advisor.getMethodMetaData().hasGroup(ctMethod, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  if (!advisor.hasAnnotation(ctMethod, sub)) return Boolean.FALSE;
               }
            }
         }
         else
         {
            if (!advisor.getMethodMetaData().hasTag(refMethod, sub))
            {
               if (!advisor.getDefaultMetaData().hasTag(sub))
               {
                  try
                  {
                     if (!advisor.hasAnnotation(refMethod, sub)) return Boolean.FALSE;
                  }
                  catch (Exception e)
                  {
                     throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
                  }
               }
            }
         }
      }
      else if (node.getClazz().isInstanceOf())
      {
         if (ctMethod != null)
         {
            try
            {
               if (!Util.subtypeOf(ctMethod.getDeclaringClass(), node.getClazz())) return Boolean.FALSE;
            }
            catch (NotFoundException e)
            {
               throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
            }
         }
         else if (!Util.subtypeOf(refMethod.getDeclaringClass(), node.getClazz())) return Boolean.FALSE;

      }
      else if (node.getClazz().isTypedef())
      {
         if (ctMethod != null)
         {
            try
            {
               if (!Util.matchesTypedef(ctMethod.getDeclaringClass(), node.getClazz(), advisor)) return Boolean.FALSE;
            }
            catch (Exception e)
            {
               throw new RuntimeException(e);
            }
         }
         else if (!Util.matchesTypedef(refMethod.getDeclaringClass(), node.getClazz(), advisor)) return Boolean.FALSE;
      }
      else if (!node.getClazz().matches(classname))
      {
         return Boolean.FALSE;
      }

      return Boolean.TRUE;
   }
}
