/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtConstructor;
import javassist.CtField;
import javassist.CtMethod;
import javassist.NotFoundException;
import javassist.expr.MethodCall;
import javassist.expr.NewExpr;
import org.jboss.aop.Advisor;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ParseException;
import org.jboss.aop.pointcut.ast.PointcutExpressionParser;

import java.io.StringReader;
import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 **/
public class PointcutExpression implements Pointcut
{
   protected String name;
   protected String expr;
   protected ASTStart ast;

   public PointcutExpression(String name, String expr) throws ParseException
   {
      this.name = name;
      this.expr = expr;


      ast = new PointcutExpressionParser(new StringReader(expr)).Start();
   }

   public String getName()
   {
      return name;
   }

   public String getExpr()
   {
      return expr;
   }

   public boolean softMatch(Advisor advisor)
   {
      SoftClassMatcher matcher = new SoftClassMatcher(advisor, advisor.getName(), ast);
      return matcher.matches();
   }

   public boolean matchesCall(ClassAdvisor callingAdvisor, MethodCall methodCall) throws NotFoundException
   {
      MethodCallMatcher matcher = new MethodCallMatcher(callingAdvisor, methodCall, ast);
      return matcher.matches();
   }

   public boolean matchesCall(ClassAdvisor callingAdvisor, NewExpr methodCall) throws NotFoundException
   {
      NewExprMatcher matcher = new NewExprMatcher(callingAdvisor, methodCall, ast);
      return matcher.matches();
   }

   public boolean matchesExecution(Advisor advisor, Method m)
   {
      ExecutionMethodMatcher matcher = new ExecutionMethodMatcher(advisor, m, ast);
      return matcher.matches();
   }

   public boolean matchesExecution(ClassAdvisor advisor, Constructor c)
   {
      ExecutionConstructorMatcher matcher = new ExecutionConstructorMatcher(advisor, c, ast);
      return matcher.matches();
   }

   public boolean matchesGet(ClassAdvisor advisor, Field f)
   {
      FieldMatcher matcher = new FieldGetMatcher(advisor, f, ast);
      return matcher.matches();
   }

   public boolean matchesSet(ClassAdvisor advisor, Field f)
   {
      FieldMatcher matcher = new FieldSetMatcher(advisor, f, ast);
      return matcher.matches();
   }

   public boolean matchesExecution(Advisor advisor, CtMethod m) throws NotFoundException
   {
      ExecutionMethodMatcher matcher = new ExecutionMethodMatcher(advisor, m, ast);
      return matcher.matches();
   }

   public boolean matchesExecution(ClassAdvisor advisor, CtConstructor c) throws NotFoundException
   {
      ExecutionConstructorMatcher matcher = new ExecutionConstructorMatcher(advisor, c, ast);
      return matcher.matches();
   }

   public boolean matchesGet(ClassAdvisor advisor, CtField f) throws NotFoundException
   {
      FieldMatcher matcher = new FieldGetMatcher(advisor, f, ast);
      return matcher.matches();
   }

   public boolean matchesSet(ClassAdvisor advisor, CtField f) throws NotFoundException
   {
      FieldMatcher matcher = new FieldSetMatcher(advisor, f, ast);
      return matcher.matches();
   }

   public boolean matchesCall(ClassAdvisor advisor, AccessibleObject within, Class calledClass, Method calledMethod)
   {
      CallMatcher matcher = new CallMatcher(advisor, within, calledClass, calledMethod, ast);
      return matcher.matches();
   }

   public boolean matchesCall(ClassAdvisor advisor, AccessibleObject within, Class calledClass, Constructor calledCon)
   {
      ConstructorCallMatcher matcher = new ConstructorCallMatcher(advisor, within, calledClass, calledCon, ast);
      return matcher.matches();
   }


}
