/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut;

import javassist.CtBehavior;
import javassist.CtConstructor;
import javassist.CtMethod;
import javassist.NotFoundException;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.pointcut.ast.ASTConstructor;
import org.jboss.aop.pointcut.ast.ASTMethod;
import org.jboss.aop.pointcut.ast.ASTStart;
import org.jboss.aop.pointcut.ast.ASTWithin;
import org.jboss.aop.pointcut.ast.ASTWithincode;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 **/
public class WithinMatcher extends MatcherHelper
{
   CtBehavior behavior;
   ClassAdvisor advisor;
   AccessibleObject accessible;

   public WithinMatcher(ClassAdvisor advisor, CtBehavior behavior, ASTStart start) throws NotFoundException
   {
      super(start);
      this.advisor = advisor;
      this.behavior = behavior;
   }

   public WithinMatcher(ClassAdvisor advisor, AccessibleObject behavior, ASTStart start)
   {
      super(start);
      this.advisor = advisor;
      this.accessible = behavior;
   }

   protected Boolean resolvePointcut(Pointcut p)
   {
      throw new RuntimeException("NOT REACHABLE");
   }

   public Class getDeclaringClass()
   {
      if (accessible instanceof Constructor) return ((Constructor)accessible).getDeclaringClass();
      else if (accessible instanceof Method) return ((Method)accessible).getDeclaringClass();
      return null;
   }

   public Object visit(ASTWithin node, Object data)
   {
      if (behavior != null)
      {
         if (!Util.matchesClassExpr(node.getClazz(), behavior.getDeclaringClass(), advisor)) return Boolean.FALSE;
      }
      else
      {
         if (!Util.matchesClassExpr(node.getClazz(), getDeclaringClass(), advisor)) return Boolean.FALSE;
      }
      return Boolean.TRUE;
   }

   public Object visit(ASTWithincode node, Object data)
   {
      return node.jjtGetChild(0).jjtAccept(this, null);
   }

   public Object visit(ASTMethod node, Object data)
   {
      if (behavior != null)
      {
         try
         {
            if (behavior instanceof CtConstructor) return Boolean.FALSE;
            MethodMatcher matcher = new MethodMatcher(advisor, (CtMethod) behavior, null);
            return matcher.matches(node);
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      if (accessible instanceof Constructor) return Boolean.FALSE;
      MethodMatcher matcher = new MethodMatcher(advisor, (Method) accessible, null);
      return matcher.matches(node);

   }

   public Object visit(ASTConstructor node, Object data)
   {
      if (behavior != null)
      {
         try
         {
            if (behavior instanceof CtMethod) return Boolean.FALSE;
            ConstructorMatcher matcher = new ConstructorMatcher(advisor, (CtConstructor) behavior, null);
            return matcher.matches(node);
         }
         catch (NotFoundException e)
         {
            throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
         }
      }
      if (accessible instanceof Method) return Boolean.FALSE;
      ConstructorMatcher matcher = new ConstructorMatcher(advisor, (Constructor) accessible, null);
      return matcher.matches(node);

   }

}
