/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut.ast;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.5 $
 */
public class ClassExpression
{
   private String original;
   private Pattern classPattern;
   private boolean isAnnotation = false;
   private boolean isInstanceOf = false;
   private boolean isTypedef = false;


   public ClassExpression(String expr)
   {
      original = expr;
      if (expr.startsWith("@"))
      {
         isAnnotation = true;
      }
      else
      {
         expr = original;
         if (expr.startsWith("$instanceof{"))
         {
            isInstanceOf = true;
            expr = expr.substring(12, expr.lastIndexOf("}"));

         }
         else if (expr.startsWith("$typedef{"))
         {
            isTypedef = true;
            expr = expr.substring(9, expr.lastIndexOf("}"));
         }

         expr = expr.replaceAll("\\.", "\\\\.");
         expr = expr.replaceAll("\\*", ".*");
         expr = expr.replaceAll("\\[", "\\\\[");
         expr = expr.replaceAll("]", "\\\\]");
         expr = expr.replaceAll("\\$", "\\\\\\$");
         classPattern = Pattern.compile(expr);
      }

   }

   public boolean isSimple()
   {
      return !(isAnnotation || isInstanceOf || isTypedef);
   }


   public boolean matches(String classname)
   {
      if (isAnnotation) return false;
      Matcher m = classPattern.matcher(classname);
      return m.matches();
   }

   public boolean matchesAnnotation(String annotation)
   {
      if (!isAnnotation) return false;
      Matcher m = classPattern.matcher(annotation);
      return m.matches();
   }

   public boolean isAnnotation()
   {
      return this.isAnnotation;
   }

   public boolean isInstanceOf()
   {
      return this.isInstanceOf;
   }

   public boolean isTypedef()
   {
      return this.isTypedef;
   }

   public String getOriginal()
   {
      return original;
   }

   public static String simpleType(Class type)
   {
      Class ret = type;
      if (ret.isArray())
      {
         Class arr = ret;
         String array = "";
         while (arr.isArray())
         {
            array += "[]";
            arr = arr.getComponentType();
         }
         return arr.getName() + array;
      }
      return ret.getName();
   }
}
