/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.pointcut.ast;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 *
 **/
public class IdentifierExpression
{
   private String original;
   private Pattern namePattern;
   private boolean isAnnotation;

   public IdentifierExpression(String expr)
   {
      original = expr;
      if (expr.startsWith("@"))
      {
         isAnnotation = true;
      }
      else
      {
         expr = expr.replaceAll("\\*", ".*");
         namePattern = Pattern.compile(expr);
      }
   }

   public boolean matches(String name)
   {
      if (isAnnotation) return false;
      Matcher m = namePattern.matcher(name);
      return m.matches();
   }

   public boolean matchesAnnotation(String annotation)
   {
      if (!isAnnotation) return false;
      Matcher m = namePattern.matcher(annotation);
      return m.matches();
   }

   public String getOriginal()
   {
      return original;
   }

   public boolean isAnnotation()
   {
      return this.isAnnotation;
   }
}
