/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.util;

import java.lang.reflect.Constructor;


/**
 * Compares Constructors. This is used to make sure the constructor IDs line up between
 * the advisor and the instrumentor.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class ConstructorComparator implements java.util.Comparator
{
   private ConstructorComparator() {}

   public static final java.util.Comparator INSTANCE = new ConstructorComparator();
        
   private int compare(Constructor m1, Constructor m2) {
      try {
         Class[] args1 = m1.getParameterTypes();
         Class[] args2 = m2.getParameterTypes();
         if (args1.length < args2.length) return -1;
         if (args1.length > args2.length) return 1;
         for (int i = 0; i < args1.length; i++) {
            int result = args1[i].getName().compareTo(
               args2[i].getName());
            if (result != 0) return result;
         }
      } 
      catch (Exception e) {
         throw new RuntimeException(e);
      }
      // unreachable.
      throw new Error();
   }

   public int compare(Object o1, Object o2) {
      return compare((Constructor) o1, (Constructor) o2);
   }
}
