/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aop.util;

import javassist.CtClass;
import javassist.CtConstructor;
import javassist.bytecode.Descriptor;

/**
 * Compares CtConstructors. Used to ensure that the constructor IDs line up between the
 * instrumentor and advisor.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class CtConstructorComparator implements java.util.Comparator {

    private CtConstructorComparator() {}

    public static final java.util.Comparator INSTANCE = new CtConstructorComparator();
        
    private int compare(CtConstructor m1, CtConstructor m2) {
        try {
            CtClass[] args1 = m1.getParameterTypes();
            CtClass[] args2 = m2.getParameterTypes();
            if (args1.length < args2.length) return -1;
            if (args1.length > args2.length) return 1;
            for (int i = 0; i < args1.length; i++) {
                String name1 =
                    Descriptor.toJavaName(Descriptor.toJvmName(args1[i]));
                String name2 =
                    Descriptor.toJavaName(Descriptor.toJvmName(args2[i]));
                int result = name1.compareTo(name2);
                if (result != 0) return result;
            }
        }
        catch (Exception e) {
            throw new RuntimeException(e);
        }
        
        // unreachable.
        throw new Error();
    }

    public int compare(Object o1, Object o2) {
        return compare((CtConstructor) o1, (CtConstructor) o2);
    }

}
