/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects;

import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;

/**
 * This aspect should be scoped PER_JOINPOINT
 * It allows a field to be like a ThreadLocal
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 */
public class ThreadbasedAspect
{
   private ThreadLocal threadbased = new ThreadLocal();

   public Object access(FieldReadInvocation invocation) throws Throwable
   {
      // just in case we have a primitive, we can't return null
      if (threadbased.get() == null) return invocation.invokeNext();
      return threadbased.get();
   }

   public Object access(FieldWriteInvocation invocation) throws Throwable
   {
      threadbased.set(invocation.getValue());
      return null;
   }
}
