package org.jboss.test.aop.annotated;

/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aop.joinpoint.MethodInvocation;

/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 * @@org.jboss.aop.Aspect (scope=org.jboss.aop.advice.Scope.PER_CLASS)
 */
public class AspectPerClass
{
   public int constructorCalled;
   public int methodCalled;
   public int fieldRead;
   public int fieldWrite;

   /**
    * @@org.jboss.aop.Bind (pointcut="execution(org.jboss.test.aop.annotated.POJO*->new())")
    */
   public Object constructorAdvice(ConstructorInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerClass.constructorAdvice accessing: " + invocation.getConstructor().toString());
      constructorCalled++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.annotated.POJO*->someMethod())")
    */
   public Object methodAdvice(MethodInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerClass.methodAdvice accessing: " + invocation.getMethod().toString());
      methodCalled++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="set(* org.jboss.test.aop.annotated.POJO*->field)")
    */
   public Object fieldAdvice(FieldWriteInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerClass.fieldAdvice writing to field: " + invocation.getField().getName());
      fieldWrite++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="get(* org.jboss.test.aop.annotated.POJO*->field)")
    */
   public Object fieldAdvice(FieldReadInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerClass.fieldAdvice reading field: " + invocation.getField().getName());
      fieldRead++;
      return invocation.invokeNext();
   }
}
