package org.jboss.test.aop.annotated;

/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.pointcut.CFlow;
import org.jboss.aop.pointcut.CFlowStack;
import org.jboss.aop.pointcut.Pointcut;

/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.5 $
 * @@org.jboss.aop.Aspect (scope=org.jboss.aop.advice.Scope.PER_VM)
 */
public class AspectPerVM
{
   public int constructorCalled;
   public int methodCalled;
   public int fieldRead;
   public int fieldWrite;
   public int anotherPOJOAccess;
   public int cflowAccess;
   
   /**
    * @@org.jboss.aop.PointcutDef ("execution(org.jboss.test.aop.annotated.AnotherPOJO->new(..))") 
    */   
   public static Pointcut anotherPOJOConstructors;
   
   /**
    * @@org.jboss.aop.PointcutDef ("get(* org.jboss.test.aop.annotated.AnotherPOJO->field)") 
    */
   public static Pointcut anotherPOJOFieldReads;

   /**
    * @@org.jboss.aop.PointcutDef ("set(* org.jboss.test.aop.annotated.AnotherPOJO->field)") 
    */
   public static Pointcut anotherPOJOFieldWrites;
   
   /**
    * @@org.jboss.aop.PointcutDef ("execution(* org.jboss.test.aop.annotated.AnotherPOJO->*(..))") 
    */
   public static Pointcut anotherPOJOPublicMethods;

   /**
    * @@org.jboss.aop.PointcutDef ("org.jboss.test.aop.annotated.AspectPerVM.anotherPOJOFieldWrites OR org.jboss.test.aop.annotated.AspectPerVM.anotherPOJOFieldReads") 
    */
   public static Pointcut anotherPOJOFields;

   /**
    * @@org.jboss.aop.CFlowStackDef (cflows={@org.jboss.aop.CFlowDef (expr = "void org.jboss.test.aop.annotated.POJO->cflowMethod1()", called=true), @org.jboss.aop.CFlowDef (expr = "void org.jboss.test.aop.annotated.POJO->cflowMethod2()", called=true)})
    */
   public static CFlowStack cf1And2Stack;
      
   /**
    * @@org.jboss.aop.CFlowStackDef (cflows={@org.jboss.aop.CFlowDef (expr = "void org.jboss.test.aop.annotated.POJO->cflowMethod1()", called=false), @org.jboss.aop.CFlowDef (expr = "void org.jboss.test.aop.annotated.POJO->cflowMethod2()", called=true)})
    */
   public static CFlowStack cfNot1And2Stack;

   /**
    * @@org.jboss.aop.Prepare ("all(org.jboss.test.aop.annotated.PreparePOJO)") 
    */
   public static Pointcut preparePOJO;
   
   /**
    * @@org.jboss.aop.Bind (pointcut="execution(org.jboss.test.aop.annotated.POJO*->new())")
    */
   public Object constructorAdvice(ConstructorInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.constructorAdvice accessing: " + invocation.getConstructor().toString());
      constructorCalled++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.annotated.POJO*->someMethod())")
    */
   public Object methodAdvice(MethodInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.methodAdvice accessing: " + invocation.getMethod().toString());
      methodCalled++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="set(* org.jboss.test.aop.annotated.POJO*->field)")
    */
   public Object fieldAdvice(FieldWriteInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.fieldAdvice writing to field: " + invocation.getField().getName());
      fieldWrite++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut="get(* org.jboss.test.aop.annotated.POJO*->field)")
    */
   public Object fieldAdvice(FieldReadInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.fieldAdvice reading field: " + invocation.getField().getName());
      fieldRead++;
      return invocation.invokeNext();
   }

   /**
    * @@org.jboss.aop.Bind (pointcut = "org.jboss.test.aop.annotated.AspectPerVM.anotherPOJOFields OR org.jboss.test.aop.annotated.AspectPerVM.anotherPOJOConstructors OR org.jboss.test.aop.annotated.AspectPerVM.anotherPOJOPublicMethods")
    */
   public Object anotherPOJOAdvice(Invocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.anotherPOJOAdvice");
      anotherPOJOAccess++;
      return invocation.invokeNext();
   }
   
   /**
    * @@org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.annotated.POJO*->privateMethod())", cflow="(org.jboss.test.aop.annotated.AspectPerVM.cf1And2Stack OR org.jboss.test.aop.annotated.AspectPerVM.cfNot1And2Stack)")
    */
   public Object cflowAdvice(Invocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.cflowAdvice");
      cflowAccess++;
      return invocation.invokeNext();
   }
   
   /**
    * @@org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.annotated.POJO*->dynamicCFlowMethod())", cflow="org.jboss.test.aop.annotated.SimpleDynamicCFlow")
    */
   public Object dynamicCFlowAdvice(Invocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.dynamicCFlowAdvice");
      cflowAccess++;
      return invocation.invokeNext();
   }
   
    
}
