/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.aop.annotationc;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.jboss.aop.annotation.AnnotationElement;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * Tests an annotated introduction
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.1 $
 */
public class AnnotationTester extends TestCase
{
   public static Test suite()
   {
      TestSuite suite = new TestSuite("AnnotationTester");
      suite.addTestSuite(AnnotationTester.class);
      return suite;
   }
   // Constants ----------------------------------------------------
   // Attributes ---------------------------------------------------

   // Static -------------------------------------------------------

   // Constructors -------------------------------------------------
   //     * @@org.jboss.test.aop.annotationc.complex(ch='a', string="hello world", flt=5.5, dbl=6.6, shrt=5, lng=6, integer=7, bool=true, annotation=@single("hello"), array={"hello", "world"), clazz=java.lang.String)

   public AnnotationTester(String name)
   {
      super(name);
   }

   public void testClassAnnotation()
   {
      empty e = (empty) AnnotationElement.getAnyAnnotation(BytecodePOJO.class, empty.class);
      if (e == null) throw new RuntimeException("failed to get @empty");
      single s = (single) AnnotationElement.getAnyAnnotation(BytecodePOJO.class, single.class);
      if (s == null) throw new RuntimeException("failed to get @single");
      if (!s.value().equals("hello world")) throw new RuntimeException("@single.value mismatch: " + s.value());

      complex c = (complex) AnnotationElement.getAnyAnnotation(BytecodePOJO.class, complex.class);
      if (c == null) throw new RuntimeException("failed to get @complex");
      if (c.ch() != 'a') throw new RuntimeException("@complex.ch has wrong value");
      if (!c.string().equals("hello world")) throw new RuntimeException("@complex.string has wrong value");
      if (c.flt() != 5.5) throw new RuntimeException("@complex.flt has wrong value");
      if (c.dbl() != 6.6) throw new RuntimeException("@complex.dbl has wrong value");
      if (c.shrt() != 5) throw new RuntimeException("@complex.shrt has wrong value");
      if (c.lng() != 6) throw new RuntimeException("@complex.lng has wrong value");
      if (c.integer() != 7) throw new RuntimeException("@complex.integer has wrong value");
      if (c.bool() == false) throw new RuntimeException("@complex.bool has wrong value");
      s = c.annotation();
      if (s == null) throw new RuntimeException("@complex.annotation is null");
      if (!s.value().equals("hello")) throw new RuntimeException("@complex.annotation has wrong value");
      if (!c.array()[0].equals("hello")) throw new RuntimeException("@complex.array[0] has wrong value");
      if (!c.array()[1].equals("world")) throw new RuntimeException("@complex.array[1] has wrong value");
      if (!java.lang.String.class.equals(c.clazz())) throw new RuntimeException("@complex.clazz has wrong value");
      if (c.enumVal() != MyEnum.ONE) throw new RuntimeException("@complex.enumVal has wrong value");
   }

   public void testConstructorAnnotation()
   {
      Constructor con = BytecodePOJO.class.getConstructors()[0];
      empty e = (empty) AnnotationElement.getAnyAnnotation(con, empty.class);
      if (e == null) throw new RuntimeException("failed to get @empty");
      single s = (single) AnnotationElement.getAnyAnnotation(con, single.class);
      if (s == null) throw new RuntimeException("failed to get @single");
      if (!s.value().equals("hello world")) throw new RuntimeException("@single.value mismatch: " + s.value());

      complex c = (complex) AnnotationElement.getAnyAnnotation(con, complex.class);
      if (c == null) throw new RuntimeException("failed to get @complex");
      if (c.ch() != 'a') throw new RuntimeException("@complex.ch has wrong value");
      if (!c.string().equals("hello world")) throw new RuntimeException("@complex.string has wrong value");
      if (c.flt() != 5.5) throw new RuntimeException("@complex.flt has wrong value");
      if (c.dbl() != 6.6) throw new RuntimeException("@complex.dbl has wrong value");
      if (c.shrt() != 5) throw new RuntimeException("@complex.shrt has wrong value");
      if (c.lng() != 6) throw new RuntimeException("@complex.lng has wrong value");
      if (c.integer() != 7) throw new RuntimeException("@complex.integer has wrong value");
      if (c.bool() == false) throw new RuntimeException("@complex.bool has wrong value");
      s = c.annotation();
      if (s == null) throw new RuntimeException("@complex.annotation is null");
      if (!s.value().equals("hello")) throw new RuntimeException("@complex.annotation has wrong value");
      if (!c.array()[0].equals("hello")) throw new RuntimeException("@complex.array[0] has wrong value");
      if (!c.array()[1].equals("world")) throw new RuntimeException("@complex.array[1] has wrong value");
      if (!java.lang.String.class.equals(c.clazz())) throw new RuntimeException("@complex.clazz has wrong value");
      if (c.enumVal() != MyEnum.TWO) throw new RuntimeException("@complex.enumVal has wrong value");
   }

   public void testMethodAnnotation()
   {
      Method con = BytecodePOJO.class.getMethods()[0];
      empty e = (empty) AnnotationElement.getAnyAnnotation(con, empty.class);
      if (e == null) throw new RuntimeException("failed to get @empty");
      single s = (single) AnnotationElement.getAnyAnnotation(con, single.class);
      if (s == null) throw new RuntimeException("failed to get @single");
      if (!s.value().equals("hello world")) throw new RuntimeException("@single.value mismatch: " + s.value());

      complex c = (complex) AnnotationElement.getAnyAnnotation(con, complex.class);
      if (c == null) throw new RuntimeException("failed to get @complex");
      if (c.ch() != 'a') throw new RuntimeException("@complex.ch has wrong value");
      if (!c.string().equals("hello world")) throw new RuntimeException("@complex.string has wrong value");
      if (c.flt() != 5.5) throw new RuntimeException("@complex.flt has wrong value");
      if (c.dbl() != 6.6) throw new RuntimeException("@complex.dbl has wrong value");
      if (c.shrt() != 5) throw new RuntimeException("@complex.shrt has wrong value");
      if (c.lng() != 6) throw new RuntimeException("@complex.lng has wrong value");
      if (c.integer() != 7) throw new RuntimeException("@complex.integer has wrong value");
      if (c.bool() == false) throw new RuntimeException("@complex.bool has wrong value");
      s = c.annotation();
      if (s == null) throw new RuntimeException("@complex.annotation is null");
      if (!s.value().equals("hello")) throw new RuntimeException("@complex.annotation has wrong value");
      if (!c.array()[0].equals("hello")) throw new RuntimeException("@complex.array[0] has wrong value");
      if (!c.array()[1].equals("world")) throw new RuntimeException("@complex.array[1] has wrong value");
      if (!java.lang.String.class.equals(c.clazz())) throw new RuntimeException("@complex.clazz has wrong value");
      if (c.enumVal() != MyEnum.TWO) throw new RuntimeException("@complex.enumVal has wrong value");
   }

   public void testPrimitiveClassMemberAnnotation()
   {
      Method[] methods = BytecodePOJO.class.getMethods();
      primitive(methods[1], void.class);
      primitive(methods[2], int.class);
      primitive(methods[3], short.class);
      primitive(methods[4], long.class);
      primitive(methods[5], double.class);
      primitive(methods[6], float.class);
      primitive(methods[7], char.class);
      primitive(methods[8], boolean.class);
   }

   private void primitive(Method method, Class primitive)
   {
      ClassAnnotation ca = (ClassAnnotation) AnnotationElement.getAnyAnnotation(method, ClassAnnotation.class);
      if (ca == null) throw new RuntimeException("Failed to get ClassAnnotation for: " + method);
      if (!ca.value().equals(primitive)) throw new RuntimeException("failed to match " + primitive.getName() + "  for method: " + method);
   }

   public void testFieldAnnotation() throws Exception
   {
      Field con = BytecodePOJO.class.getField("field");
      empty e = (empty) AnnotationElement.getAnyAnnotation(con, empty.class);
      if (e == null) throw new RuntimeException("failed to get @empty");
      single s = (single) AnnotationElement.getAnyAnnotation(con, single.class);
      if (s == null) throw new RuntimeException("failed to get @single");
      if (!s.value().equals("hello world")) throw new RuntimeException("@single.value mismatch: " + s.value());

      complex c = (complex) AnnotationElement.getAnyAnnotation(con, complex.class);
      if (c == null) throw new RuntimeException("failed to get @complex");
      if (c.ch() != 'a') throw new RuntimeException("@complex.ch has wrong value");
      if (!c.string().equals("hello world")) throw new RuntimeException("@complex.string has wrong value");
      if (c.flt() != 5.5) throw new RuntimeException("@complex.flt has wrong value");
      if (c.dbl() != 6.6) throw new RuntimeException("@complex.dbl has wrong value");
      if (c.shrt() != 5) throw new RuntimeException("@complex.shrt has wrong value");
      if (c.lng() != 6) throw new RuntimeException("@complex.lng has wrong value");
      if (c.integer() != 7) throw new RuntimeException("@complex.integer has wrong value");
      if (c.bool() == false) throw new RuntimeException("@complex.bool has wrong value");
      s = c.annotation();
      if (s == null) throw new RuntimeException("@complex.annotation is null");
      if (!s.value().equals("hello")) throw new RuntimeException("@complex.annotation has wrong value");
      if (!c.array()[0].equals("hello")) throw new RuntimeException("@complex.array[0] has wrong value");
      if (!c.array()[1].equals("world")) throw new RuntimeException("@complex.array[1] has wrong value");
      if (!java.lang.String.class.equals(c.clazz())) throw new RuntimeException("@complex.clazz has wrong value");
      if (c.enumVal() != MyEnum.ONE) throw new RuntimeException("@complex.enumVal has wrong value");
   }
}

