/* *************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ************************************* */
package org.jboss.test.aop.basic;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author <a href="mailto:kabirkhan@bigfoot.com">Kabir Khan</a>
 */

public class ReflectionPOJO
{

   final static Package LANG_PACKAGE = Package.getPackage("java.lang");

   public ReflectionPOJO()
   {
      try
      {
         System.out.println("*** reflection (from constructor): Class.newInstance");
         Class clazz = ReflectionAopPOJO.class;
         clazz.newInstance();
         if (ReflectionAspectTester.constructor == null)
         {
            throw new RuntimeException("Not intercepted");
         }

         //Check Constructor.newInstance() interception
         System.out.println("*** reflection (from constructor): Constructor.newInstance");
         Constructor constructor = clazz.getConstructor(new Class[]{Integer.TYPE});
         Object obj = constructor.newInstance(new Object[]{new Integer(4)});
         
         //Check Field.setInt() interception
         System.out.println("*** reflection (from constructor): Field.setInt");
         Field field = clazz.getField("j");
         field.setInt(obj, 10);
         if (ReflectionAspectTester.field == null
                 || ((Integer) ReflectionAspectTester.args[0]).intValue() != 10)
         {
            throw new RuntimeException("Not intercepted");
         }
               
         
         //Check Field.getInt() interception
         System.out.println("*** reflection (from constructor): Field.getInt");
         int j = field.getInt(obj);
         if (ReflectionAspectTester.field == null
                 || j != 10)
         {
            throw new RuntimeException("Not intercepted");
         }
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);
      }

   }


   public void testCreationAndFieldAccess()
   {
      try
      {

         //Check Class.newInstance() interception
         System.out.println("*** reflection (from method): Class.newInstance");
         Class clazz = ReflectionAopPOJO.class;
         clazz.newInstance();
         if (ReflectionAspectTester.constructor == null)
         {
            throw new RuntimeException("Not intercepted");
         }


         //Check Constructor.newInstance() interception
         System.out.println("*** reflection (from method): Constructor.newInstance");
         Constructor constructor = clazz.getConstructor(new Class[]{Integer.TYPE});
         Object obj = constructor.newInstance(new Object[]{new Integer(4)});


         //Check Field.setInt() interception
         System.out.println("*** reflection (from method): Field.setInt");
         Field field = clazz.getField("j");
         field.setInt(obj, 10);
         if (ReflectionAspectTester.field == null
                 || ((Integer) ReflectionAspectTester.args[0]).intValue() != 10)
         {
            throw new RuntimeException("Not intercepted");
         }


         //Check Field.getInt() interception
         System.out.println("*** reflection (from method): Field.getInt");
         int j = field.getInt(obj);
         if (ReflectionAspectTester.field == null
                 || j != 10)
         {
            throw new RuntimeException("Not intercepted");
         }

      }
      catch (Exception e)
      {
         e.printStackTrace();
         throw new RuntimeException(e);
      }
   }

   public void testCleanGetMethods()
   {
      System.out.println("*** reflection (from method): Class.getMethods()");
      Class pojoClass = ReflectionAopPOJO.class;
      Class pojoRoot = ReflectionAopRootPOJO.class;
      try
      {
         //helloWorld() is introduced
         Method[] check = new Method[]{
            pojoClass.getMethod("method", new Class[0]),
            pojoClass.getMethod("method", new Class[]{Integer.TYPE}),
            pojoClass.getMethod("method", new Class[]{Integer.TYPE, Long.TYPE}),
            pojoRoot.getMethod("method", new Class[]{Integer.TYPE, Long.TYPE, Short.TYPE}),
            pojoClass.getMethod("helloWorld", new Class[]{String.class})};

         Method[] methods = pojoClass.getMethods();

         ArrayList methodList = new ArrayList();

         for (int i = 0; i < methods.length; i++)
         {
            if (!methods[i].getDeclaringClass().getPackage().equals(LANG_PACKAGE))
            {
               methodList.add(methods[i]);
            }
         }


         for (int i = 0; i < check.length; i++)
         {
            if (!methodList.remove(check[i]))
            {
               throw new RuntimeException("\"Cleaned\" Class.getMethods() did not return all expected methods: " + check[i]);
            }
         }

         if (methodList.size() > 0)
         {
            StringBuffer sb = new StringBuffer("The following methods should not "
                    + "have ben returned by \"Cleaned\" Class.getMethods():\n");

            for (Iterator it = methodList.iterator(); it.hasNext();)
            {
               sb.append(it.next());
            }

            throw new RuntimeException(sb.toString());
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException(e);
      }
   }

   public void testCleanGetDeclaredMethods()
   {

      System.out.println("*** reflection (from method): Class.getDeclaredMethods()");
      Class clazz = ReflectionAopPOJO.class;
      try
      {
         //helloWorld() is introduced
         Method[] check = new Method[]{
            clazz.getDeclaredMethod("method", new Class[0]),
            clazz.getDeclaredMethod("method", new Class[]{Integer.TYPE}),
            clazz.getMethod("method", new Class[]{Integer.TYPE, Long.TYPE}),
            clazz.getDeclaredMethod("privateMethod", new Class[0]),
            clazz.getMethod("helloWorld", new Class[]{String.class})};

         Method[] methods = clazz.getDeclaredMethods();

         ArrayList methodList = new ArrayList();

         for (int i = 0; i < methods.length; i++)
         {
            methodList.add(methods[i]);
         }


         for (int i = 0; i < check.length; i++)
         {
            if (!methodList.remove(check[i]))
            {
               throw new RuntimeException("\"Cleaned\" Class.getDeclaredMethods() did not return all expected methods: " + check[i]);
            }
         }

         if (methodList.size() > 0)
         {
            StringBuffer sb = new StringBuffer("The following methods should not "
                    + "have ben returned by \"Cleaned\" Class.getDeclaredMethods():\n");

            for (Iterator it = methodList.iterator(); it.hasNext();)
            {
               sb.append(it.next());
            }

            throw new RuntimeException(sb.toString());
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException(e);
      }
   }

   public void testCleanGetDeclaredFields()
   {

      System.out.println("*** reflection (from method): Class.getDeclaredFields()");
      Class clazz = ReflectionAopPOJO.class;
      try
      {
         Field[] check = new Field[]{
            clazz.getDeclaredField("i"),
            clazz.getDeclaredField("j")};

         Field[] fields = clazz.getDeclaredFields();

         ArrayList fieldList = new ArrayList();

         for (int i = 0; i < fields.length; i++)
         {
            fieldList.add(fields[i]);
         }


         for (int i = 0; i < check.length; i++)
         {
            if (!fieldList.remove(check[i]))
            {
               throw new RuntimeException("\"Cleaned\" Class.getDeclaredFields() did not return all expected methods: " + check[i]);
            }
         }

         if (fieldList.size() > 0)
         {
            StringBuffer sb = new StringBuffer("The following methods should not "
                    + "have ben returned by \"Cleaned\" Class.getDeclaredFields():\n");

            for (Iterator it = fieldList.iterator(); it.hasNext();)
            {
               sb.append(it.next());
            }

            throw new RuntimeException(sb.toString());
         }
      }
      catch (NoSuchFieldException e)
      {
         throw new RuntimeException(e);
      }
   }


   public void testCleanGetInterfaces()
   {

      System.out.println("*** reflection (from method): Class.getInterfaces()");
      ReflectionAopPOJO pojo = new ReflectionAopPOJO();
      Class clazz = ReflectionAopPOJO.class;
      Class[] interfaces = clazz.getInterfaces();

      if (interfaces.length != 1)
      {
         throw new RuntimeException("Expected 1 interface, got " + interfaces.length);
      }

      if (!interfaces[0].getName().equals("org.jboss.test.aop.basic.Introduction"))
      {
         throw new RuntimeException("Did not get expected interface org.jboss.test.aop.basic.Introduction");
      }
   }

   public void testCleanGetDeclaredMethod()
   {

      System.out.println("*** reflection (from method): Class.getDeclaredMethod()");
      Class pojoClass = ReflectionAopPOJO.class;
      Class pojoRoot = ReflectionAopRootPOJO.class;

      try
      {
         Method m = pojoClass.getDeclaredMethod("method", new Class[]{Integer.TYPE, Long.TYPE, Short.TYPE});
      }
      catch (NoSuchMethodException e)
      {
         //Fine, it is declared in ReflectionAopRootPOJO
      }

      try
      {
         Method m = pojoRoot.getDeclaredMethod("method", new Class[]{Integer.TYPE, Long.TYPE, Short.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopRootPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE)
                 || !m.getParameterTypes()[1].equals(Long.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = pojoClass.getDeclaredMethod("method", new Class[]{Integer.TYPE, Long.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE)
                 || !m.getParameterTypes()[1].equals(Long.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = pojoClass.getDeclaredMethod("method", new Class[]{Integer.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = pojoClass.getDeclaredMethod("method", new Class[0]);

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO"))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = pojoClass.getDeclaredMethod("privateMethod", new Class[0]);

         if (!m.getName().equals("privateMethod")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO"))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = pojoClass.getMethod("notThere", new Class[0]);
         throw new RuntimeException("Method should not be there");
      }
      catch (NoSuchMethodException e)
      {
         //Fine
      }

      try
      {
         Method m = pojoClass.getMethod("_getAdvisor", new Class[0]);
         throw new RuntimeException("Method should have been cleaned");
      }
      catch (NoSuchMethodException e)
      {
         //Fine, this is the cleaning bit
      }

      try
      {
         Method m = pojoRoot.getMethod("_getAdvisor", new Class[0]);
         throw new RuntimeException("Method should have been cleaned");
      }
      catch (NoSuchMethodException e)
      {
         //Fine, this is the cleaning bit
      }

   }

   public void testCleanGetMethod()
   {

      System.out.println("*** reflection (from method): Class.getMethod()");
      Class clazz = ReflectionAopPOJO.class;

      try
      {
         Method m = clazz.getMethod("method", new Class[]{Integer.TYPE, Long.TYPE, Short.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopRootPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE)
                 || !m.getParameterTypes()[1].equals(Long.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = clazz.getMethod("method", new Class[]{Integer.TYPE, Long.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE)
                 || !m.getParameterTypes()[1].equals(Long.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = clazz.getMethod("method", new Class[]{Integer.TYPE});

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !m.getParameterTypes()[0].equals(Integer.TYPE))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = clazz.getMethod("method", new Class[0]);

         if (!m.getName().equals("method")
                 || !m.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO"))
         {
            throw new RuntimeException("Wrong method got");
         }
      }
      catch (NoSuchMethodException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Method m = clazz.getMethod("notThere", new Class[0]);
         throw new RuntimeException("Method should not be there");
      }
      catch (NoSuchMethodException e)
      {
         //Fine
      }

      try
      {
         Method m = clazz.getMethod("_getAdvisor", new Class[0]);
         throw new RuntimeException("Method should have been cleaned");
      }
      catch (NoSuchMethodException e)
      {
         //Fine, this is the cleaning bit
      }

   }

   public void testCleanGetDeclaredField()
   {

      System.out.println("*** reflection (from method): Class.getDeclaredField()");
      Class clazz = ReflectionAopPOJO.class;
      try
      {
         Field f = clazz.getDeclaredField("i");

         if (!f.getName().equals("i")
                 || !f.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !f.getType().equals(Integer.TYPE))
         {
            throw new RuntimeException("Wrong field got");
         }
      }
      catch (NoSuchFieldException e)
      {
         throw new RuntimeException("Expected method not there");
      }

      try
      {
         Field f = clazz.getDeclaredField("j");

         if (!f.getName().equals("j")
                 || !f.getDeclaringClass().getName().equals("org.jboss.test.aop.basic.ReflectionAopPOJO")
                 || !f.getType().equals(Integer.TYPE))
         {
            throw new RuntimeException("Wrong field got");
         }
      }
      catch (NoSuchFieldException e)
      {
         throw new RuntimeException("Expected method not there");
      }
   }

   public void testCleanGetClasses()
   {
      //No extra public classes are added by AOP, but include this test in case stuff changes in future
      System.out.println("*** reflection (from method): Class.getClasses()");
      HashSet classSet = new HashSet();
      classSet.add("org.jboss.test.aop.basic.ReflectionAopPOJO$AopPOJOInner");
      classSet.add("org.jboss.test.aop.basic.ReflectionAopRootPOJO$AopRootPOJOInner");
      Class clazz = ReflectionAopPOJO.class;

      Class[] classes = clazz.getClasses();

      for (int i = 0; i < classes.length; i++)
      {
         String name = classes[i].getName();
         if (!classSet.contains(name))
         {
            throw new RuntimeException(name + " was in the list of classes and should have been removed");
         }
      }

      if (classes.length != classSet.size())
      {
         throw new RuntimeException("Not all the declared classes were returned");
      }
   }

   public void testCleanGetDeclaredClasses()
   {
      System.out.println("*** reflection (from method): Class.getDeclaredClasses()");
      HashSet declaredClasses = new HashSet();
      declaredClasses.add("org.jboss.test.aop.basic.ReflectionAopPOJO$AopPOJOInner");

      Class clazz = ReflectionAopPOJO.class;
      Class[] classes = clazz.getDeclaredClasses();

      for (int i = 0; i < classes.length; i++)
      {
         String name = classes[i].getName();
         if (!declaredClasses.contains(name))
         {
            throw new RuntimeException(name + " was in the list of declared classes and should have been removed");
         }
      }

      if (classes.length != declaredClasses.size())
      {
         throw new RuntimeException("Not all the declared classes were returned");
      }
   }


}
