package org.jboss.test.aop.jdk15.annotated;

/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.jboss.aop.Advised;
import org.jboss.aop.Advisor;
import org.jboss.aop.AspectManager;
import org.jboss.aop.InstanceAdvisor;


/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.5 $
 */
public class AnnotatedTestCase extends TestCase
{

   public static void main(String[] args)
   {
      TestRunner.run(suite());
   }

   public static Test suite()
   {
      TestSuite suite = new TestSuite("AnnotatedTester");
      suite.addTestSuite(AnnotatedTestCase.class);
      return suite;
   }

   public AnnotatedTestCase(String name)
   {
      super(name);
   }

   public void testBinding()
   {
      System.out.println("---- POJO ---");
      POJO pojo = new POJO();
      pojo.field++;
      pojo.someMethod();
      System.out.println("---- POJO2 ---");
      POJO2 pojo2 = new POJO2();
      pojo2.field++;
      pojo2.someMethod();

      System.out.println("-- get stats --");
      AspectPerVM vm = (AspectPerVM) AspectManager.instance().getPerVMAspect("org.jboss.test.aop.jdk15.annotated.AspectPerVM");
      System.out.println("perVM stats: " + vm.constructorCalled + " " + vm.methodCalled + " " + vm.fieldRead + " " + vm.fieldWrite);
      assertEquals(2, vm.constructorCalled);
      assertEquals(2, vm.methodCalled);
      assertEquals(2, vm.fieldRead);
      assertEquals(2, vm.fieldWrite);

      Advisor advisor = ((Advised) pojo)._getAdvisor();
      AspectPerClass perClass = (AspectPerClass) advisor.getPerClassAspect("org.jboss.test.aop.jdk15.annotated.AspectPerClass");
      System.out.println("POJO perClass stats: " + perClass.constructorCalled + " " + perClass.methodCalled + " " + perClass.fieldRead + " " + perClass.fieldWrite);
      assertEquals(1, perClass.constructorCalled);
      assertEquals(1, perClass.methodCalled);
      assertEquals(1, perClass.fieldRead);
      assertEquals(1, perClass.fieldWrite);

      advisor = ((Advised) pojo2)._getAdvisor();
      perClass = (AspectPerClass) advisor.getPerClassAspect("org.jboss.test.aop.jdk15.annotated.AspectPerClass");
      System.out.println("POJO2 perClass stats: " + perClass.constructorCalled + " " + perClass.methodCalled + " " + perClass.fieldRead + " " + perClass.fieldWrite);
      assertEquals(1, perClass.constructorCalled);
      assertEquals(1, perClass.methodCalled);
      assertEquals(1, perClass.fieldRead);
      assertEquals(1, perClass.fieldWrite);

      InstanceAdvisor ia = ((Advised) pojo)._getInstanceAdvisor();
      AspectPerInstance perInstance = (AspectPerInstance) ia.getPerInstanceAspect("org.jboss.test.aop.jdk15.annotated.AspectPerInstance");
      System.out.println("pojo perInstance stats: " + perInstance.methodCalled + " " + perInstance.fieldRead + " " + perInstance.fieldWrite);
      assertEquals(1, perInstance.methodCalled);
      assertEquals(1, perInstance.fieldRead);
      assertEquals(1, perInstance.fieldWrite);

      ia = ((Advised) pojo2)._getInstanceAdvisor();
      perInstance = (AspectPerInstance) ia.getPerInstanceAspect("org.jboss.test.aop.jdk15.annotated.AspectPerInstance");
      System.out.println("pojo2 perInstance stats: " + perInstance.methodCalled + " " + perInstance.fieldRead + " " + perInstance.fieldWrite);
      assertEquals(1, perInstance.methodCalled);
      assertEquals(1, perInstance.fieldRead);
      assertEquals(1, perInstance.fieldWrite);
   }
   
   public void testCompostition() throws Exception
   {
      System.out.println("---- AnotherPOJO ---");
      AnotherPOJO apojo = new AnotherPOJO();
      apojo.field++;
      apojo.someMethod();

      System.out.println("-- get stats --");
      AspectPerVM vm = (AspectPerVM) AspectManager.instance().getPerVMAspect("org.jboss.test.aop.jdk15.annotated.AspectPerVM");
      System.out.println("perVM stats: " + vm.constructorCalled + " " + vm.methodCalled + " " + vm.fieldRead + " " + vm.fieldWrite);
      assertEquals(4, vm.anotherPOJOAccess);
   }

   public void testMixin() throws Exception
   {
      System.out.println("***** testMixin() ****");
      POJONoInterfaces pojo = new POJONoInterfaces();
       
      pojo.stuff = "hello world";
      java.rmi.MarshalledObject mo = new java.rmi.MarshalledObject(pojo);
      pojo = (POJONoInterfaces)mo.get();
      System.out.println("deserialized pojo2.stuff2: " + pojo.stuff);
      assertTrue("writeExternal was not called", ExternalizableMixin.write);
      assertTrue("readExternal was not called", ExternalizableMixin.read);

      POJONoInterfaces2 pojo2 = new POJONoInterfaces2();

      pojo2.stuff = "whatever";
      java.rmi.MarshalledObject mo2 = new java.rmi.MarshalledObject(pojo2);
      pojo2 = (POJONoInterfaces2)mo2.get();
      System.out.println("deserialized pojo2.stuff2: " + pojo2.stuff);
      assertTrue("writeExternal was not called for pojo2", ExternalizableMixin.write);
      assertTrue("readExternal was not called for pojo2", ExternalizableMixin.read);
   
   }

   public void testIntroduction() throws Exception
   {
      System.out.println("***** testIntroduction() ****");
      POJONoInterfaces pojo = new POJONoInterfaces();
      
      
      try
      {
         EmptyInterface eif = (EmptyInterface)pojo;
      }
      catch(Exception e)
      {
         throw new RuntimeException("pojo does not implement EmptyInterface");
      }

      POJONoInterfaces2 pojo2 = new POJONoInterfaces2();
      
      
      try
      {
         EmptyInterface eif = (EmptyInterface)pojo2;
      }
      catch(Exception e)
      {
         throw new RuntimeException("pojo2 does not implement EmptyInterface");
      }
      
   }

    public void testInterceptorDef()throws Exception
    {
       System.out.println("***** testInterceptorDef() ****");

       CountingInterceptor.count = 0;
       POJO pojo = new POJO();
       pojo.methodWithInterceptor();
       System.out.println("Count: " + CountingInterceptor.count);
       assertEquals("execution of POJO.methodWithInterceptor() was not intercepted", 1, CountingInterceptor.count);

       CountingInterceptor.count = 0;
       pojo.methodWithInterceptorFactory();
       System.out.println("Count: " + CountingInterceptor.count);
       assertEquals("execution of POJO.methodWithInterceptorFactory() was not intercepted", 1, CountingInterceptor.count);
    }

    public void testTypedef()throws Exception
     {
        System.out.println("***** testTypedef() ****");

        POJO pojo = new POJO();
        pojo.methodWithTypedef();
        System.out.println("Intercepted: " + TypedefAspect.intercepted);
        assertTrue("execution of POJO.methodWithTypedef() was not intercepted", TypedefAspect.intercepted);
     }

    public void testCFlow()throws Exception
    {
       System.out.println("***** testCFlow() ****");

       AspectPerVM vm = (AspectPerVM) AspectManager.instance().getPerVMAspect("org.jboss.test.aop.jdk15.annotated.AspectPerVM");
       vm.cflowAccess = 0;

       POJO pojo = new POJO();
       pojo.cflowMethod1();
       assertEquals("Wrong number of interceptions 1) for cflowAdvice", 1, vm.cflowAccess);

       vm.cflowAccess = 0;
       pojo.cflowMethod2();
       System.out.println("ints: " + vm.cflowAccess);
       assertEquals("Wrong number of interceptions for 2) cflowAdvice", 1, vm.cflowAccess );

    }

    public void testPrepare()throws Exception
    {
       System.out.println("***** testPrepare() ****");
       PreparePOJO pojo = new PreparePOJO();
       pojo.someMethod();

       Advised advised = (Advised)pojo;
       Advisor advisor = advised._getAdvisor();

    }

    public void testDynamicCFlow()throws Exception
    {
       System.out.println("***** testDynamicCFlow() ****");

       AspectPerVM vm = (AspectPerVM) AspectManager.instance().getPerVMAspect("org.jboss.test.aop.jdk15.annotated.AspectPerVM");
       vm.cflowAccess = 0;

       POJO pojo = new POJO();
       pojo.dynamicCFlowMethod();
       assertEquals("Wrong number of interceptions for dynamic cflow Advice", 0, vm.cflowAccess);

       SimpleDynamicCFlow.execute = true;
       pojo.dynamicCFlowMethod();
       assertEquals("Wrong number of interceptions for dynamic cflow Advice", 1, vm.cflowAccess);

       SimpleDynamicCFlow.execute = false;
       pojo.dynamicCFlowMethod();
       assertEquals("Wrong number of interceptions for dynamic cflow Advice (2)", 1, vm.cflowAccess);
    }

}
