package org.jboss.test.aop.jdk15.annotated;

/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

import org.jboss.aop.*;
import org.jboss.aop.Prepare;
import org.jboss.aop.pointcut.Pointcut;
import org.jboss.aop.pointcut.CFlowStack;
import org.jboss.aop.advice.Scope;
import org.jboss.aop.joinpoint.*;

/**
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.5 $
 */
@Aspect(scope = Scope.PER_VM)
        public class AspectPerVM
{
   public int constructorCalled;
   public int methodCalled;
   public int fieldRead;
   public int fieldWrite;
   public int anotherPOJOAccess;
   public int cflowAccess;

   @PointcutDef("execution(org.jboss.test.aop.jdk15.annotated.AnotherPOJO->new(..))")
   public static Pointcut anotherPOJOConstructors;

   @PointcutDef("get(* org.jboss.test.aop.jdk15.annotated.AnotherPOJO->field)")
   public static Pointcut anotherPOJOFieldReads;

   @PointcutDef("set(* org.jboss.test.aop.jdk15.annotated.AnotherPOJO->field)")
   public static Pointcut anotherPOJOFieldWrites;

   @PointcutDef("execution(* org.jboss.test.aop.jdk15.annotated.AnotherPOJO->*(..))")
   public static Pointcut anotherPOJOPublicMethods;

   @PointcutDef("org.jboss.test.aop.jdk15.annotated.AspectPerVM.anotherPOJOFieldWrites OR org.jboss.test.aop.jdk15.annotated.AspectPerVM.anotherPOJOFieldReads")
   public static Pointcut anotherPOJOFields;

   @CFlowStackDef (cflows={@CFlowDef(expr = "void org.jboss.test.aop.jdk15.annotated.POJO->cflowMethod1()", called=true), @CFlowDef(expr = "void org.jboss.test.aop.jdk15.annotated.POJO->cflowMethod2()", called=true)})
   public static CFlowStack cf1And2Stack;

   @CFlowStackDef (cflows={@CFlowDef(expr = "void org.jboss.test.aop.jdk15.annotated.POJO->cflowMethod1()", called=false), @CFlowDef(expr = "void org.jboss.test.aop.jdk15.annotated.POJO->cflowMethod2()", called=true)})
   public static CFlowStack cfNot1And2Stack;

   @Prepare ("all(org.jboss.test.aop.jdk15.annotated.PreparePOJO)")
   public static Pointcut preparePOJO;


   @Bind(pointcut = "execution(org.jboss.test.aop.jdk15.annotated.POJO*->new())")
   public Object constructorAdvice(ConstructorInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.constructorAdvice accessing: " + invocation.getConstructor().toString());
      constructorCalled++;
      return invocation.invokeNext();
   }

   @Bind(pointcut = "execution(void org.jboss.test.aop.jdk15.annotated.POJO*->someMethod())")
   public Object methodAdvice(MethodInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.methodAdvice accessing: " + invocation.getMethod().toString());
      methodCalled++;
      return invocation.invokeNext();
   }

   @Bind(pointcut = "set(* org.jboss.test.aop.jdk15.annotated.POJO*->field)")
   public Object fieldAdvice(FieldWriteInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.fieldAdvice writing to field: " + invocation.getField().getName());
      fieldWrite++;
      return invocation.invokeNext();
   }

   @Bind(pointcut = "get(* org.jboss.test.aop.jdk15.annotated.POJO*->field)")
   public Object fieldAdvice(FieldReadInvocation invocation) throws Throwable
   {
      System.out.println("AspectPerVM.fieldAdvice reading field: " + invocation.getField().getName());
      fieldRead++;
      return invocation.invokeNext();
   }

    @Bind(pointcut = "org.jboss.test.aop.jdk15.annotated.AspectPerVM.anotherPOJOFields OR org.jboss.test.aop.jdk15.annotated.AspectPerVM.anotherPOJOConstructors OR org.jboss.test.aop.jdk15.annotated.AspectPerVM.anotherPOJOPublicMethods")
    public Object anotherPOJOAdvice(Invocation invocation) throws Throwable
    {
       System.out.println("AspectPerVM.anotherPOJOAdvice");
       anotherPOJOAccess++;
       return invocation.invokeNext();
    }
    
    @org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.jdk15.annotated.POJO*->privateMethod())", cflow="(org.jboss.test.aop.jdk15.annotated.AspectPerVM.cf1And2Stack OR org.jboss.test.aop.jdk15.annotated.AspectPerVM.cfNot1And2Stack)")
    public Object cflowAdvice(Invocation invocation) throws Throwable
    {
       System.out.println("AspectPerVM.cflowAdvice");
       cflowAccess++;
       return invocation.invokeNext();
    }

    @org.jboss.aop.Bind (pointcut="execution(void org.jboss.test.aop.jdk15.annotated.POJO*->dynamicCFlowMethod())", cflow="org.jboss.test.aop.jdk15.annotated.SimpleDynamicCFlow")
    public Object dynamicCFlowAdvice(Invocation invocation) throws Throwable
    {
       System.out.println("AspectPerVM.dynamicCFlowAdvice");
       cflowAccess++;
       return invocation.invokeNext();
    }

}
