/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.aop.jdk15.annotated;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

import org.jboss.aop.AnnotationIntroductionDef;
import org.jboss.aop.Bind;
import org.jboss.aop.InterceptorDef;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.annotation.AnnotationElement;
import org.jboss.aop.introduction.AnnotationIntroduction;
import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.MethodInvocation;

/**
 *
 * @author <a href="mailto:kabir.khan@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.1.2.2 $
 */
@InterceptorDef (scope=org.jboss.aop.advice.Scope.PER_VM)
@Bind (pointcut="all(org.jboss.test.aop.jdk15.annotated.IntroducedAnnotationPOJO)")
      public class IntroducedAnnotationInterceptor implements Interceptor
{
   @AnnotationIntroductionDef (expr="method(* org.jboss.test.aop.jdk15.annotated.IntroducedAnnotationPOJO->annotationIntroductionMethod())", invisible=false, annotation="@org.jboss.test.aop.jdk15.annotated.MyAnnotation (string='hello', integer=5, bool=true)")
   public static AnnotationIntroduction annotationIntroduction;

   public static MyAnnotation lastMyAnnotation;

   public String getName()
   {
      return "TestAnnotationInterceptor";
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      System.out.println("IntroducedInterceptor");
      if (invocation instanceof MethodInvocation)
      {
         Method method = ((MethodInvocation)invocation).getMethod();
         System.out.println("executing method " + method.toString());
         MyAnnotation myAnn = (MyAnnotation)AnnotationElement.getAnyAnnotation(method, MyAnnotation.class);
         lastMyAnnotation = myAnn;
      }
      else if (invocation instanceof ConstructorInvocation)
      {
         Constructor constructor = ((ConstructorInvocation)invocation).getConstructor();
         System.out.println("executing constructor " + constructor);
         MyAnnotation myAnn = (MyAnnotation)AnnotationElement.getAnyAnnotation(constructor, MyAnnotation.class);
         lastMyAnnotation = myAnn;
      }

      return invocation.invokeNext();
   }
}
