/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.aop.reflection;

import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aop.joinpoint.MethodInvocation;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * @author <a href="mailto:bill@jboss.org">Kabir Khan</a>
 * @version $Revision: 1.2.2.1 $
 */
public class SimplePerVmInterceptor implements org.jboss.aop.advice.Interceptor
{
   public static int constructorIntercepted;
   public static int fieldReadIntercepted;
   public static int fieldWriteIntercepted;
   public static int methodIntercepted;
   
   public String getName()
   {
      return "SimplePerVmInterceptor";
   }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      System.out.println("SimplePerVmInterceptor interception: " + invocation.getClass().getName());
      if (invocation instanceof MethodInvocation)
      {
         methodIntercepted++;
         System.out.println("methodIntercepted: " + methodIntercepted);
      }
      else if (invocation instanceof FieldReadInvocation)
      {
         fieldReadIntercepted++;
         System.out.println("fieldReadIntercepted: " + fieldReadIntercepted);
      }
      else if (invocation instanceof FieldWriteInvocation)
      {
         fieldWriteIntercepted++;
         System.out.println("fieldWriteIntercepted: " + fieldWriteIntercepted);
      }
      else if (invocation instanceof ConstructorInvocation)
      {
         constructorIntercepted++;
         System.out.println("constructorIntercepted:" + constructorIntercepted);
      }
      return invocation.invokeNext();
   }
   
   public static boolean hasIntercepted()
   {
      return (constructorIntercepted > 0 || fieldReadIntercepted > 0 || fieldWriteIntercepted > 0 ||methodIntercepted> 0);
   }
   
   public static void reset()
   {
      constructorIntercepted = 0;
      fieldReadIntercepted = 0;
      fieldWriteIntercepted = 0;
      methodIntercepted = 0;
   }
}

