/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

// $Id: JBossValueTagsHandler.java,v 1.1 2003/07/05 15:03:07 andygodwin Exp $

package xdoclet.modules.jboss.aop;

import java.util.Properties;
import xdoclet.XDocletException;
import xdoclet.XDocletTagSupport;

/**
 * JBossValueTagsHandler provides a simple template variable mechanism.
 * This is essentially the same as the matchValue in XDoclet's
 * AbstractProgramElementTagsHandler, except that in that class the
 * matchValue is oftentimes set to null by template methods so the
 * value can't be relied upon to persist throughout the template.  
 *
 * @author <a href="mailto:andy_godwin@hotmail.com">Andy Godwin</a>
 * @created July 5, 2003
 * @version $Revision: 1.1 $
 */
public class JBossValueTagsHandler
       extends XDocletTagSupport
{
   /**
    * The value of the variable.
    */
   protected String value;
   
   /**
    * Sets the value of the variable.
    *
    * @param template The body of the block tag
    * @param attributes The attributes of the template tag
    * @throws XDocletException 
    */
   public void setValue(String template, Properties attributes)
          throws XDocletException
   {
      value = attributes.getProperty("value");
      generate(template);
      value = null;
   }

   /**
    * Returns the value of the variable.
    *
    * @return the value of the variable
    * @throws XDocletException 
    */
   public String value()
          throws XDocletException
   {
      return value;
   }
}

