/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.deployment;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import org.jboss.aop.AOPClassPool;
import org.jboss.aop.AspectManager;
import org.jboss.mx.loading.UnifiedClassLoader;

import java.io.File;
import java.io.FileOutputStream;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1.2.3 $
 */
public class JBossClassPool32 extends AOPClassPool
{
   /**
    * Used for dynamically created classes (see loadClass(String, byte[]), ClassLoader)
    */
   protected File tempdir = null;
   // For loadClass tmpdir creation for UCL
   protected final Object tmplock = new Object();

   protected JBossClassPool32(ClassLoader cl, ClassPool src, AspectManager manager, File tmp)
   {
      super(cl, src, manager);
      tempdir = tmp;
   }

   protected JBossClassPool32(ClassPool src, AspectManager manager)
   {
      super(src, manager);
   }

   public Class toClass(CtClass cc, ClassLoader loader)
           throws CannotCompileException
   {
      if (dcl == null || tempdir == null)
      {
         return super.toClass(cc);
      }
      Class dynClass = null;
      try
      {
         File classFile = null;
         String classFileName = cc.getName().replace('.', '/') + ".class";
         // Write the clas file to the tmpdir
         synchronized (tmplock)
         {
            classFile = new File(tempdir, classFileName);
            File pkgDirs = classFile.getParentFile();
            pkgDirs.mkdirs();
            FileOutputStream stream = new FileOutputStream(classFile);
            stream.write(cc.toBytecode());
            stream.flush();
            stream.close();
            classFile.deleteOnExit();
         }
         // We have to clear Blacklist caches or the class will never
         // be found
         //((UnifiedClassLoader)dcl).clearBlacklists();
         // To be backward compatible
         UnifiedClassLoader rcl = (UnifiedClassLoader) dcl;
         rcl.clearBlackLists();

         // Now load the class through the cl
         dynClass = dcl.loadClass(cc.getName());
      }
      catch (Exception ex)
      {
         ClassFormatError cfe = new ClassFormatError("Failed to load dyn class: " + cc.getName());
         cfe.initCause(ex);
         throw cfe;
      }

      return dynClass;
   }

}
