/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aop.deployment;

import javassist.ClassPool;
import org.jboss.aop.AOPClassPool;
import org.jboss.aop.AspectManager;
import org.jboss.aop.ClassPoolFactory;
import org.jboss.mx.loading.RepositoryClassLoader;

import java.io.File;
import java.io.IOException;
import java.net.URL;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 *
 **/
public class JBossClassPoolFactory implements ClassPoolFactory
{
   protected File tmpClassesDir;

   public JBossClassPoolFactory(File tmpClassesDir) throws IOException
   {
      this.tmpClassesDir = tmpClassesDir;

   }
   public AOPClassPool create(ClassLoader cl, ClassPool src, AspectManager manager)
   {
      try
      {
         File tempdir = createTempDir(cl);
         return new JBossClassPool(cl, src, manager, tempdir);
      }
      catch (IOException e)
      {
         throw new RuntimeException(e);
      }
   }

   public AOPClassPool create(ClassPool src, AspectManager manager)
   {
      return new JBossClassPool(src, manager);
   }

   public File createTempDir(ClassLoader cl) throws IOException
   {
      if (!(cl instanceof RepositoryClassLoader)) return null;
      File tempdir = File.createTempFile("ucl", "", tmpClassesDir);
      tempdir.delete();
      tempdir.mkdir();
      tempdir.deleteOnExit();
      RepositoryClassLoader ucl = (RepositoryClassLoader) cl;
      URL tmpURL = tempdir.toURL();
      URL tmpCP = new URL(tmpURL, "?dynamic=true");
      ucl.addURL(tmpCP);
      return tempdir;
   }

}
