/*

 * Asynchronous Engine .

 *

 * Copyright (C) 2004 Claude Hussenet

 * Contact : chussenet@yahoo.com

 *

 * Distributable under LGPL license.

 * See terms of license at gnu.org.

 */

package org.jboss.aspects.asynchronous.concurrent;

import org.jboss.aspects.asynchronous.AsynchronousParameters;
import org.jboss.aspects.asynchronous.AsynchronousUserTask;
import org.jboss.aspects.asynchronous.ThreadManager;
import org.jboss.aspects.asynchronous.ThreadManagerRequest;
import org.jboss.aspects.asynchronous.common.ThreadManagerRequestImpl;

import java.util.ResourceBundle;

/**
 * @author <a href="mailto:chussenet@yahoo.com">{Claude Hussenet Independent Consultant}</a>.
 * @version <tt>$Revision: 1.2 $</tt>
 */

public class ThreadManagerFactory
{

   protected ThreadManager threadManager;

   protected String DEFAULT_MIN_THREAD = "10";

   protected String DEFAULT_MAX_THREAD = "100";

   protected String DEFAULT_KEEP_ALIVE = "5000";

   protected String DEFAULT_IS_WAITING_WHEN_POOLSIZE_FULL = "FALSE";

   protected String DEFAULT_IS_POOLING = "TRUE";

   static protected String MIN_POOL_SIZE = "threadManager.minimumPoolsize";

   static protected String MAX_POOL_SIZE = "threadManager.maximumPoolsize";

   static protected String KEEP_ALIVE = "threadManager.keepAlive";

   static protected String IS_WAITING_WHEN_POOLSIZE_FULL =

   "threadManager.isWaitingWhenPoolSize";

   static protected String IS_POOLING =

   "threadManager.isPooling";

   static protected String PROPERTY_FILE =

   "org.jboss.aspects.asynchronous.threadManager";

   protected static final ThreadManagerFactory threadManagerFactory =

   new ThreadManagerFactory();

   static protected ResourceBundle rb;

   static void initResourceBundle()
   {

      try
      {

         rb = ResourceBundle.getBundle(PROPERTY_FILE);

      }
      catch (Exception e)
      {

         //System.err.println(e.getMessage());

      }

   }

   private static String getString(String key, String defaultValue)
   {

      try
      {

         if (rb == null)

            return defaultValue;

         String value = rb.getString(key);

         if (value == null)

            return defaultValue;

         else

            return value;

      }
      catch (Exception e)
      {}

      return defaultValue;

   }

   protected ThreadManagerFactory()
   {

      initResourceBundle();

      threadManager = new ThreadManagerImpl();

      threadManager.setMinimumPoolSize(Integer.parseInt(getString(MIN_POOL_SIZE, DEFAULT_MIN_THREAD)));

      threadManager.setMaximumPoolSize(Integer.parseInt(getString(MAX_POOL_SIZE, DEFAULT_MAX_THREAD)));

      threadManager.setKeepAliveTime(Integer.parseInt(getString(KEEP_ALIVE, DEFAULT_KEEP_ALIVE)));

      threadManager.setWaitWhenPoolSizeIsFull(Boolean

      .valueOf(getString(IS_WAITING_WHEN_POOLSIZE_FULL,

      DEFAULT_IS_WAITING_WHEN_POOLSIZE_FULL))

      .booleanValue());


      threadManager.setPooling(Boolean

      .valueOf(getString(IS_POOLING,

      DEFAULT_IS_POOLING))

      .booleanValue());

   }

   static public ThreadManager getThreadManager()
   {

      return threadManagerFactory.threadManager;

   }

   static public ThreadManagerRequest createNewThreadManagerRequest(String id,

                                                                    AsynchronousParameters inputParameters,

                                                                    AsynchronousUserTask taskClassNameImpl,

                                                                    long timeout)
   {

      return new ThreadManagerRequestImpl(id,

      inputParameters,

      taskClassNameImpl,

      timeout);

   }

}
