/*

 * Asynchronous Engine .

 * 

 * Copyright (C) 2004 Claude Hussenet

 * Contact : chussenet@yahoo.com

 * 

 * Distributable under LGPL license.

 * See terms of license at gnu.org.

 */

package org.jboss.aspects.asynchronous.concurrent;

import EDU.oswego.cs.dl.util.concurrent.Callable;
import EDU.oswego.cs.dl.util.concurrent.FutureResult;
import EDU.oswego.cs.dl.util.concurrent.ThreadFactoryUser;
import org.jboss.aspects.asynchronous.ProcessingTime;


/**
 * @author <a href="mailto:chussenet@yahoo.com">{Claude Hussenet Independent Consultant}</a>.
 * @version <tt>$Revision: 1.2 $</tt>
 */

public class TimedCallableImpl extends ThreadFactoryUser implements Callable, ProcessingTime
{

   private final AdapterTask function;

   private final long millis;

   private long endingTime = -1;

   private long startingTime = -1;


   public TimedCallableImpl(AdapterTask function, long millis)
   {

      this.function = function;

      this.millis = millis;

   }

   public Object call() throws Exception
   {

      FutureResult result = new FutureResult();


      Thread thread = getThreadFactory().newThread(result.setter(function));

      thread.start();

      try
      {

         startingTime = System.currentTimeMillis();

         Object obj = result.timedGet(millis);

         endingTime = System.currentTimeMillis();

         return obj;

      }
      catch (InterruptedException ex)
      {

         endingTime = System.currentTimeMillis();

         function.cleanup();

         thread.interrupt();

         throw ex;

      }

   }


   public long getEndingTime()
   {

      return endingTime;

   }

   public long getStartingTime()
   {

      return startingTime;

   }

}

