/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.ioc;

import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
import org.jboss.aop.AspectManager;
import org.jboss.aop.advice.AdviceBinding;
import org.jboss.aop.advice.InterceptorFactory;
import org.jboss.aop.pointcut.Pointcut;
import org.jboss.aop.pointcut.ast.ParseException;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 *
 **/
public class DependencyInjectorContainer
{
   private static ConcurrentReaderHashMap registry = new ConcurrentReaderHashMap();
   private static InterceptorFactory[] interceptors = new InterceptorFactory[1];

   static
   {
      interceptors[0] = new IocInterceptor();
   }

   public static void register(String property, Object value, Class injectedType)
   {
      Identifier id = new Identifier(property, injectedType);
      boolean existed = registry.containsKey(id);
      registry.put(id, value);

      if (existed) return;

      // create an IOC pointcut and aspect binding.
      // expression is any constructor and the target class has a set method of that property name
      //
      // We do a binding so that we only cut those classes that have a property.  This requires the class to be
      // prepared.
      String name = "ioc:" + id.name + ":" + id.injectedType.getName();
      String expr = "execution(*->new(..)) AND has(void *->set" + id.name + "(" + id.injectedType.getName() + "))";
      try
      {
         Pointcut p = new IocPointcut(name, expr);
         AdviceBinding binding = new AdviceBinding(name, p, null, null, interceptors);
         AspectManager.instance().addPointcut(p); // add pointcut so that load-time transformations work
         AspectManager.instance().addBinding(binding);
      }
      catch (ParseException e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
   }

   public static Object getProperty(String property, Class injectedType)
   {
      Object value = registry.get(new Identifier(property, injectedType));
      if (value instanceof Factory) return ((Factory) value).create();

      return value;
   }


}
