/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.ioc;

import org.jboss.aop.Advisor;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.advice.InterceptorFactory;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.Joinpoint;

import java.lang.reflect.Method;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.4 $
 *
 **/
public class IocInterceptor implements Interceptor, InterceptorFactory
{
   public IocInterceptor()
   {
   }

   public boolean isDeployed()
   {
      return true;
   }

   public String getName()
   {
      return getClass().getName();
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      Object rtn = invocation.invokeNext();
      Method[] methods = rtn.getClass().getMethods();
      for (int i = 0; i < methods.length; i++)
      {
         if (!methods[i].getName().startsWith("set")) continue;
         if (!methods[i].getReturnType().equals(Void.TYPE)) continue;
         if (methods[i].getParameterTypes().length != 1) continue;
         String propname = methods[i].getName().substring(3);
         Class type = methods[i].getParameterTypes()[0];
         Object value = DependencyInjectorContainer.getProperty(propname, type);
         Object[] args = {value};
         methods[i].invoke(rtn, args);
      }
      return rtn;
   }

   public Interceptor create(Advisor advisor, Joinpoint jp)
   {
      return this;
   }

}
