/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.ioc;

import javassist.CtConstructor;
import javassist.CtField;
import javassist.CtMethod;
import javassist.NotFoundException;
import javassist.expr.MethodCall;
import javassist.expr.NewExpr;
import org.jboss.aop.Advisor;
import org.jboss.aop.ClassAdvisor;
import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.pointcut.PointcutExpression;
import org.jboss.aop.pointcut.ast.ParseException;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * This overides pointcut to work around a deficiency in the JBoss AOP
 * framework.  The construction execution matcher searches to make sure
 * that the IOC interceptor hasn't already been bound to the constructor
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 *
 **/
public class IocPointcut extends PointcutExpression
{

   public IocPointcut(String name, String expr) throws ParseException
   {
      super(name, expr);
   }

   public boolean matchesExecution(ClassAdvisor advisor, Constructor c)
   {
      if (advisor.getConstructorInterceptors() != null)
      {
         Interceptor[] interceptors = advisor.getConstructorInterceptors()[0];
         if (interceptors != null && interceptors.length > 0)
         {
            for (int i = 0; i < interceptors.length; i++)
            {
               if (interceptors[i] instanceof IocInterceptor) return false;
            }
         }
      }

      return super.matchesExecution(advisor, c);
   }

   //-- These should just return false

   public boolean matchesCall(ClassAdvisor callingAdvisor, MethodCall methodCall) throws NotFoundException
   {
      return false;
   }

   public boolean matchesCall(ClassAdvisor callingAdvisor, NewExpr methodCall) throws NotFoundException
   {
      return false;
   }

   public boolean matchesExecution(Advisor advisor, Method m)
   {
      return false;
   }

   public boolean matchesGet(ClassAdvisor advisor, Field f)
   {
      return false;
   }

   public boolean matchesSet(ClassAdvisor advisor, Field f)
   {
      return false;
   }

   public boolean matchesExecution(Advisor advisor, CtMethod m) throws NotFoundException
   {
      return false;
   }

   public boolean matchesExecution(ClassAdvisor advisor, CtConstructor c) throws NotFoundException
   {
      return false;
   }

   public boolean matchesGet(ClassAdvisor advisor, CtField f) throws NotFoundException
   {
      return false;
   }

   public boolean matchesSet(ClassAdvisor advisor, CtField f) throws NotFoundException
   {
      return false;
   }

   public boolean matchesCall(ClassAdvisor advisor, AccessibleObject within, Class calledClass, Method calledMethod)
   {
      return false;
   }

   public boolean matchesCall(ClassAdvisor advisor, AccessibleObject within, Class calledClass, Constructor calledCon)
   {
      return false;
   }
}
