/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.logging;

import java.util.ArrayList;

/**
 * An interceptor that tests whether logging should be done to
 * the invocation response.
 *
 * @author  <a href="mailto:adrian@jboss">Adrian Brock</a>.
 * @version $Revision: 1.2 $
 */
public class InvocationLogInterceptor
   implements org.jboss.aop.advice.Interceptor
{
   // Constants -----------------------------------------------------

   // Attributes ----------------------------------------------------

   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------

   // Public --------------------------------------------------------

   // Interceptor Implementation ------------------------------------

   public String getName()
   {
      return "InvocationLogInterceptor";
   }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation)
      throws Throwable
   {
      Object started = invocation.getMetaData(ThreadLocalAppender.LOG, ThreadLocalAppender.LOG);

      ArrayList log = null;
      if (started != null)
      {
         // Some earlier invocation started the logging
         if (ThreadLocalAppender.getList() != null)
            started = null;
         else
         {
            // We are the first, start the logging
            log = new ArrayList();
            ThreadLocalAppender.setList(log);
         }
      }

      try
      {
         // Perform the invocation and attach the log when we started it
         Object response = invocation.invokeNext();
         if (started != null)
            invocation.addResponseAttachment(ThreadLocalAppender.LOG, log);
         return response;
      }
      finally
      {
         // If we started the logging, stop it
         if (started != null)
            ThreadLocalAppender.setList(null);
      }
   }

   // Y Overrides ---------------------------------------------------

   // Protected -----------------------------------------------------

   // Private -------------------------------------------------------

   // Inner Classes -------------------------------------------------
}
