/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aspects.remoting;

import org.jboss.aop.util.PayloadKey;
import org.jboss.remoting.ClientInterceptor;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.InvokerLocator;

import java.util.Map;
/**
 * Checks to see if this object is local in VM
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
 * @version $Revision: 1.2 $
 */
public class AOPToRemotingClientAdapter implements org.jboss.aop.advice.Interceptor, java.io.Serializable
{
   public static final AOPToRemotingClientAdapter singleton = new AOPToRemotingClientAdapter();
   public static final String REMOTING = "REMOTING";
   public static final String REMOTING_CONTEXT = "REMOTING_CONTEXT";
   public static final String INVOKER_LOCATOR = "INVOKER_LOCATOR";
   public static final String SUBSYSTEM = "SUBSYSTEM";
   public static final String CLIENT_INTERCEPTOR = "CLIENT_INTERCEPTOR";

   public String getName() { return "AOPToRemotingClientAdapter"; }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      InvokerLocator locator = (InvokerLocator)invocation.getMetaData(REMOTING, INVOKER_LOCATOR);
      /* This is ok if we are using clustering
         if (locator == null)
         {
         throw new RuntimeException("No InvokerLocator supplied.  Can't invoke remotely!");
         }
      */
      String subsystem = (String)invocation.getMetaData(REMOTING, SUBSYSTEM);
      if (subsystem == null) subsystem = "AOP";

      ClientInterceptor ci = (ClientInterceptor)invocation.getMetaData(REMOTING, CLIENT_INTERCEPTOR);
      if (ci == null)
      {
         throw new RuntimeException("No remoting client interceptor supplied.  Can't invoke remotely--nothing to invoke!");
      }
      //transfer parameters

      Map requestPayload = (Map)invocation.getMetaData(REMOTING, REMOTING_CONTEXT);
      //It might not be set as transient
      invocation.getMetaData().removeMetaData(REMOTING, REMOTING_CONTEXT);

      InvocationRequest ireq = new InvocationRequest("sessionID", subsystem, invocation, requestPayload, null, locator);
      try
      {
         org.jboss.aop.joinpoint.InvocationResponse ir =  (org.jboss.aop.joinpoint.InvocationResponse)ci.invoke(ireq);
         invocation.setResponseContextInfo(ir.getContextInfo());
         invocation.addResponseAttachment(REMOTING_CONTEXT, ireq.getReturnPayload());
         return ir.getResponse();
      }
      finally
      {
         //restore metadata.
         invocation.getMetaData().addMetaData(REMOTING, REMOTING_CONTEXT, requestPayload,  PayloadKey.TRANSIENT);
      } // end of try-catch


   }
}
