/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aspects.remoting;

import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.aop.proxy.ClassProxyFactory;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
/**
 * Checks to see if this object is local in VM
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class ForwardingInterceptor implements org.jboss.aop.advice.Interceptor, ClusterConstants
{
   private final Object obj;
   public ForwardingInterceptor(Object obj)
   {
      this.obj = obj;
   }

   public String getName() { return "ForwardingInterceptor"; }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      if (invocation instanceof MethodInvocation)
      {
         // For non-advised methods, we can only do public method invocations
         org.jboss.aop.joinpoint.MethodInvocation methodInvocation = (org.jboss.aop.joinpoint.MethodInvocation)invocation;
         long methodHash = methodInvocation.getMethodHash();
         HashMap methodMap = ClassProxyFactory.getMethodMap(obj.getClass());
         Method method = (Method)methodMap.get(new Long(methodHash));
         Object[] args = methodInvocation.getArguments();
         try
         {
            return method.invoke(obj, args);
         }
         catch (InvocationTargetException ex)
         {
            throw ex.getTargetException();
         }
      }
      else
      {
         throw new RuntimeException("field invocations not implemented");
      }
   }
}
