/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aspects.remoting;

import org.jboss.remoting.Client;
import org.jboss.remoting.InvokerLocator;

import java.io.ObjectStreamException;
/**
 * Checks to see if this object is local in VM
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 */
public class InvokeRemoteInterceptor implements org.jboss.aop.advice.Interceptor, java.io.Serializable
{
   public static final InvokeRemoteInterceptor singleton = new InvokeRemoteInterceptor();
   public static final String REMOTING = "REMOTING";
   public static final String INVOKER_LOCATOR = "INVOKER_LOCATOR";
   public static final String SUBSYSTEM = "SUBSYSTEM";

   public String getName() { return "InvokeRemoteInterceptor"; }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      InvokerLocator locator = (InvokerLocator)invocation.getMetaData(REMOTING, INVOKER_LOCATOR);
      if (locator == null)
      {
         throw new RuntimeException("No InvokerLocator supplied.  Can't invoke remotely!");
      }
      String subsystem = (String)invocation.getMetaData(REMOTING, SUBSYSTEM);
      if (subsystem == null) subsystem = "AOP";
      Client client = new Client(locator, subsystem);
      org.jboss.aop.joinpoint.InvocationResponse response = (org.jboss.aop.joinpoint.InvocationResponse)client.invoke(invocation, null);
      invocation.setResponseContextInfo(response.getContextInfo());
      return response.getResponse();
   }

   Object readResolve() throws ObjectStreamException {
      return singleton;
   }
}
