/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aspects.remoting;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aop.joinpoint.InvocationBase;
import org.jboss.aop.metadata.MetaDataResolver;
import org.jboss.aop.metadata.SimpleMetaData;
/**
 * Prepare Metadata to be marshalled across the wire.
 * This interceptor iterates through an Invocation's
 * metadata resolver list to pull out transportable
 * metadata and stuff it into the invocation
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 */
public class MergeMetaDataInterceptor implements org.jboss.aop.advice.Interceptor, java.io.Serializable
{
   public static final MergeMetaDataInterceptor singleton = new MergeMetaDataInterceptor();

   public String getName() { return "MergeMetaDataInterceptor"; }

   protected org.jboss.aop.metadata.SimpleMetaData merge(Invocation invocation, MetaDataResolver resolver, SimpleMetaData merged)
   {
      if (resolver == null) return merged;
      org.jboss.aop.metadata.SimpleMetaData metadata =  resolver.getAllMetaData(invocation);
      if (metadata == null) return merged;
      if (merged == null)
      {
         merged = new org.jboss.aop.metadata.SimpleMetaData();
      }
      merged.mergeIn(metadata);
      return merged;
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      org.jboss.aop.metadata.SimpleMetaData merged = null;
      merged = merge(invocation, ((InvocationBase)invocation).getInstanceResolver(), merged);

      org.jboss.aop.metadata.MetaDataResolver threadMetaData = org.jboss.aop.metadata.ThreadMetaData.instance().getAllMetaData(invocation);
      merged = merge(invocation, threadMetaData, merged);

      if (merged != null)
      {
         // Invocation data should override all merged data
         merged.mergeIn(invocation.getMetaData());

         // replace invocation's metadata with merged
         invocation.setMetaData(merged);
      }
      return invocation.invokeNext();
   }
}
