/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/

package org.jboss.aspects.remoting;

import org.jboss.aop.Dispatcher;
import org.jboss.aop.InstanceAdvisor;
import org.jboss.aop.proxy.ClassProxy;
import org.jboss.aop.proxy.ClassProxyFactory;
import org.jboss.aop.util.PayloadKey;
import org.jboss.remoting.InvokerLocator;
/**
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 */

public class Remoting
{
   public static ClassProxy createRemoteProxy(Object objectId, Class clazz, String uri)
      throws Exception
   {
      return createRemoteProxy(objectId, clazz, new InvokerLocator(uri), null);
   }

   public static ClassProxy createRemoteProxy(Object objectId, Class clazz, String uri, String stackName) throws Exception
   {
      return createRemoteProxy(objectId, clazz, new InvokerLocator(uri), stackName);
   }
   public static ClassProxy createRemoteProxy(Object objectId, Class clazz, InvokerLocator locator)
      throws Exception
   {
      return createRemoteProxy(objectId, clazz, locator, null);
   }

   public static ClassProxy createRemoteProxy(Object objectId, Class clazz, InvokerLocator locator, String stackName)
      throws Exception
   {
      ClassProxy proxy = ClassProxyFactory.newInstance(clazz);
      InstanceAdvisor advisor = proxy._getInstanceAdvisor();
      if (stackName != null)
      {
         advisor.insertInterceptorStack(stackName);
      }
      advisor.insertInterceptor(IsLocalInterceptor.singleton);
      advisor.insertInterceptor(MergeMetaDataInterceptor.singleton);
      advisor.insertInterceptor(InvokeRemoteInterceptor.singleton);
      advisor.getMetaData().addMetaData(InvokeRemoteInterceptor.REMOTING, 
                                        InvokeRemoteInterceptor.INVOKER_LOCATOR,
                                        locator,
                                        PayloadKey.AS_IS);
      advisor.getMetaData().addMetaData(InvokeRemoteInterceptor.REMOTING, 
                                        InvokeRemoteInterceptor.SUBSYSTEM,
                                        "AOP",
                                        PayloadKey.AS_IS);
      
      advisor.getMetaData().addMetaData(Dispatcher.DISPATCHER,
                                        Dispatcher.OID,
                                        objectId,
                                        PayloadKey.AS_IS);
      return proxy;
   }

}
