/***************************************
 *                                     *
 *  JBoss: The OpenSource J2EE WebOS   *
 *                                     *
 *  Distributable under LGPL license.  *
 *  See terms of license at gnu.org.   *
 *                                     *
 ***************************************/
package org.jboss.aspects.remoting;


import org.jboss.aop.Dispatcher;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.ServerInterceptor;
import org.jboss.remoting.ServerInterceptorChain;

/**
 * RemotingToAOPServerInterceptor is a ServerInterceptor that will forward requests to the
 * aop Dispatcher
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
 * @version $Revision: 1.2 $
 *
 * @jmx.mbean description="Server side remoting interceptor that hands off invocations to the aop dispatcher."
 * @jboss.xmbean
 */
public class RemotingToAOPServerInterceptor implements ServerInterceptor
{


   // Implementation of org.jboss.remoting.ServerInterceptor

   public Object invoke(ServerInterceptorChain.InterceptorIterator interceptorIterator, InvocationRequest invocation) throws Throwable
   {
      org.jboss.aop.joinpoint.Invocation inv =(org.jboss.aop.joinpoint.Invocation)invocation.getParameter();
      return Dispatcher.singleton.invoke(inv);
   }

   /**
    * The <code>getInstance</code> method
    *
    * @return a <code>ServerInterceptor</code> value
    *
    * @jmx.managed-attribute description="returns this instance"
    *      access="read-only"
    */
   public ServerInterceptor getInstance()
   {
      return this;
   }


}
