/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

package org.jboss.aspects.remoting;

import org.jboss.ha.framework.server.HATarget;

import java.util.ArrayList;
/**
 * Checks to see if this object is local in VM
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.2 $
 */
public class ReplicantsManagerInterceptor implements org.jboss.aop.advice.Interceptor, ClusterConstants
{
   private final HATarget target;
   private final String family;
   private final String name;
   public ReplicantsManagerInterceptor(String family, HATarget target)
   {
      this.family = family;
      this.target = target;
      name = family + "/ReplicantsManagerInterceptor";
   }

   public String getName() { return name; }

   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      Object response = invocation.invokeNext();
      String clientFamily = (String)invocation.getMetaData(CLUSTERED_REMOTING, CLUSTER_FAMILY);

      // Stick to family name
      // This is for multiple protocols.
      if (!family.equals(clientFamily)) 
      {
         return response;
      }

      Long clientViewId = (Long)invocation.getMetaData(CLUSTERED_REMOTING, CLUSTER_VIEW_ID);
      if (clientViewId == null)
      {
         // Maybe we're being invoked upon by a non-clustered proxy
         return response;
      }
      if (clientViewId.longValue() != target.getCurrentViewId())
      {
         invocation.addResponseAttachment("replicants", new ArrayList(target.getReplicants()));
         invocation.addResponseAttachment("viewId", new Long(target.getCurrentViewId()));
      }
      return response;
   }
}
