/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.security;

import org.jboss.security.SecurityAssociation;

import java.io.ObjectStreamException;
import java.security.Principal;

/**
 * The AuthenticationInterceptor authenticates the caller.
 *
 * @author <a href="bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3.2.1 $
 */
public final class SecurityClientInterceptor implements org.jboss.aop.advice.Interceptor, java.io.Serializable
{
   public static final SecurityClientInterceptor singleton = new SecurityClientInterceptor();

   public String getName()
   {
      return "SecurityClientInterceptor";
   }

   /**
    * Authenticates the caller using the principal and credentials in the
    * Infocation if thre is a security manager and an invcocation method.
    */
   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      // Get Principal and credentials 
      Principal principal = SecurityAssociation.getPrincipal();
      if (principal != null) invocation.getMetaData().addMetaData("security", "principal", principal);

      Object credential = SecurityAssociation.getCredential();
      if (credential != null) invocation.getMetaData().addMetaData("security", "credential", credential);

      return invocation.invokeNext();
   }

   Object readResolve() throws ObjectStreamException
   {
      return singleton;
   }
}
