/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.security;

import java.util.Set;

public class SecurityConstructorConfig extends org.jboss.aop.metadata.ConstructorConfig
{
   /**
    * The unchecked element specifies that a constructor is not checked for
    * authorization by the container prior to invocation of the constructor.
    * Used in: constructor-permission
    */
   private boolean unchecked = false;
   /**
    * The exclude-list element defines a set of constructors which the Assembler
    * marks to be uncallable. It contains one or more constructors. If the constructor
    * permission relation contains constructors that are in the exclude list, the
    * Deployer should consider those constructors to be uncallable.
    */
   private boolean excluded = false;
   private Set permissions;

   // Static --------------------------------------------------------

   // Constructors --------------------------------------------------
   public SecurityConstructorConfig()
   {
   }

   // Public --------------------------------------------------------

   public boolean isUnchecked()
   {
      return unchecked;
   }

   public boolean isExcluded()
   {
      return excluded;
   }

   public Set getRoles()
   {
      return permissions;
   }

   public void setRoles(Set perm)
   {
      permissions = perm;
   }

   public void setUnchecked()
   {
      unchecked = true;
   }

   public void setExcluded()
   {
      excluded = true;
   }
}
