/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.tx;

import org.jboss.aop.advice.Interceptor;
import org.jboss.aop.joinpoint.Invocation;
import org.jboss.tm.TransactionPropagationContextFactory;
import org.jboss.tm.TransactionPropagationContextUtil;

import java.io.ObjectStreamException;

/**
 * Comment
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.3 $
 */
public class ClientTxPropagationInterceptor implements Interceptor, java.io.Serializable
{
   public static String TRANSACTION_PROPAGATION_CONTEXT = "TransactionPropagationContext";

   public static final ClientTxPropagationInterceptor singleton = new ClientTxPropagationInterceptor();

   public String getName()
   {
      return "ClientTxPropagationInterceptor";
   }

   public Object invoke(Invocation invocation) throws Throwable
   {
      TransactionPropagationContextFactory tpcFactory = TransactionPropagationContextUtil.getTPCFactoryClientSide();
      if (tpcFactory != null)
      {
         Object tpc = tpcFactory.getTransactionPropagationContext();
         if (tpc != null)
         {
            invocation.getMetaData().addMetaData(TRANSACTION_PROPAGATION_CONTEXT, TRANSACTION_PROPAGATION_CONTEXT, tpc);
         }
      }
      return invocation.invokeNext();
   }

   Object readResolve() throws ObjectStreamException {
      return singleton;
   }
}
