/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.tx;

import org.jboss.aop.annotation.AnnotationElement;
import org.jboss.aop.joinpoint.ConstructorInvocation;
import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aspects.Injected;
import org.jboss.tm.TxManager;

import javax.transaction.TransactionManager;
import java.lang.reflect.Method;

/**
 * This aspect should be scoped PER_JOINPOINT
 * It allows a field to be like a ThreadLocal
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.1 $
 */
public class TransactionManagerInjector
{
   public Object access(FieldReadInvocation invocation) throws Throwable
   {
      return TxManager.getInstance();
   }

   public Object access(FieldWriteInvocation invocation) throws Throwable
   {
      throw new RuntimeException("It is illegal to set an injected TransactionManager field.");
   }

   public Object allocation(ConstructorInvocation invocation) throws Throwable
   {
      Object obj = invocation.invokeNext();

      try
      {
         Object[] arg = {TxManager.getInstance()};
         Method[] methods = obj.getClass().getMethods();
         for (int i = 0; i < methods.length; i++)
         {
            if (methods[i].getParameterTypes().length == 1)
            {
               if (methods[i].getParameterTypes()[0].equals(TransactionManager.class))
               {
                  if (AnnotationElement.isAnyAnnotationPresent(methods[i], Injected.class))
                  {
                     methods[i].invoke(obj, arg);
                  }
               }
            }
         }
      }
      catch (Exception e)
      {
         throw new RuntimeException(e);  //To change body of catch statement use Options | File Templates.
      }
      return obj;
   }
}
