/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.txlock;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.logging.Logger;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 *  This interceptor handles chooses an object to invoke
 *  on based on the transaction
 *
 *  @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 *  @version $Revision: 1.3 $
 */
public class TxLockInterceptor implements org.jboss.aop.advice.Interceptor
{
   /**
    * Logging instance
    */
   protected Logger log = Logger.getLogger(this.getClass());

   private final TransactionManager tm;
   private final QueuedTxLock lock;

   public TxLockInterceptor(TransactionManager tm, QueuedTxLock lock)
   {
      this.tm = tm;
      this.lock = lock;
   }

   public String getName()
   {
      return "TxLockInterceptor";
   }

   /**
    *
    */
   public Object invoke(Invocation invocation) throws Throwable
   {
      Transaction tx = tm.getTransaction();
      if (tx == null) return invocation.invokeNext();

      else
      {
      }


      lock.schedule(tx, invocation);
      try
      {
         return invocation.invokeNext();
      }
      finally
      {
         lock.endInvocation(tx);
      }
   }


}
