/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.versioned;

import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.logging.Logger;

import java.lang.reflect.Method;
import java.util.HashMap;



/**
 *
 *  @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 *  @version $Revision: 1.2 $
 */
public class CollectionStateChangeInterceptor implements org.jboss.aop.advice.Interceptor
{

   /**
    * Logging instance
    */
   private static Logger log = Logger.getLogger(CollectionStateChangeInterceptor.class);
   protected CollectionStateManager manager;

   public CollectionStateChangeInterceptor(CollectionStateManager manager)
   {
      this.manager = manager;
   }

   public String getName() { return "CollectionChangeInterceptor"; }


   /**
    *
    */
   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      if (invocation instanceof MethodInvocation)
      {
         org.jboss.aop.joinpoint.MethodInvocation methodInvocation = (org.jboss.aop.joinpoint.MethodInvocation)invocation;
         long methodHash = methodInvocation.getMethodHash();
         Method managerMethod = manager.isManagerMethod(methodHash);
         if (managerMethod != null)
         {
            Object[] args = methodInvocation.getArguments();
            return managerMethod.invoke(manager, args);
         }
         Object target = methodInvocation.getTargetObject();
         Method method = methodInvocation.getMethod();
         if (method == null)
         {
            HashMap methodMap = manager.getMethodMap();
            method = (Method)methodMap.get(new Long(methodHash));
         }
         return method.invoke(target, methodInvocation.getArguments());
      }
      return invocation.invokeNext();
   }

}
