/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.versioned;

import EDU.oswego.cs.dl.util.concurrent.ReadWriteLock;
import EDU.oswego.cs.dl.util.concurrent.WriterPreferenceReadWriteLock;
import org.jboss.aop.InstanceAdvised;
import org.jboss.aop.util.PayloadKey;
import org.jboss.logging.Logger;
import org.jboss.util.id.GUID;

/**
 *
 *  @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 *  @version $Revision: 1.1 $
 */
public abstract class StateManager implements java.io.Externalizable
{
   public static final String STATE_MANAGER = "STATE_MANAGER";
   protected static Logger log = Logger.getLogger(StateManager.class);


   public static StateManager getStateManager(InstanceAdvised obj)
   {
      return (StateManager)obj._getInstanceAdvisor().getMetaData().getMetaData(STATE_MANAGER, STATE_MANAGER);
   }

   public static void setStateManager(InstanceAdvised obj, StateManager manager)
   {
      obj._getInstanceAdvisor().getMetaData().addMetaData(STATE_MANAGER, STATE_MANAGER, manager, PayloadKey.TRANSIENT);
   }

   protected GUID guid;
   protected long timeout;
   transient protected ReadWriteLock lock = new WriterPreferenceReadWriteLock();

   public StateManager() {}

   public StateManager(GUID guid, long timeout)
   {
      this.guid = guid;
      this.timeout = timeout;
   }

   public ReadWriteLock getLock() { return lock; }
   public GUID getGUID() { return guid; }

   public void acquireWriteLock()
   {
      try
      {
         if (!lock.writeLock().attempt(timeout))
         {
            throw new LockAttemptFailure("failed to acquire write lock: " + guid);
         }
         log.trace("writelock acquired for: " + guid);
      }
      catch (InterruptedException ignored)
      {
         throw new LockAttemptFailure("failed to acquire write lock: " + guid);
      }
   }

   public void acquireReadLock()
   {
      try
      {
         if (!lock.readLock().attempt(timeout))
         {
            throw new LockAttemptFailure("failed to acquire read lock: " + guid);
         }
         log.trace("readlock acquired for: " + guid);
      }
      catch (InterruptedException ignored)
      {
         throw new LockAttemptFailure("failed to acquire read lock: " + guid);
      }
   }

   public void releaseReadLock()
   {
      lock.readLock().release();
      log.trace("readlock released for: " + guid);
}
   public void releaseWriteLock()
   {
      lock.writeLock().release();
      log.trace("writelock released for: " + guid);
   }

   public void writeExternal(java.io.ObjectOutput out)
      throws java.io.IOException
   {
      out.writeObject(guid);
      out.writeLong(timeout);
   }

   public void readExternal(java.io.ObjectInput in)
      throws java.io.IOException, ClassNotFoundException
   {
      this.guid = (GUID)in.readObject();
      this.timeout = in.readLong();
      this.lock = new WriterPreferenceReadWriteLock();
   }


}
