/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.versioned;
import org.jboss.util.LRUCachePolicy;

/**
 * This is a LRU cache.  The TxCache itself is not transactional
 * but any accesses to objects within the cache ARE transactional.
 */
public class TxCache extends LRUCachePolicy
{
   protected long lockTimeout;
   protected LocalSynchronizationManager synchManager;
   protected DistributedVersionManager versionManager;
   public TxCache(int maxSize, long lockTimeout)
   {
      super(2, maxSize);
      this.lockTimeout = lockTimeout;
      synchManager = new LocalSynchronizationManager(null);
      versionManager = new DistributedVersionManager(lockTimeout, synchManager);
      this.create();
      this.start();
   }

   public void insert(Object key, Object obj)
   {
      try
      {
         Object versioned = versionManager.makeVersioned(obj);
         super.insert(key, versioned);
      }
      catch (Exception ex)
      {
         throw new RuntimeException(ex);
      }
   }
}
