/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.aspects.versioned;

import org.jboss.aop.joinpoint.FieldReadInvocation;
import org.jboss.aop.joinpoint.FieldWriteInvocation;
import org.jboss.aop.joinpoint.MethodInvocation;
import org.jboss.logging.Logger;

import javax.transaction.Transaction;
import javax.transaction.TransactionManager;

/**
 *  This interceptor handles chooses an object to invoke
 *  on based on the transaction
 *
 *  @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 *  @version $Revision: 1.2 $
 */
public class VersionedObjectInterceptor implements org.jboss.aop.advice.Interceptor
{
   /** 
    * Logging instance 
    */
   protected Logger log = Logger.getLogger(this.getClass());

   private TransactionManager tm;
   public VersionedObjectInterceptor(TransactionManager tm)
   {
      this.tm = tm;
   }

   public String getName() { return "VersionedObjectInterceptor"; }

   /**
    *
    */
   public Object invoke(org.jboss.aop.joinpoint.Invocation invocation) throws Throwable
   {
      Transaction tx = tm.getTransaction();
      VersionedObject manager = (VersionedObject)invocation.getMetaData(Versioned.VERSIONED, Versioned.VERSIONED_OBJECT);

      Object version = manager.getVersion(tx);
      if (version != null)
      {
         invocation.setTargetObject(version);
         return invocation.invokeNext();
      }

      boolean isReadonly = false;
      if (invocation instanceof MethodInvocation)
      {
         String readonly = (String)invocation.getMetaData(Versioned.VERSIONED, Versioned.READONLY);
         if (readonly != null)
         {
            isReadonly = Boolean.getBoolean(readonly.toLowerCase());
         }
      }
      else if (invocation instanceof FieldReadInvocation)
      {
         isReadonly = true;
      }
      else if (invocation instanceof FieldWriteInvocation)
      {
         isReadonly = false;
      }
      if (isReadonly) return invocation.invokeNext();

      // Ok, we're in a tx, we're not readonly, there is no previous version, so create another
      
      version = manager.createVersion(tx);
      invocation.setTargetObject(version);
      return invocation.invokeNext();
   }

}
