/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.messagedriven.support;

import java.util.HashMap;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.management.ObjectName;

import org.jboss.mx.util.ObjectNameFactory;

/**
 * A Basic Test 
 *
 * @author <a href="mailto:adrian@jboss.com>Adrian Brock</a>
 * @version <tt>$Revision: 1.4</tt>
 */
public class BasicTest
{
   protected BasicMessageDrivenUnitTest unitTest;
   protected String deployment = "invalid";
   protected ObjectName jmxDestination = ObjectNameFactory.create("does:not=exist"); 
   protected ObjectName dlqJMXDestination = ObjectNameFactory.create("jboss.mq.destination:service=Queue,name=DLQ");
   protected String connectionFactoryJNDI = "ConnectionFactory";
   protected Destination destination;
   
   protected Connection connection;
   protected Session session;
   protected HashMap producers = new HashMap();
   
   public String getDeployment()
   {
      return deployment;
   }
   
   public ObjectName getJMXDestination()
   {
      return jmxDestination;
   }
   
   public ObjectName getDLQJMXDestination()
   {
      return dlqJMXDestination;
   }
   
   public Destination getDestination() throws Exception
   {
      if (destination != null)
         return destination;
      String jndiName = (String) unitTest.getAttribute(getJMXDestination(), "JNDIName");
      destination = (Destination) unitTest.lookup(jndiName, Destination.class);
      return destination;
   }
   
   public MessageProducer getMessageProducer() throws Exception
   {
      return getMessageProducer(getDestination());
   }
   
   public MessageProducer getMessageProducer(Destination destination) throws Exception
   {
      MessageProducer producer = (MessageProducer) producers.get(destination);
      if (producer == null)
         producer = getSession().createProducer(destination);
      return producer;
   }

   public Session getSession() throws Exception
   {
      if (session != null)
         return session;
      
      return getConnection().createSession(false, Session.AUTO_ACKNOWLEDGE);
   }
   
   public Connection getConnection() throws Exception
   {
      if (connection != null)
         return connection;
      
      ConnectionFactory factory = (ConnectionFactory) unitTest.lookup(connectionFactoryJNDI, ConnectionFactory.class);
      connection = factory.createConnection();
      return connection;
   }
   
   public Message getTestMessage() throws Exception
   {
      return getSession().createMessage();
   }
   
   public void tidyup() throws Exception
   {
      if (connection != null)
         connection.close();
   }
   
   public void run(BasicMessageDrivenUnitTest unitTest) throws Exception
   {
      this.unitTest = unitTest;
      try
      {
         Operation[] ops = unitTest.getOperations();
         for (int i = 0; i < ops.length; ++i)
            ops[i].run();
      }
      finally
      {
         tidyup();
      }
   }
   
   protected Operation[] getOperations()
   {
      return new Operation[0];
   }
}
