package org.jboss.test.security.test;

import javax.security.jacc.WebResourcePermission;

import junit.framework.TestCase;

/** Tests of the JAAC WebResourcePermission
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1.4.2 $
 */
public class WebResourcePermissionUnitTestCase
   extends TestCase
{

   public WebResourcePermissionUnitTestCase(String name)
   {
      super(name);
   }

   public void testCtor2() throws Exception
   {
      String nullActions = null;
      WebResourcePermission p = new WebResourcePermission("/", nullActions);
      String actions = p.getActions();
      assertTrue("actions("+actions+") == null", actions == null);

      String[] emtpy = {};
      p = new WebResourcePermission("/", emtpy);
      actions = p.getActions();
      assertTrue("actions("+actions+") == null", actions == null);

      p = new WebResourcePermission("/", "POST");
      actions = p.getActions();
      assertTrue("actions("+actions+") == POST", actions.equals("POST"));

      p = new WebResourcePermission("/", "GET,POST,PUT,DELETE,HEAD,OPTIONS,TRACE");
      actions = p.getActions();
      assertTrue("actions("+actions+") == null", actions == null);

      p = new WebResourcePermission("/", "TRACE,GET,DELETE");
      actions = p.getActions();
      assertTrue("actions("+actions+") == DELETE,GET,TRACE",
         actions.equals("DELETE,GET,TRACE"));
   }

   public void testImpliesPermission() throws Exception
   {
      String nullActions = null;
      WebResourcePermission p0 = new WebResourcePermission("/", nullActions);
      WebResourcePermission p1 = new WebResourcePermission("/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/", "");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      String[] emtpy = {};
      p0 = new WebResourcePermission("/", emtpy);
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/*", nullActions);
      p1 = new WebResourcePermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/*", "GET");
      p1 = new WebResourcePermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/any/*", "GET");
      p1 = new WebResourcePermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p1 = new WebResourcePermission("/any/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/any/more/*", "GET");
      p1 = new WebResourcePermission("/any/more/andsome", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("*.jsp", "POST,GET");
      p1 = new WebResourcePermission("/snoop.jsp", "GET,POST");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("*.jsp", "POST,GET,TRACE");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/snoop.jsp", "POST,GET,TRACE");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebResourcePermission("/:/secured.jsp:/unchecked.jsp:/excluded.jsp:/sslprotected.jsp", "POST,GET");
      p1 = new WebResourcePermission("/:/secured.jsp:/excluded.jsp:/sslprotected.jsp:/unchecked.jsp", "GET,POST");
      assertTrue("p0.implies(p1)", p0.implies(p1));
      
      p0 = new WebResourcePermission("/restricted/*", "DELETE,GET,HEAD,POST,PUT");
      p1 = new WebResourcePermission("/restricted/SecureServlet", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));
   }

   public void testNotImpliesPermission() throws Exception
   {
      String nullActions = null;
      WebResourcePermission p0 = new WebResourcePermission("/", "GET");
      WebResourcePermission p1 = new WebResourcePermission("/", nullActions);
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebResourcePermission("/", "POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebResourcePermission("/", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("/any/*", "GET");
      p1 = new WebResourcePermission("/anymore", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebResourcePermission("/anyx", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebResourcePermission("/any/more", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("/*", "GET");
      p1 = new WebResourcePermission("/anyx", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("*.jsp", "GET");
      p1 = new WebResourcePermission("/", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("*.jsp", "GET");
      p1 = new WebResourcePermission("/*", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("*.jsp", "GET");
      p1 = new WebResourcePermission("/jsp", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebResourcePermission("*.jsp", "GET");
      p1 = new WebResourcePermission("/snoop,jsp", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);
   }
}
