package org.jboss.test.security.test;

import javax.security.jacc.WebUserDataPermission;

import junit.framework.TestCase;

/** Tests of the JAAC WebUserDataPermission
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1 $
 */
public class WebUserDataPermissionUnitTestCase
   extends TestCase
{

   public WebUserDataPermissionUnitTestCase(String name)
   {
      super(name);
   }

   public void testCtor2() throws Exception
   {
      String nullActions = null;
      WebUserDataPermission p = new WebUserDataPermission("/", nullActions);
      String actions = p.getActions();
      assertTrue("actions("+actions+") == null", actions == null);
      
      p = new WebUserDataPermission("/", "POST");
      actions = p.getActions();
      assertTrue("actions("+actions+") == POST", actions.equals("POST"));

      p = new WebUserDataPermission("/", "GET,POST,PUT,DELETE,HEAD,OPTIONS,TRACE");
      actions = p.getActions();
      assertTrue("actions("+actions+") == null", actions == null);

      p = new WebUserDataPermission("/", "TRACE,GET,DELETE");
      actions = p.getActions();
      assertTrue("actions("+actions+") == DELETE,GET,TRACE",
         actions.equals("DELETE,GET,TRACE"));

      p = new WebUserDataPermission("/", "TRACE,GET,DELETE:NONE");
      actions = p.getActions();
      assertTrue("actions("+actions+") == DELETE,GET,TRACE",
         actions.equals("DELETE,GET,TRACE"));

      p = new WebUserDataPermission("/", "TRACE,GET,DELETE:CONFIDENTIAL");
      actions = p.getActions();
      assertTrue("actions("+actions+") == DELETE,GET,TRACE:CONFIDENTIAL",
         actions.equals("DELETE,GET,TRACE:CONFIDENTIAL"));
   }

   public void testImpliesPermission() throws Exception
   {
      String nullActions = null;
      WebUserDataPermission p0 = new WebUserDataPermission("/", nullActions);
      WebUserDataPermission p1 = new WebUserDataPermission("/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/", "");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/*", nullActions);
      p1 = new WebUserDataPermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/*", "GET");
      p1 = new WebUserDataPermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/any/*", "GET");
      p1 = new WebUserDataPermission("/any", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p1 = new WebUserDataPermission("/any/", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/any/more/*", "GET");
      p1 = new WebUserDataPermission("/any/more/andsome", "GET");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("*.jsp", "POST,GET");
      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST:NONE");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("*.jsp", "POST,GET,TRACE");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/snoop.jsp", "POST,GET,TRACE");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("/:/secured.jsp:/unchecked.jsp:/excluded.jsp:/sslprotected.jsp", "POST,GET");
      p1 = new WebUserDataPermission("/:/secured.jsp:/excluded.jsp:/sslprotected.jsp:/unchecked.jsp", "GET,POST");
      assertTrue("p0.implies(p1)", p0.implies(p1));
      
      p0 = new WebUserDataPermission("*.jsp", "POST,GET,TRACE:NONE");
      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST");
      assertTrue("p0.implies(p1)", p0.implies(p1));

      p0 = new WebUserDataPermission("*.jsp", "POST,GET,TRACE:CONFIDENTIAL");
      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST:CONFIDENTIAL");
      assertTrue("p0.implies(p1)", p0.implies(p1));
   }

   public void testNotImpliesPermission() throws Exception
   {
      String nullActions = null;
      WebUserDataPermission p0 = new WebUserDataPermission("/", "GET");
      WebUserDataPermission p1 = new WebUserDataPermission("/", nullActions);
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebUserDataPermission("/", "POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebUserDataPermission("/", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("/any/*", "GET");
      p1 = new WebUserDataPermission("/anymore", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebUserDataPermission("/anyx", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p1 = new WebUserDataPermission("/any/more", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("/*", "GET");
      p1 = new WebUserDataPermission("/anyx", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "GET");
      p1 = new WebUserDataPermission("/", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "GET");
      p1 = new WebUserDataPermission("/*", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "GET");
      p1 = new WebUserDataPermission("/jsp", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "GET");
      p1 = new WebUserDataPermission("/snoop,jsp", "GET");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "POST,GET,TRACE:CONFIDENTIAL");
      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);

      p0 = new WebUserDataPermission("*.jsp", "POST,GET,TRACE:CONFIDENTIAL");
      p1 = new WebUserDataPermission("/snoop.jsp", "GET,POST:INTEGRAL");
      assertTrue("! p0.implies(p1)", p0.implies(p1) == false);
   }
}
