/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.webservice.exception;

// $Id: ExceptionTestCase.java,v 1.3.2.4 2004/12/01 15:41:55 tdiesler Exp $

import junit.framework.Test;
import org.jboss.test.JBossTestCase;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Properties;

/**
 * Test user exception propagation.
 *
 * @author Thomas.Diesler@jboss.org
 * @since 23-Sep-2004
 */
public class ExceptionTestCase extends JBossTestCase
{
   public ExceptionTestCase(String name)
   {
      super(name);
   }

   /** Deploy the test archives */
   public static Test suite() throws Exception
   {
      return getDeploySetup(ExceptionTestCase.class, "ws4ee-exception.jar, ws4ee-exception-client.jar");
   }

   /** Test simple exception propagation */
   public void testException() throws Exception
   {
      InitialContext iniCtx = getInitialContext();
      ExceptionService service = (ExceptionService)iniCtx.lookup("java:comp/env/service/ExceptionService");
      ExceptionServiceInterface port = service.getPort();
      try
      {
         port.throwException();
         fail("Should have failed with UserException");
      }
      catch (UserException usrex)
      {
         // do nothing
      }
      catch (Exception e)
      {
         fail("Unexpected Exception: " + e);
      }
   }

   /** Test exception with message */
   public void testExceptionWithMessage() throws Exception
   {
      InitialContext iniCtx = getInitialContext();
      ExceptionService service = (ExceptionService)iniCtx.lookup("java:comp/env/service/ExceptionService");
      ExceptionServiceInterface port = service.getPort();

      String message = "Don't worry it's just a test";
      try
      {
         port.throwExceptionWithMessage(message);
         fail("Should have failed with UserException");
      }
      catch (UserMessageException usrex)
      {
         assertEquals(message, usrex.getMessage());
      }
      catch (Exception e)
      {
         fail("Unexpected Exception: " + e);
      }
   }

   /** Test a complex user exception */
   public void testComplexUserException() throws Exception
   {
      InitialContext iniCtx = getInitialContext();
      ExceptionService service = (ExceptionService)iniCtx.lookup("java:comp/env/service/ExceptionService");
      ExceptionServiceInterface port = service.getPort();

      String message = "Don't worry it's just a test";
      try
      {
         port.throwComplexUserException(message, 200);
         fail("Should have failed with UserException");
      }
      catch (ComplexUserException usrex)
      {
         assertEquals(message, usrex.getMessage());
         assertEquals(200, usrex.getErrorCode());
      }
      catch (Exception e)
      {
         fail("Unexpected Exception: " + e);
      }
   }

   /**
    * Get the client's env context, see tracker [840598] for details
    */
   protected InitialContext getInitialContext() throws NamingException
   {
      Properties env = new Properties();
      env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
      env.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming.client");
      env.setProperty(Context.PROVIDER_URL, "jnp://" + getServerHost() + ":1099");
      env.setProperty("j2ee.clientName", "ws4ee-client");
      return new InitialContext(env);
   }
}
