/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.test.webservice.ws4eesimple;

// $Id: ClientStubTestCase.java,v 1.1.2.2 2004/12/01 15:41:58 tdiesler Exp $

import junit.framework.Test;
import org.jboss.test.JBossTestCase;
import org.jboss.webservice.client.Stub;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.rpc.Service;
import java.util.Properties;

/**
 * Tests the JBossWS client stub
 *
 * @author Thomas.Diesler@jboss.org
 * @since 09-Nov-2004
 */
public class ClientStubTestCase extends JBossTestCase
{

   /**
    * Construct the test case with a given name
    */
   public ClientStubTestCase(String name)
   {
      super(name);
   }

   /**
    * deploy the test archives
    */
   public static Test suite() throws Exception
   {
      return getDeploySetup(ClientStubTestCase.class, "ws4ee-simple.jar, ws4ee-simple-client.ear");
   }

   /**
    * Test client application access
    */
   public void testClientStub() throws Exception
   {
      Context iniCtx = getInitialContext();
      Service service = (Service)iniCtx.lookup("java:comp/env/service/HelloWsService");
      HelloWs port = (HelloWs)service.getPort(HelloWs.class);

      String res = port.sayHello("Hello");
      assertEquals("'Hello' to you too!", res);

      Stub stub = (org.jboss.webservice.client.Stub)port;
      stub.setUsername("kermit");
      assertEquals("kermit", stub.getUsername());

      stub.setPassword("thefrog");
      assertEquals("thefrog", stub.getPassword());

      assertTrue(stub.getSessionMaintain() == null || stub.getSessionMaintain().booleanValue() == false);
      stub.setSessionMaintain(new Boolean(true));
      assertTrue(stub.getSessionMaintain().booleanValue());

      assertEquals("http://" + getServerHost() + ":8080/ws4ee-simple/HelloWsService", stub.getEndpointAddress());
      stub.setEndpointAddress("http://" + getServerHost() + ":8080/ws4ee-simple/DummyService");
      assertEquals("http://" + getServerHost() + ":8080/ws4ee-simple/DummyService", stub.getEndpointAddress());

      stub.setTimeout(new Integer(100));
      assertEquals(new Integer(100), stub.getTimeout());

      stub._setProperty(Stub.CLIENT_TIMEOUT_PROPERTY, new Integer(200));
      assertEquals(new Integer(200), stub._getProperty(Stub.CLIENT_TIMEOUT_PROPERTY));
   }

   /**
    * Get the client's env context, see tracker [840598] for details
    */
   protected InitialContext getInitialContext() throws NamingException
   {
      Properties env = new Properties();
      env.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
      env.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming.client");
      env.setProperty(Context.PROVIDER_URL, "jnp://" + getServerHost() + ":1099");
      env.setProperty("j2ee.clientName", "ws4ee-client");
      return new InitialContext(env);
   }
}
