/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.jaxr.ebxmlrr;

import com.sun.ebxml.registry.interfaces.soap.SOAPSender;
import org.jboss.logging.Logger;

import java.net.URL;

/**
 * Class file that does preloading of ebxmlrr data into the
 * database
 * @author <mailto:Anil.Saldhana@jboss.org>Anil Saldhana
 */
public class EbxmlrrDataLoader {

    private Logger log = null;
    private String registryOperator = null;

    private boolean loadNAICS = false;
    private boolean loadUNSPSC = false;

    public EbxmlrrDataLoader(Logger log, String ro, boolean loadNAICS, boolean loadUNSPSC)
    {
        this.log = log;
        this.registryOperator = ro;
    }

    public void load()
    {
        /**We will use the class @see com.sun.ebxml.registry.interfaces.soap.SOAPSender
         * to send the data to the ebxml registry. JBoss has a method callSoapSender
         * that is a mirror of the main method.
         */
        loadSchemes();
        //loadConfig();
        if(loadNAICS) loadNAICS();
        if( loadUNSPSC ) loadUNSPSC();
        loadGeo();
        loadSchemes_cppa();
    }//end method

    public void loadSchemes()
    {
        loadObjectTypes();
        loadAssocTypes();
        loadPhoneTypes();
        loadEmailTypes();
        loadSubjectRoles();
        loadSubjectGroups();
        loadCMS();
        loadErrorHandlingModels();
        loadInvocationModels();
        loadURLTypes();
        loadPostalAddressCodes();

    }
    public void loadConfig()
    {
       makeTheCall("META-INF/SubmitObjectsRequest_Config.xml", "RegistryOperator",
              "META-INF/defaultACP.xml","text/xml,urn:uuid:38aff259-d20c-486f-8907-b4c97d99bdc5" );
    }

    public void loadNAICS()
    {
       makeTheCall("META-INF/SubmitObjectsRequest_NAICS.xml", "Farrukh");
    }
     public void   loadUNSPSC()
     {
       makeTheCall("META-INF/SubmitObjectsRequest_UNSPSC.xml", "Farrukh");
     }
     public void   loadGeo()
     {
       makeTheCall("META-INF/SubmitObjectsRequest_ISO3166.xml", "Farrukh");
     }


     public void   loadSchemes_cppa()
     {
          loadObjectTypes_cppa();
     }

    /**
     *   Loading ObjectTypes taxonomy as RegistryOperator
     */
    public void loadObjectTypes()
    {
        /**<target name="loadObjectTypes" >
         *
         * <echo message="Loading ObjectTypes taxonomy as RegistryOperator"/>
         *   <java classname="com.sun.ebxml.registry.interfaces.soap.SOAPSender" fork="yes"  failonerror="true">
         *  <sysproperty key="http.proxyHost" value="${proxyHost}"/>
         * <sysproperty key="http.proxyPort" value="${proxyPort}"/>
         *          <sysproperty key="org.apache.commons.logging.Log" value="${apacheLog}"/>
         *     <arg line="req=${samples}/SubmitObjectsRequest_ObjectTypeScheme.xml alias=RegistryOperator
         *  localCall=${localCall} url=${ebxmlrr.url.soap} res=response.xml " />
         **/

        makeTheCall("META-INF/SubmitObjectsRequest_ObjectTypeScheme.xml", this.registryOperator);
    }

    /**
     *   Loading AssociationType  taxonomy as RegistryOperator
     */
    public void loadAssocTypes()
    {
        log.debug("Loading Association Type Schema");
        makeTheCall("META-INF/SubmitObjectsRequest_AssociationTypeScheme.xml",this.registryOperator);
    }

    /**
     *   Loading AssociationType  taxonomy as RegistryOperator
     */
    public void loadPhoneTypes()
    {
        makeTheCall("META-INF/SubmitObjectsRequest_PhoneTypeScheme.xml",this.registryOperator);
    }
    /**
     *   Loading AssociationType  taxonomy as RegistryOperator
     */
    public void loadEmailTypes()
    {
        makeTheCall("META-INF/SubmitObjectsRequest_EmailTypeScheme.xml",this.registryOperator);
    }

    //********PRIVATE METHODS *******************
    private void makeTheCall(String fname)
    {
        URL uri = this.getClass().getClassLoader().
                        getResource(fname);

        String filename = "req="+uri.getPath();
      String[] args={getSignedSoapFilePath(),filename,
                     "alias=RegistryOperator", "res=response.xml",
                     "url=http://localhost:8080/ebxmlrr/registry/soap"};

        SOAPSender.callSOAPSender(args);
    }

    private void makeTheCall(String fname, String alias)
    {
         URL uri = this.getClass().getClassLoader().
                        getResource(fname);

        String filename = "req="+uri.getPath();
      String[] args={getSignedSoapFilePath(),filename,
                     "alias="+alias, "res=response.xml",
                     "url=http://localhost:8080/ebxmlrr/registry/soap"};

        SOAPSender.callSOAPSender(args);
    }

    private void makeTheCall(String fname, String alias, String attach, String ext)
    {
         URL uri = this.getClass().getClassLoader().
                        getResource(fname);

        String filename = "req="+uri.getPath();

        //For attach
        URL attachuri =  this.getClass().getClassLoader().
                        getResource(attach);
        String attachfname = "attach="+attachuri.getPath()+","+ext;

        String[] args={getSignedSoapFilePath(),filename, attachfname,
                     "alias="+alias, "res=response.xml",
                     "url=http://localhost:8080/ebxmlrr/registry/soap"};

        SOAPSender.callSOAPSender(args);
    }

    private String getSignedSoapFilePath()
    {
        java.net.URL soapuri = this.getClass().getClassLoader().
                         getResource("META-INF/signedSOAPRequest.xml");
        String soappath = soapuri.getPath();
        return "signedSOAPFile="+soappath;
    }
    //*********************************************

    private void loadSubjectRoles()
    {
        makeTheCall("META-INF/SubmitObjectsRequest_SubjectRoleScheme.xml",this.registryOperator);
    }
        private void loadSubjectGroups()
        {
         makeTheCall("META-INF/SubmitObjectsRequest_SubjectGroupScheme.xml",this.registryOperator);
        }
        private void loadCMS()
        {
            makeTheCall("META-INF/SubmitObjectsRequest_CMSScheme.xml",this.registryOperator);
        }
        private void loadErrorHandlingModels()
        {
          makeTheCall("META-INF/SubmitObjectsRequest_ErrorHandlingModelScheme.xml",this.registryOperator);
        }
        private void loadInvocationModels()
        {
          makeTheCall("META-INF/SubmitObjectsRequest_InvocationModelScheme.xml",this.registryOperator);
        }
        private void loadURLTypes()
        {
           makeTheCall("META-INF/SubmitObjectsRequest_URLTypeScheme.xml",this.registryOperator);
        }
        private void loadPostalAddressCodes()
        {
            makeTheCall("META-INF/SubmitObjectsRequest_PostalAddressCodeScheme.xml",this.registryOperator);
        }

    private void loadObjectTypes_cppa()
     {
       makeTheCall("META-INF/SubmitObjectsRequest_ObjectTypeScheme.xml", "Farrukh");
     }
}
