/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.jaxr.ebxmlrr;

import org.jboss.system.ServiceMBeanSupport;

import javax.sql.DataSource;
import javax.naming.*;
import javax.xml.registry.ConnectionFactory;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.sql.SQLException;
import java.sql.Connection;
import java.sql.Statement;

//ebxmlrr specific files
import com.sun.ebxml.registry.interfaces.soap.SOAPSender;
import com.sun.xml.registry.ebxml.ConnectionFactoryImpl;

/**
 * MBean for the Ebxmlrr open source project integrated as part of the JAXR requirements
 * for J2EE 1.4 compliance
 * @jmx.mbean name="jboss:service=Ebxmlrr"
 *				  extends="org.jboss.system.ServiceMBean"
 *
 * @author <a href="mailto:Anil.Saldhana@jboss.com">Anil Saldhana</a>
 * @version $Revision: 1.2 $
 */
public class EbxmlrrService
 extends ServiceMBeanSupport
   implements EbxmlrrServiceMBean{

   protected boolean dropOnStart = false;
   protected boolean createOnStart = false;
   protected boolean dropOnStop = false;
   protected boolean loadNAICS = false;
   protected boolean loadUNSPSC = false;

   protected DataSource datasource = null;

   protected String datasourceurl = null;

   protected String registryOperator = null;

    protected String bindJaxr = null;
    protected boolean shouldBindJaxr = true;

   protected Connection getConnection() throws SQLException, NamingException{
       try{
            if(datasource == null ){
                InitialContext ctx = new InitialContext();
                //Object obj = ctx.lookup("java:/DefaultDS");
                Object obj = ctx.lookup(datasourceurl);
                log.debug(obj.getClass().getName());
                    //datasource = (DataSource) ctx.lookup("java:/DefaultDS");
                datasource = (DataSource) obj;
                //datasource = (DataSource) ctx.lookup("jboss.jca:name=DefaultDS,service=DataSourceBinding");
            }//end if
       }catch( Exception e){
           log.error(e);
       }
       return datasource.getConnection();
   }

   protected void runDrop() throws SQLException, IOException {
      log.debug("EbxmlrrService: Inside runDrop");

      locateAndRunScript("ebxmlrr_drop.ddl"); 
      log.debug("EbxmlrrService: Exit runDrop");
   }

   protected void runCreate()
      throws  SQLException, IOException {
       log.debug("EbxmlrrService: Inside runCreate");
       locateAndRunScript("ebxmlrr_create.ddl");
       locateAndRunScript("ebxmlrr_intrinsic_data.ddl");
       //Now load the other data as SOAP Messages
       EbxmlrrDataLoader ebxmlLoad = new EbxmlrrDataLoader( log, registryOperator,loadNAICS,loadUNSPSC);
       ebxmlLoad.load();
   }

   private void locateAndRunScript( String name)
    throws  SQLException, IOException {
      log.debug("EbxmlrrService: Inside locateScript");
      InputStream input =
         getClass().getClassLoader().getResourceAsStream(
            "META-INF/"+name);
      if (input != null) {
         try {
            runScript(input);
         } finally {
            input.close();
         }
      }
   }

   protected void runScript(InputStream stream)
      throws  SQLException, IOException {
      boolean firstError = true;
      BufferedReader reader = new BufferedReader(new InputStreamReader(stream));

      try {
            Connection connection = this.getConnection();
            if( connection != null )
                log.debug( "HaHa!  Obtained the Connection");
            Statement statement = connection.createStatement();
            try {
               String nextStatement = "";
               String nextLine;
               while ((nextLine = reader.readLine()) != null) {
                   log.debug("Statement Obtained="+nextLine);
                   nextLine = nextLine.trim();
                   if(nextLine.indexOf("--") != -1 ) continue;
                  int semicolon = nextLine.indexOf(";");
                  if (semicolon != -1) {
                     nextStatement += nextLine.substring(0, semicolon);
                     try {
                        log.debug("Statement to execute:"+nextStatement);
                        statement.execute(nextStatement);
                     } catch (SQLException e) {
                        String err="Could not execute a statement of ebxmlrr script::";

                        if (firstError) {
                           log.warn( err+e.getLocalizedMessage()) ;

                           firstError = false;
                        }
                     }
                     nextStatement = nextLine.substring(semicolon + 1);
                  } else {
                     nextStatement += nextLine;
                  }
               }
               if (!nextStatement.equals("")) {
                  try {
                      log.debug("Statement to execute:"+nextStatement);
                     statement.execute(nextStatement);
                  } catch (SQLException e) {
                     log.warn(
                        "Could not execute last statement of a ebxmlrr init script: "+e.getLocalizedMessage());
                  }
               }
            }finally {
               statement.close();
            }
         } catch(NamingException nm){
                log.error("Looks like DataSource was tried with the wrong JNDI name");
          log.error(nm);
            }
       finally {

      }
   }

   /**
    * starts the ebxmlrr service
    */
   public void startService() throws Exception {
     log.debug("EbxmlrrService: Inside startService with dropOnStart="+dropOnStart+" createOnStart="+createOnStart);

       if( shouldBindJaxr){
            //Bind a ConnectionFactory object to JNDI
            InitialContext ctx = new InitialContext();
            ConnectionFactory  factory = (ConnectionFactory)new
                ConnectionFactoryImpl();

            bind(ctx,bindJaxr,factory);
       }

      if (dropOnStart) {
         runDrop();
      }
      if (createOnStart) {
         runCreate();
      }
   }

   /** stop the service */
   public void stopService() throws Exception {
     log.debug("EbxmlrrService: Inside stopService with dropOnStop="+dropOnStop);
      if (dropOnStop) {
         runDrop();
      }
   }


   /**
    * @return boolean
    * @jmx:managed-attribute
    */
   public boolean isDropOnStart() {
      return dropOnStart;
   }

   /**
    * Sets the dropOnStart.
    * @param dropOnStart The dropOnStart to set
    * @jmx:managed-attribute
    */
   public void setDropOnStart(boolean dropOnStart) {
      this.dropOnStart = dropOnStart;
   }

   /**
    * @return boolean
    * @jmx:managed-attribute
    */
   public boolean isDropOnStop() {
      return dropOnStop;
   }

   /**
    * Sets the dropOnStop.
    * @param dropOnStop The dropOnStop to set
    * @jmx:managed-attribute
    */
   public void setDropOnStop(boolean dropOnStop) {
      this.dropOnStop = dropOnStop;
   }

   /**
    * @return boolean
    * @jmx:managed-attribute
    */
   public boolean isCreateOnStart() {
      return createOnStart;
   }

   /**
    * Sets the createOnStart.
    * @param createOnStart The createOnStart to set
    * @jmx:managed-attribute
    */
   public void setCreateOnStart(boolean createOnStart) {
      this.createOnStart = createOnStart;
   }

   /**
    * @return String
    * @jmx:managed-attribute
    */
   public String getDataSource() {
      return datasourceurl;
   }

   /**
    * Sets the Datasource Url.
    * @param ds The datasourceurl to set
    * @jmx:managed-attribute
    */
   public void setDataSourceUrl(String ds) {
      this.datasourceurl = ds;
   }

   /**
    * @return String
    * @jmx:managed-attribute
    */
   public String getRegistryOperator() {
      return registryOperator;
   }

   /**
    * Sets the RegistryOperator.
    * @param ro The datasourceurl to set
    * @jmx:managed-attribute
    */
   public void setRegistryOperator(String ro) {
      this.registryOperator = ro;
   }

   /**
    * @return boolean
    * @jmx:managed-attribute
    */
   public boolean isLoadNAICS() {
      return loadNAICS;
   }

   /**
    * Sets the dropOnStart.
    * @param ln The loadNAICS to set
    * @jmx:managed-attribute
    */
   public void setLoadNAICS(boolean ln) {
      this.loadNAICS = ln;
   }

   /**
    * @return boolean
    * @jmx:managed-attribute
    */
   public boolean isLoadUNSPSC() {
      return loadUNSPSC;
   }

   /**
    * Sets the dropOnStart.
    * @param ln The loadUNSPSC to set
    * @jmx:managed-attribute
    */
   public void setLoadUNSPSC(boolean ln) {
      this.loadUNSPSC = ln;
   }

   /**
    * Sets the JAXR ConnectionFactory.
    * @param str The context to bind the Jaxr factory to set
    * @jmx:managed-attribute
    */
   public void setBindJaxr(String str) {
      this.bindJaxr = str;
   }


   /**
    * gets the JAXR ConnectionFactory.
    * @jmx:managed-attribute
    */
   public boolean getShouldBindJaxr( ) {
      return shouldBindJaxr ;
   }

    /**
    * Sets the JAXR ConnectionFactory.
    * @param str Should a Jaxr Connection Factory bound
    * @jmx:managed-attribute
    */
   public void setShouldBindJaxr(boolean str) {
      this.shouldBindJaxr = str;
   }


   /**
    * gets the JAXR ConnectionFactory.
    * @jmx:managed-attribute
    */
   public String getBindJaxr( ) {
      return bindJaxr ;
   }

   private void bind(Context ctx, String name, Object val)
      throws NamingException
   {
      /**if (log.isDebugEnabled())
         log.debug("attempting to bind " + val + " to " + name);

      // Bind val to name in ctx, and make sure that all
      // intermediate contexts exist
      Name n = ctx.getNameParser("").parse(name);
      while (n.size() > 1)
      {
         String ctxName = n.get(0);
         try
         {
            ctx = (Context)ctx.lookup(ctxName);
         } catch (NameNotFoundException e)
         {
            ctx = ctx.createSubcontext(ctxName);
         }
         n = n.getSuffix(1);
      }

      ctx.bind(n.get(0), val);  **/
      ctx.bind(name,val);
   }
}
