/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package com.underworld.crimeportal.ejb;

// $Id: OrganizationClientBean.java,v 1.7 2004/08/03 15:19:52 tdiesler Exp $

import com.underworld.crimeportal.OrganizationEndpoint;
import com.underworld.crimeportal.OrganizationEndpointService;
import org.apache.axis.AxisFault;
import org.jboss.logging.Logger;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.rpc.Service;
import java.rmi.RemoteException;

/**
 * An example of a SLSB
 *
 * @author Thomas.Diesler@jboss.org
 * @since 26-Apr-2004
 */
public class OrganizationClientBean implements SessionBean
{
   // provide logging
   private static final Logger log = Logger.getLogger(OrganizationClientBean.class);

   private SessionContext ctx;

   /** Get the contact info from the EJB service */
   public String getContactInfoEJB(String organization) throws RemoteException
   {
      Service service = null;
      try
      {
         InitialContext iniCtx = new InitialContext();
         service = (Service)iniCtx.lookup("java:/comp/env/service/OrganizationServiceEJB");
         OrganizationEndpoint endpoint = (OrganizationEndpoint)service.getPort(OrganizationEndpoint.class);
         String info = endpoint.getContactInfo(organization);
         return info;
      }
      catch (NamingException e)
      {
         throw new EJBException(e);
      }
      catch(AxisFault e)
      {
         log.error("Axis fault: " + e.toString(), e.getCause());
         throw e;
      }
      catch (Exception e)
      {
         throw new EJBException("Cannot invoke webservice", e);
      }
   }

   /** Get the contact info from the JSE service */
   public String getContactInfoJSE(String organization) throws RemoteException
   {
      try
      {
         InitialContext iniCtx = new InitialContext();
         OrganizationEndpointService service = (OrganizationEndpointService)iniCtx.lookup("java:comp/env/service/OrganizationServiceJSE");
         OrganizationEndpoint endpoint = (OrganizationEndpoint)service.getOrganizationPort();
         String info = endpoint.getContactInfo(organization);
         return info;
      }
      catch (NamingException e)
      {
         throw new EJBException(e);
      }
      catch(AxisFault e)
      {
         log.error("Axis fault: " + e.toString(), e.getCause());
         throw e;
      }
      catch (Exception e)
      {
         throw new EJBException("Cannot invoke webservice", e);
      }
   }

   // SLSB Lifecycle ***************************************************************************************************

   public void ejbCreate()
   {
      log.debug("ejbCreate");
   }

   public void setSessionContext(SessionContext ctx) throws EJBException, RemoteException
   {
      log.debug("setSessionContext");
      this.ctx = ctx;
   }

   public void ejbRemove() throws EJBException, RemoteException
   {
      log.debug("ejbRemove");
   }

   public void ejbActivate() throws EJBException, RemoteException
   {
      log.debug("ejbActivate");
   }

   public void ejbPassivate() throws EJBException, RemoteException
   {
      log.debug("ejbPassivate");
   }
}
