/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice.complexbean;

// $Id: ComplexBeanTestCase.java,v 1.1.2.2 2004/11/13 18:49:08 tdiesler Exp $

import junit.framework.TestCase;

import javax.naming.InitialContext;

/**
 * Test echo of custom beans
 *
 * @author Thomas.Diesler@jboss.org
 * @since 26-Apr-2004
 */
public class ComplexBeanTestCase extends TestCase
{

   public ComplexBeanTestCase(String name)
   {
      super(name);
   }

   /**
    * Test echo of a custom bean that contains an empty array of another bean.
    */
   public void testOrderNoPositions() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      javax.xml.rpc.Service service = (javax.xml.rpc.Service)iniCtx.lookup("java:comp/env/service/CustomerOrderService");
      CustomerOrderEndpoint endpoint = (CustomerOrderEndpoint)service.getPort(CustomerOrderEndpoint.class);

      CustomerOrder order = new CustomerOrder();
      order.setCustomerID("cuid_1");
      order.setOrderID("oid_1");

      // [todo] when leaving the positions property null
      // the deserializer injects an empty position bean.
      order.setPositions(new CustomerOrderPosition[1]);

      CustomerOrder ret = endpoint.echoCustomerOrder(order);
      assertEquals(order, ret);
   }

   /**
    * Test echo of a custom bean that contains an array of another bean.
    */
   public void testOrderSomePositions() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      javax.xml.rpc.Service service = (javax.xml.rpc.Service)iniCtx.lookup("java:comp/env/service/CustomerOrderService");
      CustomerOrderEndpoint endpoint = (CustomerOrderEndpoint)service.getPort(CustomerOrderEndpoint.class);

      CustomerOrder order = new CustomerOrder();
      order.setCustomerID("cuid_1");
      order.setOrderID("oid_1");
      CustomerOrderPosition[] positions = new CustomerOrderPosition[2];
      positions[0] = new CustomerOrderPosition("pos_1");
      positions[1] = new CustomerOrderPosition("pos_2");
      order.setPositions(positions);

      CustomerOrder ret = endpoint.echoCustomerOrder(order);
      assertEquals(order, ret);
   }

   /**
    * Test echo of a custom bean.
    */
   public void testCustomerArray() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      javax.xml.rpc.Service service = (javax.xml.rpc.Service)iniCtx.lookup("java:comp/env/service/CustomerOrderService");
      CustomerOrderEndpoint endpoint = (CustomerOrderEndpoint)service.getPort(CustomerOrderEndpoint.class);

      Customer cu1 = new Customer();
      cu1.setCustomerID("cuid_1");
      cu1.setCustomerName("JBoss Inc.");
      cu1.setCustomerAddress("MiddleWare Avenue 1");
      Customer cu2 = new Customer();
      cu2.setCustomerID("cuid_2");
      cu2.setCustomerName("Microsoft Corp.");
      cu2.setCustomerAddress("SomeOther Street 11");

      CustomerArray cuArr = new CustomerArray();
      cuArr.setCustomerArray(new Customer[]{cu1, cu2});
      CustomerArray ret = endpoint.echoCustomerArr(cuArr);
      assertEquals(cuArr, ret);
   }
}
