/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package com.underworld.crimeportal;

// $Id: ServerSideJMSTestCase.java,v 1.1.2.1 2004/11/13 18:49:11 tdiesler Exp $

import junit.framework.TestCase;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.transport.jms.JMSConstants;
import org.apache.axis.transport.jms.JMSTransport;

import javax.naming.InitialContext;
import javax.xml.namespace.QName;
import javax.xml.rpc.ParameterMode;
import java.io.InputStream;
import java.util.HashMap;


/**
 * A web service client that connects to a MDB endpoint using
 * the dynamic invokation interface (DII).
 *
 * @author Thomas.Diesler@jboss.org
 * @since 26-Apr-2004
 */
public class ServerSideJMSTestCase extends TestCase
{
   private static final String ENDPOINT_QUEUE = "queue/MessageDrivenEndpointQueue";
   private static final String TARGET_NAMESPACE = "http://com.underworld.crimeportal";

   public ServerSideJMSTestCase(String name)
   {
      super(name);
   }

   /**
    * Construct an Axis call and use JMSTransport to send it to the specified queue
    */
   public void testSOAPMessageToEndpointQueue() throws Exception
   {
      // create the transport
      InitialContext ctx = new InitialContext();
      HashMap ctxEnv = new HashMap(ctx.getEnvironment());
      ctxEnv.put("transport.jms.ConnectionFactoryJNDIName", "UIL2ConnectionFactory");
      JMSTransport transport = new JMSTransport(null, ctxEnv);

      try
      {
         // create a new Call object
         InputStream is = getClass().getClassLoader().getResourceAsStream("jms/client-config.xml");
         Service service = new Service(new FileProvider(is));
         Call call = (Call)service.createCall();

         call.setOperationName(new QName(TARGET_NAMESPACE, "getContactInfo"));
         call.addParameter("organization", XMLType.XSD_STRING, ParameterMode.IN);
         call.setReturnType(XMLType.XSD_STRING);
         call.setProperty(JMSConstants.DESTINATION, ENDPOINT_QUEUE);
         call.setTimeout(new Integer(1000));
         call.setTransport(transport);

         String info = (String)call.invoke(new String[]{"mafia"});
         assertEquals("The 'mafia' boss is currently out of office, please call again.", info);
      }
      finally
      {
         transport.shutdown();
      }
   }
}
