/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package com.underworld.crimeportal;

// $Id: ServerSideSecTestCase.java,v 1.1.2.1 2004/11/13 18:49:13 tdiesler Exp $

import com.underworld.crimeportal.ejb.SecuredRemote;
import com.underworld.crimeportal.ejb.SecuredRemoteHome;
import junit.framework.TestCase;
import org.jboss.security.SecurityAssociation;
import org.jboss.security.SimplePrincipal;

import javax.naming.InitialContext;
import javax.xml.namespace.QName;
import javax.xml.rpc.Service;
import javax.xml.rpc.Stub;
import java.rmi.RemoteException;


/**
 * A web service client that connects to a secured SLSB endpoint using.
 *
 * @author Thomas.Diesler@jboss.org
 * @since 26-Apr-2004
 */
public class ServerSideSecTestCase extends TestCase
{
   public static final String USERNAME = "kermit";
   public static final String PASSWORD = "thefrog";

   public ServerSideSecTestCase(String name)
   {
      super(name);
   }

   protected void tearDown() throws Exception
   {
      // reset the security association after each test
      SecurityAssociation.setPrincipal(null);
      SecurityAssociation.setCredential(null);
   }

   /** Test required principal/credential for this bean
    */
   public void testRoleSecuredSLSB() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      SecuredRemoteHome home = (SecuredRemoteHome)iniCtx.lookup("ejb/RoleSecuredSLSB");

      SecuredRemote bean = null;
      try
      {
         bean = home.create();
         fail("Security exception expected");
      }
      catch (Exception e)
      {
         // all cool, now try again with valid credentials
         SecurityAssociation.setPrincipal(new SimplePrincipal(USERNAME));
         SecurityAssociation.setCredential(PASSWORD);
         bean = home.create();
      }

      String info = bean.getContactInfo("mafia");
      assertEquals("The 'mafia' boss is currently out of office, please call again.", info);
   }

   /** Test that the remote access to this bean is unchecked
    */
   public void testBasicSecuredSLSB() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      SecuredRemoteHome home = (SecuredRemoteHome)iniCtx.lookup("ejb/BasicSecuredSLSB");

      SecuredRemote bean = home.create();
      String info = bean.getContactInfo("mafia");
      assertEquals("The 'mafia' boss is currently out of office, please call again.", info);
   }

   public void testRoleSecuredServiceAccess() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      Service service = (Service)iniCtx.lookup("java:comp/env/service/SecuredSLSB");
      OrganizationEndpoint endpoint = (OrganizationEndpoint)service.getPort(new QName("RoleSecuredPort"), OrganizationEndpoint.class);

      try
      {
         endpoint.getContactInfo("mafia");
         fail("Security exception expected");
      }
      catch (RemoteException ignore)
      {
         // ignore expected exception
      }

      Stub stub = (Stub)endpoint;
      stub._setProperty(Stub.USERNAME_PROPERTY, USERNAME);
      stub._setProperty(Stub.PASSWORD_PROPERTY, PASSWORD);

      String info = endpoint.getContactInfo("mafia");
      assertEquals("The 'mafia' boss is currently out of office, please call again.", info);
   }

   public void testBasicSecuredServiceAccess() throws Exception
   {
      InitialContext iniCtx = new InitialContext();
      Service service = (Service)iniCtx.lookup("java:comp/env/service/SecuredSLSB");
      OrganizationEndpoint endpoint = (OrganizationEndpoint)service.getPort(new QName("BasicSecuredPort"), OrganizationEndpoint.class);

      try
      {
         endpoint.getContactInfo("mafia");
         fail("Security exception expected");
      }
      catch (RemoteException ignore)
      {
         // ignore expected exception
      }

      Stub stub = (Stub)endpoint;
      stub._setProperty(Stub.USERNAME_PROPERTY, USERNAME);
      stub._setProperty(Stub.PASSWORD_PROPERTY, PASSWORD);

      String info = endpoint.getContactInfo("mafia");
      assertEquals("The 'mafia' boss is currently out of office, please call again.", info);
   }
}
