/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.webservice;

// $Id: PortComponentInfo.java,v 1.5.4.6 2004/11/16 11:42:25 tdiesler Exp $

import org.jboss.deployment.DeploymentInfo;
import org.jboss.webservice.deployment.ServiceDescription;
import org.jboss.webservice.metadata.PortComponentMetaData;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

/**
 * This object registered with the AxisService.
 * <p/>
 * The key is [deploymentName]#[port-component-name]
 *
 * @author Thomas.Diesler@jboss.org
 * @since 28-May-2004
 */
public class PortComponentInfo
{
   private DeploymentInfo di;
   private PortComponentMetaData pcMetaData;
   private ServiceDescription serviceDesc;
   private String serviceID;
   private ObjectName objectName;

   public PortComponentInfo(DeploymentInfo di, PortComponentMetaData pcMetaData)
   {
      this.di = di;
      this.pcMetaData = pcMetaData;
      this.serviceID = di.getCanonicalName() + "#" + pcMetaData.getPortComponentName();

      try
      {
         this.objectName = ObjectName.getInstance("jboss.ws4ee:port-component=" + serviceID);
      }
      catch (MalformedObjectNameException e)
      {
         throw new IllegalArgumentException("Invalid serviceID: " + serviceID);
      }
   }

   public String getServiceID()
   {
      return serviceID;
   }

   public ObjectName getObjectName()
   {
      return objectName;
   }

   public DeploymentInfo getDeploymentInfo()
   {
      return di;
   }

   public PortComponentMetaData getPortComponentMetaData()
   {
      return pcMetaData;
   }

   public String getServiceEndpointURL()
   {
      return pcMetaData.getServiceEndpointURL().toExternalForm();
   }

   public void setServiceDesc(ServiceDescription serviceDesc)
   {
      this.serviceDesc = serviceDesc;
   }

   public ServiceDescription getServiceDescription()
   {
      return serviceDesc;
   }

   /**
    * Returns a string representation of the object.
    */
   public String toString()
   {
      return "[id=" + serviceID + ",address=" + pcMetaData.getServiceEndpointURL() + "]";
   }
}

